@@ -3,7 +3,7 @@
  *                      but exported to plug-ins
  *****************************************************************************
  * Copyright (C) 1999-2002 VideoLAN
- * $Id: input_ext-plugins.h,v 1.47 2004/01/26 20:48:09 fenrir Exp $
+ * $Id: input_ext-plugins.h 7209 2004-03-31 20:52:31Z gbazin $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -55,8 +55,8 @@ VLC_EXPORT( int,  input_UnselectES,( input_thread_t *, es_descriptor_t * ) );
 /*****************************************************************************
  * Prototypes from input_dec.c
  *****************************************************************************/
-decoder_t * input_RunDecoder( input_thread_t *, es_descriptor_t * );
-void input_EndDecoder( input_thread_t *, es_descriptor_t * );
+VLC_EXPORT( decoder_t *, input_RunDecoder, ( input_thread_t *, es_descriptor_t * ) );
+VLC_EXPORT( void, input_EndDecoder, ( input_thread_t *, es_descriptor_t * ) );
 
 VLC_EXPORT( void, input_DecodePES, ( decoder_t *, pes_packet_t * ) );
 VLC_EXPORT( void, input_DecodeBlock,( decoder_t *, block_t * ) );
@@ -82,13 +82,6 @@ VLC_EXPORT( int,  input_ClockManageControl, ( input_thread_t *, pgrm_descriptor_
 VLC_EXPORT( void, input_ClockManageRef, ( input_thread_t *, pgrm_descriptor_t *, mtime_t ) );
 VLC_EXPORT( mtime_t, input_ClockGetTS, ( input_thread_t *, pgrm_descriptor_t *, mtime_t ) );
 
-/*****************************************************************************
- * Prototypes from input_info.c
- *****************************************************************************/
-VLC_EXPORT( input_info_category_t *, input_InfoCategory, ( input_thread_t *, char * ) );
-VLC_EXPORT( int, input_AddInfo, ( input_info_category_t *, char *, char *, ... ) );
-int input_DelInfo( input_thread_t * p_input ); /* no need to export this */
-
 /*****************************************************************************
  * Prototypes from input_ext-plugins.h (buffers management)
  *****************************************************************************/
