@@ -1,11 +1,8 @@
 /*****************************************************************************
  * video_output.h : video output thread
- * This module describes the programming interface for video output threads.
- * It includes functions allowing to open a new thread, send pictures to a
- * thread, and destroy a previously opened video output thread.
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: video_output.h,v 1.96 2003/07/14 21:32:58 sigmunau Exp $
+ * $Id: video_output.h 7694 2004-05-16 22:06:34Z gbazin $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
@@ -14,7 +11,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -25,33 +22,41 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
-/*****************************************************************************
- * vout_chroma_t: Chroma conversion function
- *****************************************************************************
+/**
+ * \defgroup video_output Video Output
+ * This module describes the programming interface for video output threads.
+ * It includes functions allowing to open a new thread, send pictures to a
+ * thread, and destroy a previously opened video output thread.
+ * @{
+ */
+
+/**
+ * Chroma conversion function
+ *
  * This is the prototype common to all conversion functions.
- * Parameters:
- *      p_source                        source picture
- *      p_dest                          destination picture
+ * \param p_vout video output thread
+ * \param p_source source picture
+ * \param p_dest destination picture
  * Picture width and source dimensions must be multiples of 16.
- *****************************************************************************/
+ */
 typedef void (vout_chroma_convert_t)( vout_thread_t *,
                                       picture_t *, picture_t * );
 
 typedef struct vout_chroma_t
 {
-    /* conversion functions */
+    /** conversion functions */
     vout_chroma_convert_t *pf_convert;
 
-    /* Private module-dependant data */
+    /** Private module-dependant data */
     chroma_sys_t *      p_sys;                               /* private data */
 
-    /* Plugin used and shortcuts to access its capabilities */
+    /** Plugin used and shortcuts to access its capabilities */
     module_t * p_module;
 
 } vout_chroma_t;
 
 /**
- * \brief video output thread descriptor
+ * Video output thread descriptor
  *
  * Any independant video output device, such as an X11 window or a GGI device,
  * is represented by a video output thread, and described using the following
@@ -68,10 +73,11 @@ struct vout_thread_t
     vlc_mutex_t         change_lock;                 /**< thread change lock */
     vout_sys_t *        p_sys;                     /**< system output method */
     /**@}*/
-    
+
     /** \name Current display properties */
     /**@{*/
-    uint16_t            i_changes;           /**< changes made to the thread */
+    uint16_t            i_changes;          /**< changes made to the thread.
+                                                      \see \ref vout_changes */
     float               f_gamma;                                  /**< gamma */
     vlc_bool_t          b_grayscale;         /**< color or grayscale display */
     vlc_bool_t          b_info;            /**< print additional information */
@@ -79,11 +85,15 @@ struct vout_thread_t
     vlc_bool_t          b_scale;                  /**< allow picture scaling */
     vlc_bool_t          b_fullscreen;         /**< toogle fullscreen display */
     vlc_bool_t          b_override_aspect;       /**< aspect ratio overriden */
-    mtime_t             render_time;           /**< last picture render time */
+    uint32_t            render_time;           /**< last picture render time */
     unsigned int        i_window_width;              /**< video window width */
     unsigned int        i_window_height;            /**< video window height */
+    unsigned int        i_alignment;          /**< video alignment in window */
+
+    intf_thread_t       *p_parent_intf;   /**< parent interface for embedded
+                                                               vout (if any) */
     /**@}*/
-    
+
     /** \name Plugin used and shortcuts to access its capabilities */
     /**@{*/
     module_t *   p_module;
@@ -92,6 +102,7 @@ struct vout_thread_t
     int       ( *pf_manage )     ( vout_thread_t * );
     void      ( *pf_render )     ( vout_thread_t *, picture_t * );
     void      ( *pf_display )    ( vout_thread_t *, picture_t * );
+    int       ( *pf_control )    ( vout_thread_t *, int, va_list );
     /**@}*/
 
     /** \name Statistics
@@ -115,6 +126,8 @@ struct vout_thread_t
     picture_t           p_picture[2*VOUT_MAX_PICTURES];        /**< pictures */
     subpicture_t        p_subpicture[VOUT_MAX_PICTURES];    /**< subpictures */
 
+    subpicture_t *      p_last_osd_message;
+
     /* Statistics */
     count_t          c_loops;
     count_t          c_pictures, c_late_pictures;
@@ -131,10 +144,11 @@ struct vout_thread_t
 
     /* text renderer data */
     text_renderer_sys_t * p_text_renderer_data;        /**< private data for
-						           the text renderer */
+                                                           the text renderer */
     module_t *            p_text_renderer_module;  /**< text renderer module */
-    int ( *pf_add_string ) ( vout_thread_t *, char *, text_style_t *, int,  
-			     int, int, mtime_t, mtime_t ); /**< callback used when a new string needs to be shown on the vout */
+    /** callback used when a new string needs to be shown on the vout */
+    subpicture_t * ( *pf_add_string ) ( vout_thread_t *, char *, text_style_t *, int,
+                             int, int, mtime_t, mtime_t );
 };
 
 #define I_OUTPUTPICTURES p_vout->output.i_pictures
@@ -142,18 +156,42 @@ struct vout_thread_t
 #define I_RENDERPICTURES p_vout->render.i_pictures
 #define PP_RENDERPICTURE p_vout->render.pp_picture
 
-/* Flags for changes - these flags are set in the i_changes field when another
- * thread changed a variable */
-#define VOUT_INFO_CHANGE        0x0001                     /* b_info changed */
-#define VOUT_GRAYSCALE_CHANGE   0x0002                /* b_grayscale changed */
-#define VOUT_INTF_CHANGE        0x0004                /* b_interface changed */
-#define VOUT_SCALE_CHANGE       0x0008                    /* b_scale changed */
-#define VOUT_GAMMA_CHANGE       0x0010                      /* gamma changed */
-#define VOUT_CURSOR_CHANGE      0x0020                   /* b_cursor changed */
-#define VOUT_FULLSCREEN_CHANGE  0x0040               /* b_fullscreen changed */
-#define VOUT_SIZE_CHANGE        0x0200                       /* size changed */
-#define VOUT_DEPTH_CHANGE       0x0400                      /* depth changed */
-#define VOUT_CHROMA_CHANGE      0x0800               /* change chroma tables */
+/** \defgroup vout_changes Flags for changes
+ * These flags are set in the vout_thread_t::i_changes field when another
+ * thread changed a variable
+ * @{
+ */
+/** b_info changed */
+#define VOUT_INFO_CHANGE        0x0001
+/** b_grayscale changed */
+#define VOUT_GRAYSCALE_CHANGE   0x0002
+/** b_interface changed */
+#define VOUT_INTF_CHANGE        0x0004
+/** b_scale changed */
+#define VOUT_SCALE_CHANGE       0x0008
+/** gamma changed */
+#define VOUT_GAMMA_CHANGE       0x0010
+/** b_cursor changed */
+#define VOUT_CURSOR_CHANGE      0x0020
+/** b_fullscreen changed */
+#define VOUT_FULLSCREEN_CHANGE  0x0040
+/** size changed */
+#define VOUT_SIZE_CHANGE        0x0200
+/** depth changed */
+#define VOUT_DEPTH_CHANGE       0x0400
+/** change chroma tables */
+#define VOUT_CHROMA_CHANGE      0x0800
+/** change/recreate picture buffers */
+#define VOUT_PICTURE_BUFFERS_CHANGE 0x1000
+/**@}*/
+
+/* Alignment flags */
+#define VOUT_ALIGN_LEFT         0x0001
+#define VOUT_ALIGN_RIGHT        0x0002
+#define VOUT_ALIGN_HMASK        0x0003
+#define VOUT_ALIGN_TOP          0x0004
+#define VOUT_ALIGN_BOTTOM       0x0008
+#define VOUT_ALIGN_VMASK        0x000C
 
 #define MAX_JITTER_SAMPLES      20
 
@@ -170,8 +208,9 @@ VLC_EXPORT( int, vout_VarCallback, ( vlc_object_t *, const char *, vlc_value_t,
 VLC_EXPORT( int,             vout_ChromaCmp,      ( uint32_t, uint32_t ) );
 
 VLC_EXPORT( picture_t *,     vout_CreatePicture,  ( vout_thread_t *, vlc_bool_t, vlc_bool_t, unsigned int ) );
-VLC_EXPORT( void,            vout_InitPicture,    ( vlc_object_t *, picture_t *, int, int, uint32_t ) );
-VLC_EXPORT( void,            vout_AllocatePicture,( vout_thread_t *, picture_t *, int, int, uint32_t ) );
+VLC_EXPORT( void,            vout_InitFormat,     ( video_frame_format_t *, uint32_t, int, int, int ) );
+VLC_EXPORT( void,            vout_InitPicture,    ( vlc_object_t *, picture_t *, uint32_t, int, int, int ) );
+VLC_EXPORT( void,            vout_AllocatePicture,( vlc_object_t *, picture_t *, uint32_t, int, int, int ) );
 VLC_EXPORT( void,            vout_DestroyPicture, ( vout_thread_t *, picture_t * ) );
 VLC_EXPORT( void,            vout_DisplayPicture, ( vout_thread_t *, picture_t * ) );
 VLC_EXPORT( void,            vout_DatePicture,    ( vout_thread_t *, picture_t *, mtime_t ) );
@@ -181,6 +220,43 @@ VLC_EXPORT( void,            vout_PlacePicture,   ( vout_thread_t *, unsigned in
 picture_t *     vout_RenderPicture  ( vout_thread_t *, picture_t *,
                                                        subpicture_t * );
 
+VLC_EXPORT( int, vout_vaControlDefault, ( vout_thread_t *, int, va_list ) );
+VLC_EXPORT( void *, vout_RequestWindow, ( vout_thread_t *, int *, int *, unsigned int *, unsigned int * ) );
+VLC_EXPORT( void,   vout_ReleaseWindow, ( vout_thread_t *, void * ) );
+VLC_EXPORT( int, vout_ControlWindow, ( vout_thread_t *, void *, int, va_list ) );
+
+static inline int vout_vaControl( vout_thread_t *p_vout, int i_query,
+                                  va_list args )
+{
+    if( p_vout->pf_control )
+        return p_vout->pf_control( p_vout, i_query, args );
+    else
+        return VLC_EGENERIC;
+}
+
+static inline int vout_Control( vout_thread_t *p_vout, int i_query, ... )
+{
+    va_list args;
+    int i_result;
+
+    va_start( args, i_query );
+    i_result = vout_vaControl( p_vout, i_query, args );
+    va_end( args );
+    return i_result;
+}
+
+enum output_query_e
+{
+    VOUT_SET_ZOOM,         /* arg1= double           res=    */
+    VOUT_SET_STAY_ON_TOP,  /* arg1= vlc_bool_t       res=    */
+    VOUT_REPARENT,
+    VOUT_CLOSE
+};
+
+/**
+ * \addtogroup subpicture
+ * @{
+ */
 VLC_EXPORT( subpicture_t *,  vout_CreateSubPicture,   ( vout_thread_t *, int ) );
 VLC_EXPORT( void,            vout_DestroySubPicture,  ( vout_thread_t *, subpicture_t * ) );
 VLC_EXPORT( void,            vout_DisplaySubPicture,  ( vout_thread_t *, subpicture_t * ) );
@@ -188,3 +264,7 @@ VLC_EXPORT( void,            vout_DisplaySubPicture,  ( vout_thread_t *, subpict
 subpicture_t *  vout_SortSubPictures    ( vout_thread_t *, mtime_t );
 void            vout_RenderSubPictures  ( vout_thread_t *, picture_t *,
                                                            subpicture_t * );
+/** @}*/
+/**
+ * @}
+ */
