@@ -1,8 +1,8 @@
 /*****************************************************************************
  * cddax.c : CD digital audio input module for vlc using libcdio
  *****************************************************************************
- * Copyright (C) 2000,2003 VideoLAN
- * $Id: access.c,v 1.19 2003/12/28 20:50:20 asmax Exp $
+ * Copyright (C) 2000, 2003, 2004 VideoLAN
+ * $Id: access.c 7587 2004-05-04 05:14:15Z rocky $
  *
  * Authors: Rocky Bernstein <rocky@panix.com>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -52,7 +52,7 @@
 #include "cdda.h"
 
 /* how many blocks Open will read in each loop */
-#define CDDA_BLOCKS_ONCE 1
+#define CDDA_BLOCKS_ONCE 20
 #define CDDA_DATA_ONCE   (CDDA_BLOCKS_ONCE * CDIO_CD_FRAMESIZE_RAW)
 
 #define CDDA_MRL_PREFIX "cddax://"
@@ -72,6 +72,13 @@ static int  CDDAFixupPlayList( input_thread_t *p_input,
                               cdda_data_t *p_cdda, const char *psz_source,
                               bool play_single_track);
 
+#if NEED_UPDATE_VAR
+static void CDDAUpdateVar( input_thread_t *p_input, int i_entry, int i_action,
+			   const char *p_varname, char *p_label,
+			   const char *p_debug_label );
+#endif
+
+
 /****************************************************************************
  * Private functions
  ****************************************************************************/
@@ -164,7 +171,7 @@ CDDAPlay( input_thread_t *p_input, int i_track )
 {
   cdda_data_t *p_cdda = (cdda_data_t *) p_input->p_access_data;
 
-  if( i_track >= p_cdda->i_nb_tracks || i_track < 1 )
+  if( i_track > p_cdda->i_nb_tracks || i_track < 1 )
     return VLC_FALSE;
 
   CDDASetArea( p_input, p_input->stream.pp_areas[i_track] );
@@ -187,29 +194,38 @@ static int CDDARead( input_thread_t * p_input, byte_t * p_buffer,
 
     p_cdda = (cdda_data_t *)p_input->p_access_data;
 
+    /* Compute the number of blocks we have to read */
+    i_blocks = i_len / CDIO_CD_FRAMESIZE_RAW;
     i_read = 0;
 
-    /* Compute the number of blocks we have to read */
+    if( !p_cdda->i_header_pos )
+    {
+        p_cdda->i_header_pos = sizeof(WAVEHEADER);
+        i_blocks = (i_len - sizeof(WAVEHEADER)) / CDIO_CD_FRAMESIZE_RAW;
+        memcpy( p_buffer, &p_cdda->waveheader, sizeof(WAVEHEADER) );
+        p_buffer += sizeof(WAVEHEADER);
+        i_read += sizeof(WAVEHEADER);
+    }
 
-    i_blocks = i_len / CDIO_CD_FRAMESIZE_RAW;
-
-    for ( i_index = 0; i_index < i_blocks; i_index++ )
+    for( i_index = 0; i_index < i_blocks; i_index++ )
     {
 
-      if (cdio_read_audio_sector(p_cdda->p_cddev->cdio, p_buffer,
-                                 p_cdda->i_sector) != 0)
+        if( cdio_read_audio_sector( p_cdda->p_cddev->cdio, p_buffer,
+                                    p_cdda->i_sector) != 0 )
         {
-          msg_Err( p_input, "could not read sector %d", p_cdda->i_sector );
-          return -1;
+            msg_Err( p_input, "could not read sector %lu", 
+		     (long unsigned int) p_cdda->i_sector );
+            return -1;
         }
 
         p_cdda->i_sector ++;
-        if ( p_cdda->i_sector == p_cdda->p_sectors[p_cdda->i_track + 1] )
+        if( p_cdda->i_sector == p_cdda->p_sectors[p_cdda->i_track + 1] )
         {
             input_area_t *p_area;
 
             dbg_print( (INPUT_DBG_LSN|INPUT_DBG_CALL),
-                       "end of track, cur: %u", p_cdda->i_sector );
+                       "end of track, cur: %lu", 
+		       (long unsigned int) p_cdda->i_sector );
 
             /*???? if ( p_cdda->i_track >= p_cdda->i_nb_tracks - 1 )*/
                 return 0; /* EOF */
@@ -223,6 +239,7 @@ static int CDDARead( input_thread_t * p_input, byte_t * p_buffer,
             vlc_mutex_unlock( &p_input->stream.stream_lock );
         }
         i_read += CDIO_CD_FRAMESIZE_RAW;
+        p_buffer += CDIO_CD_FRAMESIZE_RAW;
     }
 
     if ( i_len % CDIO_CD_FRAMESIZE_RAW ) /* this should not happen */
@@ -252,9 +269,13 @@ static int CDDASetArea( input_thread_t * p_input, input_area_t * p_area )
 {
     cdda_data_t *p_cdda = (cdda_data_t*) p_input->p_access_data;
     vlc_value_t val;
+    vlc_value_t text;
 
     dbg_print( (INPUT_DBG_CALL|INPUT_DBG_EXT), "");
 
+    text.psz_string = _("Track");
+    var_Change( p_input, "title", VLC_VAR_SETTEXT, &text, NULL );
+
     /* we can't use the interface slider until initilization is complete */
     p_input->stream.b_seekable = 0;
 
@@ -305,52 +326,74 @@ static void CDDASeek( input_thread_t * p_input, off_t i_off )
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
     dbg_print( (INPUT_DBG_CALL|INPUT_DBG_EXT|INPUT_DBG_SEEK),
-    "sector %ud, offset: %lld, i_tell: %lld",  p_cdda->i_sector, i_off,
+               "sector %lu, offset: %lld, i_tell: %lld",  
+	       (long unsigned int) p_cdda->i_sector, i_off,
                p_input->stream.p_selected_area->i_tell );
 
 }
 
-#define meta_info_add_str(title, str)                          \
-  if ( str ) {                                                 \
-    dbg_print( INPUT_DBG_META, "field %s: %s\n", title, str);  \
-    input_AddInfo( p_cat, _(title), "%s", str );               \
+#if NEED_UPDATE_VAR
+/****************************************************************************
+ Update the "varname" variable to i_num without triggering a callback.
+****************************************************************************/
+static void
+CDDAUpdateVar( input_thread_t *p_input, int i_num, int i_action,
+              const char *p_varname, char *p_label, 
+	      const char *p_debug_label)
+{
+  vlc_value_t val;
+  val.i_int = i_num;
+  if (p_label) {
+    vlc_value_t text;
+    text.psz_string = p_label;
+    var_Change( p_input, p_varname, VLC_VAR_SETTEXT, &text, NULL );
   }
-
+  var_Change( p_input, p_varname, i_action, &val, NULL );
+}
+#endif
 
 static void InformationCreate( input_thread_t *p_input  )
 {
   cdda_data_t *p_cdda = (cdda_data_t *) p_input->p_access_data;
-  input_info_category_t *p_cat;
-
-  p_cat = input_InfoCategory( p_input, "General" );
 
 #ifdef HAVE_LIBCDDB
   if (p_cdda->i_cddb_enabled) {
 
-    meta_info_add_str( "Title", p_cdda->cddb.disc->title );
-    meta_info_add_str( "Artist", p_cdda->cddb.disc->artist );
-    meta_info_add_str( "Genre", p_cdda->cddb.disc->genre );
-    meta_info_add_str( "Extended Data", p_cdda->cddb.disc->ext_data );
-    {
-      char year[5];
-      if (p_cdda->cddb.disc->year != 0) {
-        snprintf(year, 5, "%d", p_cdda->cddb.disc->year);
-        meta_info_add_str( "Year", year );
-      }
-      if ( p_cdda->cddb.disc->discid ) {
-        input_AddInfo( p_cat, _("CDDB Disc ID"), "%x",
-                       p_cdda->cddb.disc->discid );
-      }
+    dbg_print( INPUT_DBG_META, "field %s: %s\n", "Title",
+               p_cdda->cddb.disc->title );
+    input_Control( p_input, INPUT_ADD_INFO, _("General"), _("Title"),
+                   "%s", p_cdda->cddb.disc->title );
 
-      if ( p_cdda->cddb.disc->category != CDDB_CAT_INVALID ) {
-        input_AddInfo( p_cat, _("CDDB Disc Category"), "%s",
-                       CDDB_CATEGORY[p_cdda->cddb.disc->category] );
-      }
+    dbg_print( INPUT_DBG_META, "field %s: %s\n", "Artist",
+               p_cdda->cddb.disc->artist );
+    input_Control( p_input, INPUT_ADD_INFO, _("General"), _("Artist"),
+                   "%s", p_cdda->cddb.disc->artist );
 
-    }
+    dbg_print( INPUT_DBG_META, "field %s: %s\n", "Genre",
+               p_cdda->cddb.disc->genre );
+    input_Control( p_input, INPUT_ADD_INFO, _("General"), _("Genre"),
+                   "%s", p_cdda->cddb.disc->genre );
+
+    dbg_print( INPUT_DBG_META, "field %s: %s\n", "Extended Data",
+               p_cdda->cddb.disc->ext_data );
+    input_Control( p_input, INPUT_ADD_INFO, _("General"), _("Extended Data"),
+                   "%s", p_cdda->cddb.disc->ext_data );
+
+    if (p_cdda->cddb.disc->year != 0)
+      input_Control( p_input, INPUT_ADD_INFO, _("General"),
+                     _("Year"), "%d", p_cdda->cddb.disc->year );
+    if ( p_cdda->cddb.disc->discid )
+      input_Control( p_input, INPUT_ADD_INFO, _("General"),
+                     _("CDDB Disc ID"), "%x", p_cdda->cddb.disc->discid );
+
+    if ( p_cdda->cddb.disc->category != CDDB_CAT_INVALID )
+      input_Control( p_input, INPUT_ADD_INFO, _("General"),
+                     _("CDDB Disc Category"), "%s",
+                     CDDB_CATEGORY[p_cdda->cddb.disc->category] );
   }
 
 #endif /*HAVE_LIBCDDB*/
+#define TITLE_MAX 30
 
   {
     track_t i_track = p_cdda->i_nb_tracks;
@@ -360,8 +403,39 @@ static void InformationCreate( input_thread_t *p_input  )
       / CDIO_CD_FRAMES_PER_SEC;
 
     dbg_print( INPUT_DBG_META, "Duration %ld", (long int) i_duration );
-    input_AddInfo( p_cat, _("Duration"), "%s",
-                   secstotimestr( psz_buffer, i_duration ) );
+    input_Control( p_input, INPUT_ADD_INFO, _("General"), _("Duration"), "%s",
+		   secstotimestr( psz_buffer, i_duration ) );
+
+    for( i_track = 0 ; i_track < p_cdda->i_nb_tracks ; i_track++ ) {
+      char track_str[TITLE_MAX];
+      mtime_t i_duration =
+        (p_cdda->p_sectors[i_track+1] - p_cdda->p_sectors[i_track])
+        / CDIO_CD_FRAMES_PER_SEC;
+      snprintf(track_str, TITLE_MAX, "%s %02d", _("Track"), i_track+1);
+      input_Control( p_input, INPUT_ADD_INFO, track_str, _("Duration"), "%s",
+                     secstotimestr( psz_buffer, i_duration ) );
+
+#ifdef HAVE_LIBCDDB
+      if (p_cdda->i_cddb_enabled) {
+        cddb_track_t *t=cddb_disc_get_track(p_cdda->cddb.disc,
+                                            i_track);
+        if (t != NULL) {
+          if ( t->artist != NULL && strlen(t->artist) ) {
+            input_Control( p_input, INPUT_ADD_INFO, track_str,
+                           _("Artist"), "%s", t->artist );
+          }
+          if ( t->title != NULL && strlen(t->title) ) {
+            input_Control( p_input, INPUT_ADD_INFO, track_str,
+                           _("Title"), "%s",  t->title );
+          }
+          if ( t->ext_data != NULL && strlen(t->ext_data) ) {
+            input_Control( p_input, INPUT_ADD_INFO, track_str,
+                           _("Extended Data"), "%s",  t->ext_data );
+          }
+        }
+      }
+#endif
+    }
   }
 }
 
@@ -439,7 +513,6 @@ GetCDDBInfo( const input_thread_t *p_input, cdda_data_t *p_cdda )
       cdio_get_track_lba(cdio, CDIO_CDROM_LEADOUT_TRACK)
       / CDIO_CD_FRAMES_PER_SEC;
 
-
     if (!cddb_disc_calc_discid(p_cdda->cddb.disc)) {
       msg_Err( p_input, "CDDB disc calc failed" );
       goto cddb_destroy;
@@ -525,7 +598,7 @@ CDDAFormatStr(const input_thread_t *p_input, cdda_data_t *p_cdda,
   bool saw_control_prefix = false;
   size_t format_len = strlen(format_str);
 
-  bzero(temp_str, TEMP_STR_SIZE);
+  memset(temp_str, 0, TEMP_STR_SIZE);
 
   for (i=0; i<format_len; i++) {
 
@@ -583,6 +656,14 @@ CDDAFormatStr(const input_thread_t *p_input, cdda_data_t *p_cdda,
           add_format_str_info(t->artist);
       } else goto not_special;
       break;
+    case 'e':
+      if (p_cdda->i_cddb_enabled) {
+        cddb_track_t *t=cddb_disc_get_track(p_cdda->cddb.disc,
+                                            i_track-1);
+        if (t != NULL && t->ext_data != NULL)
+          add_format_str_info(t->ext_data);
+      } else goto not_special;
+      break;
 #endif
 
     case 'M':
@@ -635,16 +716,17 @@ CDDACreatePlayListItem(const input_thread_t *p_input, cdda_data_t *p_cdda,
 {
   mtime_t i_duration =
     (p_cdda->p_sectors[i_track] - p_cdda->p_sectors[i_track-1])
-    / CDIO_CD_FRAMES_PER_SEC;
+    * (1000000 / CDIO_CD_FRAMES_PER_SEC) ;
   char *p_author;
   char *p_title;
   char *config_varname = MODULE_STRING "-title-format";
+  playlist_item_t *p_item;
 
 #ifdef HAVE_LIBCDDB
   if (p_cdda->i_cddb_enabled) {
     config_varname = MODULE_STRING "-cddb-title-format";
   }
-#endif
+#endif /*HAVE_LIBCDDB*/
 
   snprintf(psz_mrl, psz_mrl_max, "%s%s@T%u",
            CDDA_MRL_PREFIX, psz_source, i_track);
@@ -654,28 +736,69 @@ CDDACreatePlayListItem(const input_thread_t *p_input, cdda_data_t *p_cdda,
                           psz_mrl, i_track);
 
   dbg_print( INPUT_DBG_META, "mrl: %s, title: %s, duration, %ld, pos %d",
-             psz_mrl, p_title, (long int) i_duration, i_pos );
+             psz_mrl, p_title, (long int) i_duration / 1000000 , i_pos );
+  playlist_AddExt( p_playlist, psz_mrl, p_title, playlist_operation,
+                         i_pos, i_duration , NULL, 0);
 
-  playlist_AddExt( p_playlist, psz_mrl, p_title, i_duration * 1000000,
-                   0, 0, playlist_operation, i_pos );
+  if( i_pos == PLAYLIST_END ) i_pos = p_playlist->i_size - 1;
+
+  vlc_mutex_lock( &p_playlist->object_lock );
+  p_item = playlist_ItemGetByPos( p_playlist, i_pos );
+  vlc_mutex_unlock( &p_playlist->object_lock );
+  if( !p_item )
+      return;
+
+  vlc_mutex_lock( &p_item->input.lock );
 
   p_author =
     CDDAFormatStr( p_input, p_cdda,
                    config_GetPsz( p_input, MODULE_STRING "-author-format" ),
                    psz_mrl, i_track );
 
-  /* FIXME: This is horrible, but until the playlist interface is fixed up
-     something like this has to be done for the "Author" field.
-   */
-  if( i_pos == PLAYLIST_END ) i_pos = p_playlist->i_size - 1;
-  free(p_playlist->pp_items[i_pos]->psz_author);
-  p_playlist->pp_items[i_pos]->psz_author = strdup(p_author);
+  playlist_ItemAddInfo( p_item ,  _("General"),_("Author"), p_author);
 
+#ifdef HAVE_LIBCDDB
+  if (p_cdda->i_cddb_enabled) {
+    const char *psz_general_cat = _("General");
+
+    playlist_ItemAddInfo( p_item, psz_general_cat, _("Album"),
+                      "%s", p_cdda->cddb.disc->title);
+    playlist_ItemAddInfo( p_item, psz_general_cat, _("Disc Artist(s)"),
+                      "%s", p_cdda->cddb.disc->artist);
+    playlist_ItemAddInfo( p_item, psz_general_cat,
+                        _("CDDB Disc Category"),
+                      "%s", CDDB_CATEGORY[p_cdda->cddb.disc->category]);
+    playlist_ItemAddInfo( p_item, psz_general_cat, _("Genre"),
+                      "%s", p_cdda->cddb.disc->genre);
+    if ( p_cdda->cddb.disc->discid ) {
+      playlist_ItemAddInfo( p_item, psz_general_cat, _("CDDB Disc ID"),
+                        "%x", p_cdda->cddb.disc->discid );
+    }
+    if (p_cdda->cddb.disc->year != 0) {
+      playlist_ItemAddInfo( p_item, psz_general_cat,
+                        _("Year"), "%5d", p_cdda->cddb.disc->year );
+    }
+
+    if (p_cdda->i_cddb_enabled) {
+      cddb_track_t *t=cddb_disc_get_track(p_cdda->cddb.disc,
+                                          i_track-1);
+      if (t != NULL && t->artist != NULL) {
+        playlist_ItemAddInfo( p_item, psz_general_cat,
+                          _("Track Artist"), "%s", t->artist );
+        playlist_ItemAddInfo( p_item , psz_general_cat,
+                          _("Track Title"), "%s",  t->title );
+      }
+    }
+
+  }
+#endif /*HAVE_LIBCDDB*/
+
+  vlc_mutex_unlock( &p_item->input.lock );
 }
 
 static int
 CDDAFixupPlayList( input_thread_t *p_input, cdda_data_t *p_cdda,
-                  const char *psz_source, bool play_single_track)
+                   const char *psz_source, bool play_single_track)
 {
   int i;
   playlist_t * p_playlist;
@@ -734,7 +857,6 @@ CDDAFixupPlayList( input_thread_t *p_input, cdda_data_t *p_cdda,
         CDDACreatePlayListItem(p_input, p_cdda, p_playlist, i, psz_mrl,
                                psz_mrl_max, psz_source, PLAYLIST_APPEND,
                                PLAYLIST_END);
-
       }
 
     playlist_Command( p_playlist, PLAYLIST_GOTO, 0 );
@@ -823,6 +945,7 @@ E_(Open)( vlc_object_t *p_this )
     int                     i;
     int                     i_track = 1;
     cddev_t                 *p_cddev;
+    vlc_value_t             val;
     bool                    play_single_track = false;
 
     /* Set where to log errors messages from libcdio. */
@@ -887,7 +1010,7 @@ E_(Open)( vlc_object_t *p_this )
     {
         msg_Warn( p_input, "could not open %s", psz_source );
         free( psz_source );
-        return -1;
+        return VLC_EGENERIC;
     }
 
     p_cdda = malloc( sizeof(cdda_data_t) );
@@ -895,7 +1018,7 @@ E_(Open)( vlc_object_t *p_this )
     {
         msg_Err( p_input, "out of memory" );
         free( psz_source );
-        return -1;
+        return VLC_ENOMEM;
     }
 
     p_cdda->p_cddev        = p_cddev;
@@ -914,15 +1037,15 @@ E_(Open)( vlc_object_t *p_this )
     else if( p_cdda->i_nb_tracks <= 0 )
         msg_Err( p_input, "no audio tracks found" );
 
-    if( p_cdda->i_nb_tracks <= 1)
+    if( p_cdda->i_nb_tracks <= 0 )
     {
         ioctl_Close( p_cdda->p_cddev );
         free( p_cdda );
         free( psz_source );
-        return -1;
+        return VLC_EGENERIC;
     }
 
-    if( i_track >= p_cdda->i_nb_tracks || i_track < 1 )
+    if( i_track > p_cdda->i_nb_tracks || i_track < 1 )
         i_track = 1;
 
     /* Set stream and area data */
@@ -953,15 +1076,9 @@ E_(Open)( vlc_object_t *p_this )
 #undef area
 
     CDDAPlay( p_input, i_track);
-
-    CDDAFixupPlayList(p_input, p_cdda, psz_source, play_single_track);
-
     vlc_mutex_unlock( &p_input->stream.stream_lock );
 
-    if( !p_input->psz_demux || !*p_input->psz_demux )
-    {
-        p_input->psz_demux = "cdda";
-    }
+    CDDAFixupPlayList(p_input, p_cdda, psz_source, play_single_track);
 
     p_input->pf_read = CDDARead;
     p_input->pf_seek = CDDASeek;
@@ -969,12 +1086,33 @@ E_(Open)( vlc_object_t *p_this )
     p_input->pf_set_program = CDDASetProgram;
 
     /* Update default_pts to a suitable value for cdda access */
-    p_input->i_pts_delay = config_GetInt( p_input,
-                                          MODULE_STRING "-caching" ) * 1000;
+    var_Create( p_input, MODULE_STRING "-caching",
+                VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_input, MODULE_STRING "-caching", &val );
+    p_input->i_pts_delay = val.i_int * 1000;
 
     free( psz_source );
 
-    return 0;
+    /* Build a WAV header for the output data */
+    memset( &p_cdda->waveheader, 0, sizeof(WAVEHEADER) );
+    SetWLE( &p_cdda->waveheader.Format, 1 ); /*WAVE_FORMAT_PCM*/
+    SetWLE( &p_cdda->waveheader.BitsPerSample, 16);
+    p_cdda->waveheader.MainChunkID = VLC_FOURCC('R', 'I', 'F', 'F');
+    p_cdda->waveheader.Length = 0;                     /* we just don't know */
+    p_cdda->waveheader.ChunkTypeID = VLC_FOURCC('W', 'A', 'V', 'E');
+    p_cdda->waveheader.SubChunkID  = VLC_FOURCC('f', 'm', 't', ' ');
+    SetDWLE( &p_cdda->waveheader.SubChunkLength, 16);
+    SetWLE( &p_cdda->waveheader.Modus, 2);
+    SetDWLE( &p_cdda->waveheader.SampleFreq, 44100);
+    SetWLE( &p_cdda->waveheader.BytesPerSample,
+            2 /*Modus*/ * 16 /*BitsPerSample*/ / 8 );
+    SetDWLE( &p_cdda->waveheader.BytesPerSec,
+	     2*16/8 /*BytesPerSample*/ * 44100 /*SampleFreq*/ );
+    p_cdda->waveheader.DataChunkID = VLC_FOURCC('d', 'a', 't', 'a');
+    p_cdda->waveheader.DataLength = 0;                 /* we just don't know */
+    p_cdda->i_header_pos = 0;
+
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -997,6 +1135,7 @@ E_(Close)( vlc_object_t *p_this )
       cddb_disc_destroy(p_cdda->cddb.disc);
 #endif
 
+    free( p_cdda->p_sectors );
     free( p_cdda );
     p_cdda_input = NULL;
 }
