@@ -6,7 +6,7 @@
  * It depends on: libdvdread for ifo files and block reading.
  *****************************************************************************
  * Copyright (C) 2001, 2003 VideoLAN
- * $Id: input.c,v 1.22 2003/05/08 19:06:44 titer Exp $
+ * $Id: input.c 6961 2004-03-05 17:34:23Z sam $
  *
  * Author: Stphane Borel <stef@via.ecp.fr>
  *
@@ -121,7 +121,7 @@ int E_(InitDVD) ( vlc_object_t *p_this )
     }
 
     p_input->p_private = (void*)&p_demux->mpeg;
-    p_demux->p_module = module_Need( p_input, "mpeg-system", NULL );
+    p_demux->p_module = module_Need( p_input, "mpeg-system", NULL, 0 );
     if( p_demux->p_module == NULL )
     {
         free( p_input->p_demux_data );
@@ -129,6 +129,7 @@ int E_(InitDVD) ( vlc_object_t *p_this )
     }
 
     p_input->pf_demux = DvdReadDemux;
+    p_input->pf_demux_control = demux_vaControlDefault;
     p_input->pf_rewind = NULL;
 
     vlc_mutex_lock( &p_input->stream.stream_lock );
