@@ -2,10 +2,10 @@
  * oss.c : OSS /dev/dsp module for vlc
  *****************************************************************************
  * Copyright (C) 2000-2002 VideoLAN
- * $Id: oss.c,v 1.56 2003/04/02 21:21:57 sigmunau Exp $
+ * $Id: oss.c 7522 2004-04-27 16:35:15Z sam $
  *
  * Authors: Michel Kaempf <maxx@via.ecp.fr>
- *          Samuel Hocevar <sam@zoy.org>
+ *          Sam Hocevar <sam@zoy.org>
  *          Christophe Massiot <massiot@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -55,7 +55,7 @@
 
 /* Patches for ignorant OSS versions */
 #ifndef AFMT_AC3
-#   define AFMT_AC3     0x00000400	/* Dolby Digital AC3 */
+#   define AFMT_AC3     0x00000400        /* Dolby Digital AC3 */
 #endif
 
 #ifndef AFMT_S16_NE
@@ -66,12 +66,11 @@
 #   endif
 #endif
 
-
 /*****************************************************************************
  * aout_sys_t: OSS audio output method descriptor
  *****************************************************************************
  * This structure is part of the audio output thread descriptor.
- * It describes the dsp specific properties of an audio device.
+ * It describes the DSP specific properties of an audio device.
  *****************************************************************************/
 struct aout_sys_t
 {
@@ -99,18 +98,19 @@ static mtime_t BufferDuration( aout_instance_t * p_aout );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define BUGGY_TEXT N_("try to work around buggy OSS drivers")
+#define BUGGY_TEXT N_("Try to work around buggy OSS drivers")
 #define BUGGY_LONGTEXT N_( \
     "Some buggy OSS drivers just don't like when their internal buffers " \
     "are completely filled (the sound gets heavily hashed). If you have one " \
     "of these drivers, then you need to enable this option." )
 
 vlc_module_begin();
-    add_category_hint( N_("OSS"), NULL, VLC_FALSE );
-    add_file( "dspdev", "/dev/dsp", aout_FindAndRestart,
-              N_("OSS dsp device"), NULL, VLC_FALSE );
-    add_bool( "oss-buggy", 0, NULL, BUGGY_TEXT, BUGGY_LONGTEXT, VLC_TRUE );
     set_description( _("Linux OSS audio output") );
+
+    add_file( "dspdev", "/dev/dsp", aout_FindAndRestart,
+              N_("OSS DSP device"), NULL, VLC_FALSE );
+    add_bool( "oss-buggy", 0, NULL, BUGGY_TEXT, BUGGY_LONGTEXT, VLC_TRUE );
+
     set_capability( "audio output", 100 );
     add_shortcut( "oss" );
     set_callbacks( Open, Close );
@@ -122,10 +122,12 @@ vlc_module_end();
 static void Probe( aout_instance_t * p_aout )
 {
     struct aout_sys_t * p_sys = p_aout->output.p_sys;
-    vlc_value_t val;
+    vlc_value_t val, text;
     int i_format, i_nb_channels;
 
-    var_Create( p_aout, "audio-device", VLC_VAR_STRING | VLC_VAR_HASCHOICE );
+    var_Create( p_aout, "audio-device", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    text.psz_string = _("Audio Device");
+    var_Change( p_aout, "audio-device", VLC_VAR_SETTEXT, &text, NULL );
 
     /* Test for multi-channel. */
 #ifdef SNDCTL_DSP_GETCHANNELMASK
@@ -161,8 +163,10 @@ static void Probe( aout_instance_t * p_aout )
                          | AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT
                          | AOUT_CHAN_LFE)) )
             {
-                val.psz_string = N_("5.1");
-                var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val );
+                val.i_int = AOUT_VAR_5_1;
+                text.psz_string = N_("5.1");
+                var_Change( p_aout, "audio-device",
+                            VLC_VAR_ADDCHOICE, &val, &text );
             }
 
             if ( (i_chanmask & DSP_BIND_SURR)
@@ -170,8 +174,10 @@ static void Probe( aout_instance_t * p_aout )
                        (AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT
                          | AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT)) )
             {
-                val.psz_string = N_("2 Front 2 Rear");
-                var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val );
+                val.i_int = AOUT_VAR_2F2R;
+                text.psz_string = N_("2 Front 2 Rear");
+                var_Change( p_aout, "audio-device",
+                            VLC_VAR_ADDCHOICE, &val, &text );
             }
         }
     }
@@ -192,8 +198,9 @@ static void Probe( aout_instance_t * p_aout )
     if( ioctl( p_sys->i_fd, SNDCTL_DSP_CHANNELS, &i_nb_channels ) >= 0
          && i_nb_channels == 2 )
     {
-        val.psz_string = N_("Stereo");
-        var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val );
+        val.i_int = AOUT_VAR_STEREO;
+        text.psz_string = N_("Stereo");
+        var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val, &text );
     }
 
     /* Reset all. */
@@ -211,8 +218,9 @@ static void Probe( aout_instance_t * p_aout )
     if( ioctl( p_sys->i_fd, SNDCTL_DSP_CHANNELS, &i_nb_channels ) >= 0
          && i_nb_channels == 1 )
     {
-        val.psz_string = N_("Mono");
-        var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val );
+        val.i_int = AOUT_VAR_MONO;
+        text.psz_string = N_("Mono");
+        var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val, &text );
         if ( p_aout->output.output.i_physical_channels == AOUT_CHAN_CENTER )
         {
             var_Set( p_aout, "audio-device", val );
@@ -234,11 +242,17 @@ static void Probe( aout_instance_t * p_aout )
         if( ioctl( p_sys->i_fd, SNDCTL_DSP_SETFMT, &i_format ) >= 0
              && i_format == AFMT_AC3 )
         {
-            val.psz_string = N_("A/52 over S/PDIF");
-            var_Change( p_aout, "audio-device", VLC_VAR_ADDCHOICE, &val );
+            val.i_int = AOUT_VAR_SPDIF;
+            text.psz_string = N_("A/52 over S/PDIF");
+            var_Change( p_aout, "audio-device",
+                        VLC_VAR_ADDCHOICE, &val, &text );
             if( config_GetInt( p_aout, "spdif" ) )
                 var_Set( p_aout, "audio-device", val );
         }
+        else if( config_GetInt( p_aout, "spdif" ) )
+        {
+            msg_Warn( p_aout, "s/pdif not supported by card" );
+        }
     }
 
     var_AddCallback( p_aout, "audio-device", aout_ChannelsRestart,
@@ -248,7 +262,7 @@ static void Probe( aout_instance_t * p_aout )
 /*****************************************************************************
  * Open: open the audio device (the digital sound processor)
  *****************************************************************************
- * This function opens the dsp as a usual non-blocking write-only file, and
+ * This function opens the DSP as a usual non-blocking write-only file, and
  * modifies the p_aout->p_sys->i_fd with the file's descriptor.
  *****************************************************************************/
 static int Open( vlc_object_t *p_this )
@@ -269,12 +283,16 @@ static int Open( vlc_object_t *p_this )
     /* Get device name */
     if( (psz_device = config_GetPsz( p_aout, "dspdev" )) == NULL )
     {
-        msg_Err( p_aout, "no audio device given (maybe /dev/dsp ?)" );
+        msg_Err( p_aout, "no audio device specified (maybe /dev/dsp?)" );
         free( p_sys );
         return VLC_EGENERIC;
     }
 
-    /* Open the sound device */
+    /* Open the sound device in non-blocking mode, because ALSA's OSS
+     * emulation and some broken OSS drivers would make a blocking call
+     * wait forever until the device is available. Since this breaks the
+     * OSS spec, we immediately put it back to blocking mode if the
+     * operation was successful. */
     p_sys->i_fd = open( psz_device, O_WRONLY | O_NDELAY );
     if( p_sys->i_fd < 0 )
     {
@@ -303,11 +321,11 @@ static int Open( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    if ( !strcmp( val.psz_string, N_("A/52 over S/PDIF") ) )
+    if ( val.i_int == AOUT_VAR_SPDIF )
     {
         p_aout->output.output.i_format = VLC_FOURCC('s','p','d','i');
     }
-    else if ( !strcmp( val.psz_string, N_("5.1") ) )
+    else if ( val.i_int == AOUT_VAR_5_1 )
     {
         p_aout->output.output.i_format = AOUT_FMT_S16_NE;
         p_aout->output.output.i_physical_channels
@@ -315,20 +333,20 @@ static int Open( vlc_object_t *p_this )
                | AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT
                | AOUT_CHAN_LFE;
     }
-    else if ( !strcmp( val.psz_string, N_("2 Front 2 Rear") ) )
+    else if ( val.i_int == AOUT_VAR_2F2R )
     {
         p_aout->output.output.i_format = AOUT_FMT_S16_NE;
         p_aout->output.output.i_physical_channels
             = AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT
                | AOUT_CHAN_REARLEFT | AOUT_CHAN_REARRIGHT;
     }
-    else if ( !strcmp( val.psz_string, N_("Stereo") ) )
+    else if ( val.i_int == AOUT_VAR_STEREO )
     {
         p_aout->output.output.i_format = AOUT_FMT_S16_NE;
         p_aout->output.output.i_physical_channels
             = AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT;
     }
-    else if ( !strcmp( val.psz_string, N_("Mono") ) )
+    else if ( val.i_int == AOUT_VAR_MONO )
     {
         p_aout->output.output.i_format = AOUT_FMT_S16_NE;
         p_aout->output.output.i_physical_channels = AOUT_CHAN_CENTER;
@@ -336,13 +354,10 @@ static int Open( vlc_object_t *p_this )
     else
     {
         /* This should not happen ! */
-        msg_Err( p_aout, "internal: can't find audio-device (%s)",
-                 val.psz_string );
+        msg_Err( p_aout, "internal: can't find audio-device (%i)", val.i_int );
         free( p_sys );
-        free( val.psz_string );
         return VLC_EGENERIC;
     }
-    free( val.psz_string );
 
     val.b_bool = VLC_TRUE;
     var_Set( p_aout, "intf-change", val );
@@ -518,7 +533,7 @@ static void Play( aout_instance_t *p_aout )
 }
 
 /*****************************************************************************
- * Close: close the dsp audio device
+ * Close: close the DSP audio device
  *****************************************************************************/
 static void Close( vlc_object_t * p_this )
 {
