@@ -2,10 +2,10 @@
  * video.c: video decoder using the ffmpeg library
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: video.c,v 1.67 2004/02/25 17:48:52 fenrir Exp $
+ * $Id: video.c 7673 2004-05-15 11:10:42Z fenrir $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
- *          Gildas Bazin <gbazin@netcourrier.com>
+ *          Gildas Bazin <gbazin@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,6 +91,12 @@ static void ffmpeg_CopyPicture    ( decoder_t *, picture_t *, AVFrame * );
 static int  ffmpeg_GetFrameBuf    ( struct AVCodecContext *, AVFrame * );
 static void ffmpeg_ReleaseFrameBuf( struct AVCodecContext *, AVFrame * );
 
+static uint32_t ffmpeg_CodecTag( vlc_fourcc_t fcc )
+{
+    uint8_t *p = (uint8_t*)&fcc;
+    return p[0] | (p[1] << 8) | (p[2] << 16) | (p[3] << 24);
+}
+
 /*****************************************************************************
  * Local Functions
  *****************************************************************************/
@@ -197,8 +203,6 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
     decoder_sys_t *p_sys;
     vlc_value_t lockval;
     vlc_value_t val;
-    int i_tmp;
-
 
     var_Get( p_dec->p_libvlc, "avcodec", &lockval );
 
@@ -217,25 +221,26 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
     p_sys->p_ff_pic = avcodec_alloc_frame();
 
     /* ***** Fill p_context with init values ***** */
+    p_sys->p_context->codec_tag = ffmpeg_CodecTag( p_dec->fmt_in.i_codec );
     p_sys->p_context->width  = p_dec->fmt_in.video.i_width;
     p_sys->p_context->height = p_dec->fmt_in.video.i_height;
     p_sys->p_context->bits_per_sample = p_dec->fmt_in.video.i_bits_per_pixel;
 
     /*  ***** Get configuration of ffmpeg plugin ***** */
-    i_tmp = config_GetInt( p_dec, "ffmpeg-workaround-bugs" );
-    p_sys->p_context->workaround_bugs  = __MAX( __MIN( i_tmp, 99 ), 0 );
-
-    i_tmp = config_GetInt( p_dec, "ffmpeg-error-resilience" );
-    p_sys->p_context->error_resilience = __MAX( __MIN( i_tmp, 99 ), -1 );
+    p_sys->p_context->workaround_bugs =
+        config_GetInt( p_dec, "ffmpeg-workaround-bugs" );
+    p_sys->p_context->error_resilience =
+        config_GetInt( p_dec, "ffmpeg-error-resilience" );
 
     var_Create( p_dec, "grayscale", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Get( p_dec, "grayscale", &val );
     if( val.b_bool ) p_sys->p_context->flags |= CODEC_FLAG_GRAY;
 
-    /* Decide if we set CODEC_FLAG_TRUNCATED */
-    var_Create( p_dec, "ffmpeg-truncated", VLC_VAR_INTEGER|VLC_VAR_DOINHERIT );
-    var_Get( p_dec, "ffmpeg-truncated", &val );
-    if( val.i_int > 0 ) p_sys->p_context->flags |= CODEC_FLAG_TRUNCATED;
+    var_Create( p_dec, "ffmpeg-vismv", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_dec, "ffmpeg-vismv", &val );
+#if LIBAVCODEC_BUILD >= 4698
+    if( val.i_int ) p_sys->p_context->debug_mv = val.i_int;
+#endif
 
     /* ***** ffmpeg frame skipping ***** */
     var_Create( p_dec, "ffmpeg-hurry-up", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
@@ -250,7 +255,14 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
         ffmpeg_PixFmtToChroma( p_sys->p_context->pix_fmt ) &&
         /* Apparently direct rendering doesn't work with YUV422P */
         p_sys->p_context->pix_fmt != PIX_FMT_YUV422P &&
-        !(p_sys->p_context->width % 16) && !(p_sys->p_context->height % 16) )
+        /* H264 uses too many reference frames */
+        p_sys->i_codec_id != CODEC_ID_H264 &&
+        !(p_sys->p_context->width % 16) && !(p_sys->p_context->height % 16) &&
+#if LIBAVCODEC_BUILD >= 4698
+        !p_sys->p_context->debug_mv )
+#else
+        1 )
+#endif
     {
         /* Some codecs set pix_fmt only after the 1st frame has been decoded,
          * so we need to do another check in ffmpeg_GetFrameBuf() */
@@ -293,6 +305,31 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
             memcpy( &p[0],  "SVQ3", 4 );
             memset( &p[4], 0, 8 );
             memcpy( &p[12], p_dec->fmt_in.p_extra, i_size );
+
+            /* Now remove all atoms before the SMI one */
+            if( p_sys->p_context->extradata_size > 0x5a &&
+                strncmp( &p[0x56], "SMI ", 4 ) )
+            {
+                uint8_t *psz = &p[0x52];
+
+                while( psz < &p[p_sys->p_context->extradata_size - 8] )
+                {
+                    int i_size = GetDWBE( psz );
+                    if( i_size <= 1 )
+                    {
+                        /* FIXME handle 1 as long size */
+                        break;
+                    }
+                    if( !strncmp( &psz[4], "SMI ", 4 ) )
+                    {
+                        memmove( &p[0x52], psz,
+                                 &p[p_sys->p_context->extradata_size] - psz );
+                        break;
+                    }
+
+                    psz += i_size;
+                }
+            }
         }
         else if( p_dec->fmt_in.i_codec == VLC_FOURCC( 'R', 'V', '1', '0' ) ||
                  p_dec->fmt_in.i_codec == VLC_FOURCC( 'R', 'V', '1', '3' ) ||
@@ -308,10 +345,11 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
                         p_dec->fmt_in.i_extra );
                 p_sys->p_context->sub_id= ((uint32_t*)p_dec->fmt_in.p_extra)[1];
 
-                msg_Warn( p_dec, "using extra data for RV codec sub_id=%08x", p_sys->p_context->sub_id );
+                msg_Warn( p_dec, "using extra data for RV codec sub_id=%08x",
+                          p_sys->p_context->sub_id );
             }
         }
-        else
+        else if( p_dec->fmt_in.i_codec != VLC_FOURCC( 'a', 'v', 'c', '1' ) )
         {
             p_sys->p_context->extradata_size = i_size;
             p_sys->p_context->extradata =
@@ -373,11 +411,14 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
 
     p_block = *pp_block;
 
-    if( p_block->i_flags&BLOCK_FLAG_DISCONTINUITY )
+    if( p_block->i_flags & BLOCK_FLAG_DISCONTINUITY )
     {
         p_sys->i_buffer = 0;
         p_sys->i_pts = 0; /* To make sure we recover properly */
 
+        p_sys->input_pts = p_sys->input_dts = 0;
+        p_sys->i_late_frames = 0;
+
         block_Release( p_block );
         return NULL;
     }
@@ -494,10 +535,7 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
         /* Nothing to display */
         if( !b_gotpicture )
         {
-            if( i_used == 0 )
-            {
-                break;
-            }
+            if( i_used == 0 ) break;
             continue;
         }
 
@@ -513,7 +551,7 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
             p_sys->i_late_frames = 0;
         }
 
-        if( !b_drawpicture || p_sys->p_ff_pic->linesize[0] == 0 )
+        if( !b_drawpicture || !p_sys->p_ff_pic->linesize[0] )
         {
             /* Do not display the picture */
             continue;
@@ -681,7 +719,7 @@ static int ffmpeg_GetFrameBuf( struct AVCodecContext *p_context,
         /* Some demuxers only set the dts so let's try to find a useful
          * timestamp from this */
         if( !p_context->has_b_frames || !p_sys->b_has_b_frames ||
-            !p_ff_pic->reference )
+            !p_ff_pic->reference || !p_sys->i_pts )
         {
             p_ff_pic->pts = p_sys->input_dts;
         }
@@ -689,7 +727,11 @@ static int ffmpeg_GetFrameBuf( struct AVCodecContext *p_context,
     }
     else p_ff_pic->pts = 0;
 
-    p_sys->input_pts = p_sys->input_dts = 0;
+    if( p_sys->i_pts ) /* make sure 1st frame has a pts > 0 */
+    {
+        p_sys->input_pts = p_sys->input_dts = 0;
+    }
+
     p_ff_pic->opaque = 0;
 
     /* Not much to do in indirect rendering mode */
