@@ -2,7 +2,7 @@
  * mpeg_ts.c : Transport Stream input module for vlc
  *****************************************************************************
  * Copyright (C) 2000-2004 VideoLAN
- * $Id: ts.c,v 1.46 2004/01/25 20:05:28 hartman Exp $
+ * $Id: ts.c 7421 2004-04-22 03:25:47Z fenrir $
  *
  * Authors: Henri Fallon <henri@via.ecp.fr>
  *          Johan Bilien <jobi@via.ecp.fr>
@@ -230,7 +230,7 @@ static int Activate( vlc_object_t * p_this )
     }
 
     p_input->p_private = (void*)&p_demux->mpeg;
-    p_demux->p_module = module_Need( p_input, "mpeg-system", NULL );
+    p_demux->p_module = module_Need( p_input, "mpeg-system", NULL, 0 );
     if( p_demux->p_module == NULL )
     {
         free( p_input->p_demux_data );
@@ -1449,6 +1449,10 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
                     i_cat = AUDIO_ES;
                     i_stream_id = 0xfa;
                     break;
+                case H264_VIDEO_ES:
+                    i_fourcc = VLC_FOURCC('h','2','6','4');
+                    i_cat = VIDEO_ES;
+                    break;
                 case MSCODEC_VIDEO_ES:
                     i_fourcc = VLC_FOURCC(0,0,0,0);   /* fixed later */
                     i_cat = VIDEO_ES;
@@ -1732,7 +1736,8 @@ static void TS_DVBPSI_HandlePMT( input_thread_t * p_input,
                 {
                     dvbpsi_iso639_dr_t *p_decoded =
                                                 dvbpsi_DecodeISO639Dr( p_dr );
-                    local_iso639_getlang(p_decoded->i_iso_639_code, psz_desc);
+                    if( p_decoded != NULL )
+                        local_iso639_getlang(p_decoded->i_iso_639_code, psz_desc);
 #if 0
                     if( p_decoded->i_code_count > 0 )
                     {
