@@ -2,7 +2,7 @@
  * gnome.c : Gnome plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000 VideoLAN
- * $Id: gnome.c,v 1.10 2003/02/05 22:11:52 sam Exp $
+ * $Id: gnome.c 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -52,13 +52,13 @@ static void Manage       ( intf_thread_t * );
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
-#define TOOLTIPS_TEXT N_("show tooltips")
+#define TOOLTIPS_TEXT N_("Show tooltips")
 #define TOOLTIPS_LONGTEXT N_("Show tooltips for configuration options.")
 
-#define TOOLBAR_TEXT N_("show text on toolbar buttons")
+#define TOOLBAR_TEXT N_("Show text on toolbar buttons")
 #define TOOLBAR_LONGTEXT N_("Show the text below icons on the toolbar.")
 
-#define PREFS_MAXH_TEXT N_("maximum height for the configuration windows")
+#define PREFS_MAXH_TEXT N_("Maximum height for the configuration windows")
 #define PREFS_MAXH_LONGTEXT N_( \
     "You can set the maximum height that the configuration windows in the " \
     "preferences menu will occupy.")
@@ -69,15 +69,15 @@ vlc_module_begin();
 #else
     int i = getenv( "DISPLAY" ) == NULL ? 15 : 100;
 #endif
-    add_category_hint( N_("GNOME"), NULL );
+    set_description( _("GNOME interface") );
+
     add_bool( "gnome-tooltips", 1, E_(GtkHideTooltips),
-              TOOLTIPS_TEXT, TOOLTIPS_LONGTEXT );
+              TOOLTIPS_TEXT, TOOLTIPS_LONGTEXT, VLC_FALSE );
     add_bool( "gnome-toolbartext", 1, GtkHideToolbarText, TOOLBAR_TEXT,
-              TOOLBAR_LONGTEXT );
+              TOOLBAR_LONGTEXT, VLC_FALSE );
     add_integer( "gnome-prefs-maxh", 480, NULL,
-                 PREFS_MAXH_TEXT, PREFS_MAXH_LONGTEXT );
+                 PREFS_MAXH_TEXT, PREFS_MAXH_LONGTEXT, VLC_TRUE );
 
-    set_description( _("GNOME interface module") );
     set_capability( "interface", i );
     set_callbacks( Open, Close );
     set_program( "gnome-vlc" );
@@ -98,7 +98,8 @@ static int Open( vlc_object_t *p_this )
         return VLC_ENOMEM;
     }
 
-    p_intf->p_sys->p_gtk_main = module_Need( p_this, "gtk_main", "gnome" );
+    p_intf->p_sys->p_gtk_main =
+        module_Need( p_this, "gui-helper", "gnome", VLC_TRUE );
     if( p_intf->p_sys->p_gtk_main == NULL )
     {
         free( p_intf->p_sys );
@@ -378,6 +379,15 @@ static void Manage( intf_thread_t *p_intf )
         p_intf->p_sys->p_sub->i_start = i_start;
         vlc_mutex_unlock( p_intf->p_sys->p_sub->p_lock );
 
+        /* If the messages list becomes too big, just clean half of it. */
+        if( gtk_text_get_length( p_intf->p_sys->p_messages_text ) >
+            VLC_MSG_QSIZE * 1000 )
+        {
+            gtk_text_set_point( p_intf->p_sys->p_messages_text, 0 );
+            gtk_text_forward_delete( p_intf->p_sys->p_messages_text,
+                gtk_text_get_length( p_intf->p_sys->p_messages_text ) / 2 );
+        }
+
         gtk_text_set_point( p_intf->p_sys->p_messages_text,
                     gtk_text_get_length( p_intf->p_sys->p_messages_text ) );
     }
@@ -442,9 +452,9 @@ static void Manage( intf_thread_t *p_intf )
                  * finished dragging the slider.
                  * Beware, the hack below is needed by the dvdplay plugin! */
                 else if( p_intf->p_sys->b_slider_free
-                /* hack -> */ && (p_intf->p_sys->f_adj_oldvalue <= 100.) )
+                /* hack -> */ && (p_intf->p_sys->f_adj_oldvalue < 100.) )
                 {
-                    if( newvalue > 0. && newvalue < 100. )
+                    if( newvalue >= 0. && newvalue < 100. )
                     {
                         off_t i_seek = ( newvalue * p_area->i_size ) / 100;
 
