@@ -1,18 +1,18 @@
 /*****************************************************************************
- * intf.h: MacOS X interface plugin
+ * intf.h: MacOS X interface module
  *****************************************************************************
- * Copyright (C) 2002-2003 VideoLAN
- * $Id: intf.h,v 1.42 2003/06/01 23:48:17 hartman Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: intf.h 7708 2004-05-17 22:14:25Z fkuehne $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
- *          Derk-Jan Hartman <thedj@users.sourceforge.net>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -31,22 +31,20 @@
 #include <Cocoa/Cocoa.h>
 
 /*****************************************************************************
- * VLCApplication interface 
+ * VLCApplication interface
  *****************************************************************************/
 @interface VLCApplication : NSApplication
 {
-    NSStringEncoding i_encoding;
     intf_thread_t *p_intf;
 }
 
-- (void)initIntlSupport;
 - (NSString *)localizedString:(char *)psz;
 - (char *)delocalizeString:(NSString *)psz;
-- (NSStringEncoding)getEncoding;
 - (NSString *)wrapString: (NSString *)o_in_string toWidth: (int)i_width;
 
 - (void)setIntf:(intf_thread_t *)p_intf;
 - (intf_thread_t *)getIntf;
+- (BOOL)hasDefinedShortcutKey:(NSEvent *)o_event;
 
 @end
 
@@ -59,6 +57,7 @@
 int ExecuteOnMainThread( id target, SEL sel, void * p_arg );
 int PlaylistChanged( vlc_object_t *p_this, const char *psz_variable,
                      vlc_value_t old_val, vlc_value_t new_val, void *param );
+unsigned int CocoaKeyToVLC( unichar i_key );
 
 /*****************************************************************************
  * intf_sys_t: description and status of the interface
@@ -77,7 +76,8 @@ struct intf_sys_t
     vlc_bool_t b_play_status;
     vlc_bool_t b_playlist_update;
     vlc_bool_t b_current_title_update;
-    
+    vlc_bool_t b_fullscreen_update;
+
     /* menus handlers */
     vlc_bool_t b_input_update;
     vlc_bool_t b_aout_update;
@@ -88,40 +88,42 @@ struct intf_sys_t
 };
 
 /*****************************************************************************
- * VLCMain interface 
+ * VLCMain interface
  *****************************************************************************/
 @interface VLCMain : NSObject
 {
     id o_prefs;                 /* VLCPrefs       */
 
     IBOutlet id o_window;       /* main window    */
+    IBOutlet id o_scrollfield;  /* info field     */
     IBOutlet id o_timefield;    /* time field     */
     IBOutlet id o_timeslider;   /* time slider    */
     float f_slider;             /* slider value   */
-    float f_slider_old;         /* old slider val */ 
+    float f_slider_old;         /* old slider val */
     IBOutlet id o_volumeslider; /* volume slider  */
 
-    IBOutlet id o_btn_playlist; /* btn playlist   */
     IBOutlet id o_btn_prev;     /* btn previous   */
-    IBOutlet id o_btn_slower;   /* btn slower     */
+    IBOutlet id o_btn_rewind;   /* btn rewind     */
     IBOutlet id o_btn_play;     /* btn play       */
     IBOutlet id o_btn_stop;     /* btn stop       */
-    IBOutlet id o_btn_faster;   /* btn faster     */
+    IBOutlet id o_btn_ff;       /* btn fast forward     */
     IBOutlet id o_btn_next;     /* btn next       */
-    IBOutlet id o_btn_prefs;    /* btn prefs      */
+    IBOutlet id o_btn_fullscreen;/* btn fullscreen      */
 
     NSImage * o_img_play;       /* btn play img   */
     NSImage * o_img_pause;      /* btn pause img  */
+    NSImage * o_img_play_pressed;       /* btn play img   */
+    NSImage * o_img_pause_pressed;      /* btn pause img  */
 
     IBOutlet id o_controls;     /* VLCControls    */
     IBOutlet id o_playlist;     /* VLCPlaylist    */
-    IBOutlet id o_info;		/* VLCInfo        */
+    IBOutlet id o_info;         /* VLCInfo        */
 
     IBOutlet id o_messages;     /* messages tv    */
     IBOutlet id o_msgs_panel;   /* messages panel */
     NSMutableArray * o_msg_arr; /* messages array */
     NSLock * o_msg_lock;        /* messages lock  */
-    IBOutlet id o_msgs_btn_crashlog;	/* messages open crashlog */
+    IBOutlet id o_msgs_btn_crashlog;    /* messages open crashlog */
 
     IBOutlet id o_error;        /* error panel    */
     IBOutlet id o_err_msg;      /* NSTextView     */
@@ -129,13 +131,17 @@ struct intf_sys_t
     IBOutlet id o_err_bug_lbl;
     IBOutlet id o_err_btn_msgs; /* Open Messages  */
     IBOutlet id o_err_btn_dismiss;
-    
-    IBOutlet id o_info_window;	/* Info panel     */
+    IBOutlet id o_err_ckbk_surpress;
+
+    IBOutlet id o_info_window;  /* Info panel     */
 
     /* main menu */
 
     IBOutlet id o_mi_about;
     IBOutlet id o_mi_prefs;
+    IBOutlet id o_mi_add_intf;
+    IBOutlet id o_mu_add_intf;
+    IBOutlet id o_mi_services;
     IBOutlet id o_mi_hide;
     IBOutlet id o_mi_hide_others;
     IBOutlet id o_mi_show_all;
@@ -163,9 +169,15 @@ struct intf_sys_t
     IBOutlet id o_mi_slower;
     IBOutlet id o_mi_previous;
     IBOutlet id o_mi_next;
+    IBOutlet id o_mi_random;
+    IBOutlet id o_mi_repeat;
     IBOutlet id o_mi_loop;
     IBOutlet id o_mi_fwd;
     IBOutlet id o_mi_bwd;
+    IBOutlet id o_mi_fwd1m;
+    IBOutlet id o_mi_bwd1m;
+    IBOutlet id o_mi_fwd5m;
+    IBOutlet id o_mi_bwd5m;
     IBOutlet id o_mi_program;
     IBOutlet id o_mu_program;
     IBOutlet id o_mi_title;
@@ -183,6 +195,8 @@ struct intf_sys_t
     IBOutlet id o_mu_channels;
     IBOutlet id o_mi_device;
     IBOutlet id o_mu_device;
+    IBOutlet id o_mi_visual;
+    IBOutlet id o_mu_visual;
 
     IBOutlet id o_mu_video;
     IBOutlet id o_mi_half_window;
@@ -208,7 +222,7 @@ struct intf_sys_t
     IBOutlet id o_mi_info;
     IBOutlet id o_mi_messages;
     IBOutlet id o_mi_bring_atf;
-    
+
     IBOutlet id o_mu_help;
     IBOutlet id o_mi_readme;
     IBOutlet id o_mi_documentation;
@@ -221,15 +235,17 @@ struct intf_sys_t
     IBOutlet id o_dmi_stop;
     IBOutlet id o_dmi_next;
     IBOutlet id o_dmi_previous;
+    IBOutlet id o_dmi_mute;
 }
 
 - (id)getControls;
-
+- (id)getPlaylist;
+- (id)getInfo;
 - (void)terminate;
 
+- (void)initStrings;
+
 - (void)manage;
-- (void)manage:(playlist_t *)p_playlist;
-- (void)manageMode:(playlist_t *)p_playlist;
 - (void)manageIntf:(NSTimer *)o_timer;
 - (void)setupMenus;
 
