@@ -1,8 +1,8 @@
 /*****************************************************************************
- * open.m: MacOS X plugin for vlc
+ * open.m: MacOS X module for vlc
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: open.m,v 1.35 2003/05/25 17:27:13 massiot Exp $
+ * $Id: open.m 7090 2004-03-15 19:33:18Z bigben $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net> 
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -40,6 +40,7 @@
 #include "intf.h"
 #include "playlist.h"
 #include "open.h"
+#include "output.h"
 
 /*****************************************************************************
  * GetEjectableMediaOfClass 
@@ -66,7 +67,7 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
         return( nil );
     }
     
-    CFDictionarySetValue( classes_to_match, CFSTR( kIOMediaEjectable ), 
+    CFDictionarySetValue( classes_to_match, CFSTR( kIOMediaEjectableKey ), 
                           kCFBooleanTrue );
     
     kern_result = IOServiceGetMatchingServices( master_port, classes_to_match, 
@@ -88,7 +89,7 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
         do
         {
             str_bsd_path = IORegistryEntryCreateCFProperty( next_media,
-                                                            CFSTR( kIOBSDName ),
+                                                            CFSTR( kIOBSDNameKey ),
                                                             kCFAllocatorDefault,
                                                             0 );
             if( str_bsd_path == NULL )
@@ -133,7 +134,6 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
 
     [o_panel setTitle: _NS("Open Source")];
     [o_mrl_lbl setTitle: _NS("Media Resource Locator (MRL)")];
-    [o_ckbox_enqueue setTitle: _NS("Only enqueue in playlist, do not play")];
 
     [o_btn_ok setTitle: _NS("OK")];
     [o_btn_cancel setTitle: _NS("Cancel")];
@@ -230,30 +230,25 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
 
     if( i_result )
     {
-        intf_thread_t * p_intf = [NSApp getIntf];
-
-        NSString *o_source = [o_mrl stringValue];
-        BOOL b_enq = [o_ckbox_enqueue state] == NSOnState ? YES : NO;
-        NSString *subPath = [o_file_sub_path stringValue];
+        NSMutableDictionary *o_dic;
+        NSMutableArray *o_options = [NSMutableArray array];
+        o_dic = [NSMutableDictionary dictionaryWithObject: [o_mrl stringValue] forKey: @"ITEM_URL"];
         
-        if (([o_file_sub_ckbox state] == NSOnState) && !([subPath isEqualTo: @""]))
+        if( [o_file_sub_ckbox state] == NSOnState )
         {
-            config_PutPsz( p_intf, "sub-file", strdup( [subPath UTF8String] ) );
-            if ( [o_file_sub_override state] )
+            [o_options addObject: [NSString stringWithFormat: @"sub-file=%s", [[o_file_sub_path stringValue] UTF8String]]];
+            if( [o_file_sub_override state] == NSOnState )
             {
-                config_PutInt( p_intf, "sub-delay", (int)( [o_file_sub_delay intValue] * 10 ) );
-                config_PutFloat( p_intf, "sub-fps", [o_file_sub_fps floatValue] );
+                [o_options addObject: [NSString stringWithFormat: @"sub-delay=%i", (int)( [o_file_sub_delay intValue] * 10 )]];
+                [o_options addObject: [NSString stringWithFormat: @"sub-fps=%f", [o_file_sub_fps floatValue]]];
             }
         }
-        else
+        if( [o_output_ckbox state] == NSOnState )
         {
-            config_PutPsz( p_intf, "sub-file", "" );
-            config_PutInt( p_intf, "sub-delay", 0 );
-            config_PutFloat( p_intf, "sub-fps", 0.0 );
+            [o_options addObject: [NSString stringWithString: [(VLCOutput *)o_sout_options getMRL]]];
         }
-        
-        [o_playlist appendArray: 
-            [NSArray arrayWithObject: o_source] atPos: -1 enqueue:b_enq];
+        [o_dic setObject: (NSArray *)[o_options copy] forKey: @"ITEM_OPTIONS"];
+        [o_playlist appendArray: [NSArray arrayWithObject: o_dic] atPos: -1 enqueue:NO];
     }
 }
 
@@ -624,6 +619,7 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
 - (IBAction)openFile:(id)sender
 {
     NSOpenPanel *o_open_panel = [NSOpenPanel openPanel];
+    int i;
     
     [o_open_panel setAllowsMultipleSelection: YES];
     [o_open_panel setCanChooseDirectories: YES];
@@ -633,18 +629,17 @@ NSArray *GetEjectableMediaOfClass( const char *psz_class )
     if( [o_open_panel runModalForDirectory: nil
             file: nil types: nil] == NSOKButton )
     {
-        intf_thread_t * p_intf = [NSApp getIntf];
-        config_PutPsz( p_intf, "sout", NULL );
-        
+        NSArray *o_array = [NSArray array];
         NSArray *o_values = [[o_open_panel filenames]
                 sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
-        
-        config_PutPsz( p_intf, "sub-file", "" );
-        config_PutInt( p_intf, "sub-delay", 0 );
-        config_PutFloat( p_intf, "sub-fps", 0.0 );
-        config_PutPsz( p_intf, "sout", "" );
-        
-        [o_playlist appendArray: o_values atPos: -1 enqueue:NO];
+
+        for( i = 0; i < (int)[o_values count]; i++)
+        {
+            NSDictionary *o_dic;
+            o_dic = [NSDictionary dictionaryWithObject:[o_values objectAtIndex:i] forKey:@"ITEM_URL"];
+            o_array = [o_array arrayByAddingObject: o_dic];
+        }
+        [o_playlist appendArray: o_array atPos: -1 enqueue:NO];
     }
 }
 
