@@ -2,7 +2,7 @@
  * intf.cpp: Qt interface
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: intf.cpp,v 1.3 2002/09/30 11:05:39 sam Exp $
+ * $Id: intf.cpp 7519 2004-04-27 09:33:58Z sam $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -264,7 +264,7 @@ void IntfWindow::DateDisplay( int i_range )
 {
     if( p_intf->p_sys->p_input )
     {
-        char psz_time[ OFFSETTOTIME_MAX_SIZE ];
+        char psz_time[ MSTRTIME_MAX_SIZE ];
 
         vlc_mutex_lock( &p_intf->p_sys->p_input->stream.stream_lock );
         p_date->setText( input_OffsetToTime( p_intf->p_sys->p_input, psz_time,
@@ -299,7 +299,7 @@ void IntfWindow::FileOpen( void )
             return;
         }
 
-        playlist_Add( p_playlist, file.latin1(),
+        playlist_Add( p_playlist, file.latin1(), file.latin1(),
                       PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
         vlc_object_release( p_playlist );
     }
@@ -321,10 +321,10 @@ void IntfWindow::FileQuit( void )
 void IntfWindow::About( void )
 {
     QMessageBox::about( this, "About",
-        "VideoLAN Client\n"
-        "(C) 1996, 1997, 1998, 1999, 2000, 2001, 2002 - the VideoLAN Team\n"
+        "VLC media player\n"
+        "(C) 1996 - 2004 - the VideoLAN Team\n"
         "\n"
-        "This is the VideoLAN client, a DVD and MPEG player.\n"
+        "This is the VLC media player, a DVD and MPEG player.\n"
         "It can play MPEG and MPEG 2 files from a file "
             "or from a network source.\n"
         "\n"
