@@ -2,7 +2,7 @@
  * os_bitmap.h: Wrapper for the Bitmap class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: os_bitmap.h,v 1.1 2003/03/18 02:21:47 ipkiss Exp $
+ * $Id: os_bitmap.h 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -25,8 +25,11 @@
 
 
 #if defined( WIN32 )
-    #include "win32_bitmap.h"
+    #include "win32/win32_bitmap.h"
     #define OSBitmap Win32Bitmap
+#elif defined X11_SKINS
+    #include "x11/x11_bitmap.h"
+    #define OSBitmap X11Bitmap
 #endif
 
 
