@@ -2,7 +2,7 @@
  * os_window.h: Wrapper for the OSWindow class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: os_window.h,v 1.1 2003/03/18 02:21:47 ipkiss Exp $
+ * $Id: os_window.h 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -25,7 +25,11 @@
 
 
 #if defined( WIN32 )
-    #include "win32_dragdrop.h"
-    #include "win32_window.h"
+    #include "win32/win32_dragdrop.h"
+    #include "win32/win32_window.h"
     #define OSWindow Win32Window
+#elif defined X11_SKINS
+    #include "x11/x11_dragdrop.h"
+    #include "x11/x11_window.h"
+    #define OSWindow X11Window
 #endif
