@@ -2,7 +2,7 @@
  * event.cpp: Event class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: event.cpp,v 1.20 2003/07/20 20:42:23 ipkiss Exp $
+ * $Id: event.cpp 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -112,8 +112,12 @@ unsigned int Event::GetMessageType( string Desc )
         return VLC_HIDE;
     else if( Desc == "VLC_OPEN" )
         return VLC_OPEN;
+    else if( Desc == "VLC_NET" )
+        return VLC_NET;
     else if( Desc == "VLC_LOAD_SKIN" )
         return VLC_LOAD_SKIN;
+    else if( Desc == "VLC_ON_TOP" )
+        return VLC_ON_TOP;
     else if( Desc == "VLC_CHANGE_TRAY" )
         return VLC_CHANGE_TRAY;
     else if( Desc == "VLC_CHANGE_TASKBAR" )
@@ -142,6 +146,10 @@ unsigned int Event::GetMessageType( string Desc )
         return VLC_STREAM_TITLE;
     else if( Desc == "VLC_HELP_TEXT" )
         return VLC_HELP_TEXT;
+    else if( Desc == "VLC_SLOWER" )
+        return VLC_SLOWER;
+    else if( Desc == "VLC_FASTER" )
+        return VLC_FASTER;
 
     // Volume control
     else if( Desc == "VLC_VOLUME_CHANGE" )
@@ -269,7 +277,7 @@ void Event::CreateEvent()
             else if( strcmp( para1, "SET" ) == 0 )
             {
                 Param1 = VLC_VOLUME_SET;
-                Param2 = atoi( para2 ) * AOUT_VOLUME_MAX / 100;
+                Param2 = atoi( para2 ) * (AOUT_VOLUME_MAX * 2) / 100;
             }
             break;
 
