@@ -2,7 +2,7 @@
  * vlcproc.cpp: VlcProc class
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: vlcproc.cpp,v 1.3 2003/03/20 09:29:07 karibu Exp $
+ * $Id: vlcproc.cpp 7209 2004-03-31 20:52:31Z gbazin $
  *
  * Authors: Olivier Teulire <ipkiss@via.ecp.fr>
  *          Emmanuel Puig    <karibu@via.ecp.fr>
@@ -23,28 +23,23 @@
  * USA.
  *****************************************************************************/
 
-
 //--- VLC -------------------------------------------------------------------
 #include <vlc/vlc.h>
 #include <vlc/intf.h>
 #include <vlc/aout.h>
 #include <vlc/vout.h>
 
-
 //--- SKIN ------------------------------------------------------------------
-#include "os_api.h"
-#include "dialog.h"
-#include "os_dialog.h"
+#include "../os_api.h"
 #include "event.h"
 #include "banks.h"
 #include "theme.h"
-#include "os_theme.h"
+#include "../os_theme.h"
 #include "themeloader.h"
 #include "window.h"
 #include "vlcproc.h"
 #include "skin_common.h"
-
-
+#include "dialogs.h"
 
 //---------------------------------------------------------------------------
 // VlcProc
@@ -52,6 +47,32 @@
 VlcProc::VlcProc( intf_thread_t *_p_intf )
 {
     p_intf = _p_intf;
+
+    playlist_t *p_playlist = (playlist_t *)vlc_object_find( p_intf,
+        VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+    if( p_playlist != NULL )
+    {
+        // We want to be notified of playlist changes
+        var_AddCallback( p_playlist, "intf-change", RefreshCallback, this );
+
+        // Raise/lower interface with middle click on vout
+        var_AddCallback( p_playlist, "intf-show", IntfShowCallback, this );
+
+        vlc_object_release( p_playlist );
+    }
+}
+//---------------------------------------------------------------------------
+VlcProc::~VlcProc()
+{
+    // Remove the refresh callback
+    playlist_t *p_playlist = (playlist_t *)vlc_object_find( p_intf,
+        VLC_OBJECT_PLAYLIST, FIND_ANYWHERE );
+    if( p_playlist != NULL )
+    {
+        var_DelCallback( p_playlist, "intf-change", RefreshCallback, this );
+        var_DelCallback( p_playlist, "intf-show", IntfShowCallback, this );
+        vlc_object_release( p_playlist );
+    }
 }
 //---------------------------------------------------------------------------
 bool VlcProc::EventProc( Event *evt )
@@ -71,7 +92,7 @@ bool VlcProc::EventProc( Event *evt )
             return true;
 
         case VLC_HIDE:
-            for( list<Window *>::const_iterator win =
+            for( list<SkinWindow *>::const_iterator win =
                     p_intf->p_sys->p_theme->WindowList.begin();
                  win != p_intf->p_sys->p_theme->WindowList.end(); win++ )
             {
@@ -82,7 +103,7 @@ bool VlcProc::EventProc( Event *evt )
             return true;
 
         case VLC_SHOW:
-            for( list<Window *>::const_iterator win =
+            for( list<SkinWindow *>::const_iterator win =
                     p_intf->p_sys->p_theme->WindowList.begin();
                  win != p_intf->p_sys->p_theme->WindowList.end(); win++ )
             {
@@ -93,15 +114,32 @@ bool VlcProc::EventProc( Event *evt )
             return true;
 
         case VLC_OPEN:
-            OpenFile( true );
+            p_intf->p_sys->p_dialogs->ShowOpen( true );
+            InterfaceRefresh();
+            return true;
+
+        case VLC_NET:
+            p_intf->p_sys->p_dialogs->ShowNet();
+            InterfaceRefresh();
             return true;
 
         case VLC_LOAD_SKIN:
             LoadSkin();
             return true;
 
+        case VLC_ON_TOP:
+            for( list<SkinWindow *>::const_iterator win =
+                    p_intf->p_sys->p_theme->WindowList.begin();
+                win != p_intf->p_sys->p_theme->WindowList.end(); win++ )
+            {
+                (*win)->ToggleOnTop();
+            }
+            // Set the indicator to the new state
+            p_intf->p_sys->b_on_top = !p_intf->p_sys->b_on_top;
+            return true;
+
         case VLC_DROP:
-            DropFile( evt->GetParam1() );
+            DropFile( evt->GetParam1(), evt->GetParam2() );
             return true;
 
         case VLC_PLAY:
@@ -124,20 +162,33 @@ bool VlcProc::EventProc( Event *evt )
             PrevStream();
             return true;
 
+        case VLC_SLOWER:
+            SlowStream();
+            return true;
+
+        case VLC_FASTER:
+            FastStream();
+            return true;
+
         case VLC_PLAYLIST_ADD_FILE:
-            OpenFile( false );
+            p_intf->p_sys->p_dialogs->ShowOpen( false );
+            InterfaceRefresh();
             return true;
 
         case VLC_LOG_SHOW:
-            p_intf->p_sys->p_theme->ShowLog( evt->GetParam2() );
+            p_intf->p_sys->p_dialogs->ShowMessages();
             return true;
 
-        case VLC_LOG_CLEAR:
-            p_intf->p_sys->p_theme->ClearLog();
+        case VLC_PREFS_SHOW:
+            p_intf->p_sys->p_dialogs->ShowPrefs();
+            return true;
+
+        case VLC_INFO_SHOW:
+            p_intf->p_sys->p_dialogs->ShowFileInfo();
             return true;
 
         case VLC_INTF_REFRESH:
-            InterfaceRefresh( (bool)evt->GetParam2() );
+            InterfaceRefresh();
             return true;
 
         case VLC_TEST_ALL_CLOSED:
@@ -164,7 +215,7 @@ bool VlcProc::EventProcEnd()
     if( p_intf->p_sys->b_all_win_closed )
         return true;
 
-    list<Window *>::const_iterator win;
+    list<SkinWindow *>::const_iterator win;
 
     // If a window has been closed, test if all are closed !
     for( win = p_intf->p_sys->p_theme->WindowList.begin();
@@ -213,18 +264,42 @@ bool VlcProc::IsClosing()
 //---------------------------------------------------------------------------
 // Private methods
 //---------------------------------------------------------------------------
-void VlcProc::InterfaceRefresh( bool All )
+
+// Refresh callback
+int VlcProc::RefreshCallback( vlc_object_t *p_this, const char *psz_variable,
+        vlc_value_t old_val, vlc_value_t new_val, void *param )
+{
+    ( (VlcProc*)param )->InterfaceRefresh();
+    return VLC_SUCCESS;
+}
+
+// Interface show/hide callback
+int VlcProc::IntfShowCallback( vlc_object_t *p_this, const char *psz_variable,
+        vlc_value_t old_val, vlc_value_t new_val, void *param )
+{
+    if( new_val.b_bool == VLC_TRUE )
+    {
+        OSAPI_PostMessage( NULL, VLC_SHOW, 1, 0 );
+    }
+    else
+    {
+        OSAPI_PostMessage( NULL, VLC_HIDE, 1, 0 );
+    }
+    return VLC_SUCCESS;
+}
+
+void VlcProc::InterfaceRefresh()
 {
     // Shortcut pointers
     intf_sys_t  *Sys      = p_intf->p_sys;
     Theme       *Thema    = Sys->p_theme;
     playlist_t  *PlayList = Sys->p_playlist;
 
-
     // Refresh
-    if( PlayList != NULL && Sys->p_input != NULL &&
-        Sys->p_input->stream.b_seekable )
+    if( PlayList != NULL )
     {
+        vlc_mutex_lock( &PlayList->object_lock );
+
         // Refresh stream control controls ! :)
         switch( PlayList->i_status )
         {
@@ -259,12 +334,18 @@ void VlcProc::InterfaceRefresh( bool All )
         else
             EnabledEvent( "next", true );
 
-
-        // Update file name text
-        if( PlayList->i_index != Sys->i_index )
+        // Update file name
+        if( PlayList->i_index >= 0 && PlayList->i_index != Sys->i_index )
         {
+            string long_name = PlayList->pp_items[PlayList->i_index]->input.psz_name;
+            int pos = long_name.rfind( DIRECTORY_SEPARATOR, long_name.size() );
+
+            // Complete file name
             Thema->EvtBank->Get( "file_name" )->PostTextMessage(
-                PlayList->pp_items[PlayList->i_index]->psz_name );
+                PlayList->pp_items[PlayList->i_index]->input.psz_name );
+            // File name without path
+            Thema->EvtBank->Get( "title" )->PostTextMessage(
+                PlayList->pp_items[PlayList->i_index]->input.psz_name + pos + 1 );
         }
 
         // Update playlists
@@ -275,6 +356,8 @@ void VlcProc::InterfaceRefresh( bool All )
             Sys->i_size  = PlayList->i_size;
             Sys->i_index = PlayList->i_index;
         }
+
+        vlc_mutex_unlock( &PlayList->object_lock );
     }
     else
     {
@@ -292,18 +375,22 @@ void VlcProc::InterfaceRefresh( bool All )
             Sys->i_size  = 0;
         }
     }
-
 }
 //---------------------------------------------------------------------------
 void VlcProc::EnabledEvent( string type, bool state )
 {
     OSAPI_PostMessage( NULL, CTRL_ENABLED, (unsigned int)
         p_intf->p_sys->p_theme->EvtBank->Get( type ), (int)state );
+
+    if( !state )
+    {
+        OSAPI_PostMessage( NULL, CTRL_SYNCHRO, (unsigned int)
+            p_intf->p_sys->p_theme->EvtBank->Get( type ), (int)false );
+    }
 }
 //---------------------------------------------------------------------------
 
 
-
 //---------------------------------------------------------------------------
 // Common VLC procedures
 //---------------------------------------------------------------------------
@@ -311,29 +398,7 @@ void VlcProc::LoadSkin()
 {
     if( p_intf->p_sys->p_new_theme_file == NULL )
     {
-        // Initialize file structure
-        OpenFileDialog *OpenFile;
-        OpenFile = (OpenFileDialog *)new OSOpenFileDialog( p_intf,
-            _("Change skin - Open new file"), false );
-        OpenFile->AddFilter( _("Skin files"), "*.vlt" );
-        OpenFile->AddFilter( _("Skin files"), "*.xml" );
-        OpenFile->AddFilter( _("All files"), "*.*" );
-
-        // Open dialog box
-        if( OpenFile->Open() )
-        {
-            p_intf->p_sys->p_new_theme_file =
-                new char[OpenFile->FileList.front().length()];
-
-            strcpy( p_intf->p_sys->p_new_theme_file,
-                    OpenFile->FileList.front().c_str() );
-
-            // Tell vlc to change skin after hiding interface
-            OSAPI_PostMessage( NULL, VLC_HIDE, VLC_LOAD_SKIN, 0 );
-        }
-
-        // Free memory
-        delete OpenFile;
+        p_intf->p_sys->p_dialogs->ShowOpenSkin( 0 /*none blocking*/ );
     }
     else
     {
@@ -368,76 +433,32 @@ void VlcProc::LoadSkin()
         delete Loader;
 
         // Uninitialize new theme
-        delete (char *)p_intf->p_sys->p_new_theme_file;
+        free( p_intf->p_sys->p_new_theme_file );
         p_intf->p_sys->p_new_theme_file = NULL;
+
+        OSAPI_PostMessage( NULL, VLC_INTF_REFRESH, 0, (int)true );
     }
 }
 //---------------------------------------------------------------------------
-void VlcProc::OpenFile( bool play )
-{
-    // Initialize file structure
-    OpenFileDialog *OpenFile;
-    if( play )
-    {
-        OpenFile = (OpenFileDialog *)new OSOpenFileDialog( p_intf,
-            _("Open file"), false );
-    }
-    else
-    {
-        OpenFile = (OpenFileDialog *)new OSOpenFileDialog( p_intf,
-            _("Add file"), true );
-    }
-    OpenFile->AddFilter( _("All files"), "*.*" );
 
-    // Check if palylist is available
-    playlist_t *p_playlist = p_intf->p_sys->p_playlist;
-    if( p_playlist == NULL )
-        return;
-
-    // Open dialog box and launch file
-    if( OpenFile->Open() )
-    {
-        list<string>::const_iterator file;
-        for( file = OpenFile->FileList.begin();
-             file != OpenFile->FileList.end(); file++ )
-        {
-            playlist_Add( p_playlist, (*file).c_str(), PLAYLIST_APPEND,
-                          PLAYLIST_END );
-        }
-
-        // Refreshing
-        if( play )
-        {
-            // Play
-            p_intf->p_sys->p_theme->EvtBank->Get( "play" )->SendEvent();
-        }
-
-        // Refresh interface !
-        p_intf->p_sys->p_theme->EvtBank->Get( "playlist_refresh" )
-            ->PostSynchroMessage();
-        InterfaceRefresh();
-    }
-
-    // Free memory
-    delete OpenFile;
-}
-//---------------------------------------------------------------------------
-void VlcProc::DropFile( unsigned int param )
+void VlcProc::DropFile( unsigned int param1, long param2 )
 {
     // Get pointer to file
-    char *FileName = (char *)param;
+    char *FileName = (char *)param1;
 
     // Add the new file to the playlist
     if( p_intf->p_sys->p_playlist != NULL )
     {
-        if( config_GetInt( p_intf, "enqueue" ) )
+        if( param2 == 0 )
         {
-            playlist_Add( p_intf->p_sys->p_playlist, FileName,
+            // Enqueue the item
+            playlist_Add( p_intf->p_sys->p_playlist, FileName, FileName,
                           PLAYLIST_APPEND, PLAYLIST_END );
         }
         else
         {
-            playlist_Add( p_intf->p_sys->p_playlist, FileName,
+            // Enqueue and play the item
+            playlist_Add( p_intf->p_sys->p_playlist, FileName, FileName,
                           PLAYLIST_APPEND | PLAYLIST_GO, PLAYLIST_END );
         }
     }
@@ -449,7 +470,6 @@ void VlcProc::DropFile( unsigned int param )
 
     // Refresh interface
     InterfaceRefresh();
-
 }
 //---------------------------------------------------------------------------
 
@@ -461,9 +481,10 @@ void VlcProc::DropFile( unsigned int param )
 //---------------------------------------------------------------------------
 void VlcProc::PauseStream()
 {
-    if( p_intf->p_sys->p_input == NULL )
+    if( p_intf->p_sys->p_playlist == NULL )
         return;
-    input_SetStatus( p_intf->p_sys->p_input, INPUT_STATUS_PAUSE );
+
+    playlist_Command( p_intf->p_sys->p_playlist, PLAYLIST_PAUSE, 0 );
 
     // Refresh interface
     InterfaceRefresh();
@@ -473,8 +494,13 @@ void VlcProc::PlayStream()
 {
     if( p_intf->p_sys->p_playlist == NULL )
         return;
+
     if( !p_intf->p_sys->p_playlist->i_size )
+    {
+        p_intf->p_sys->p_dialogs->ShowOpen( true );
+        InterfaceRefresh();
         return;
+    }
 
     playlist_Play( p_intf->p_sys->p_playlist );
 
@@ -514,6 +540,34 @@ void VlcProc::PrevStream()
     InterfaceRefresh();
 }
 //---------------------------------------------------------------------------
+void VlcProc::SlowStream()
+{
+    input_thread_t *p_input =
+        (input_thread_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                           FIND_ANYWHERE );
+    if( p_input )
+    {
+        vlc_value_t val; val.b_bool = VLC_TRUE;
+
+        var_Set( p_input, "rate-slower", val );
+        vlc_object_release( p_input );
+    }
+}
+//---------------------------------------------------------------------------
+void VlcProc::FastStream()
+{
+    input_thread_t *p_input =
+        (input_thread_t *)vlc_object_find( p_intf, VLC_OBJECT_INPUT,
+                                           FIND_ANYWHERE );
+    if( p_input )
+    {
+        vlc_value_t val; val.b_bool = VLC_TRUE;
+
+        var_Set( p_input, "rate-faster", val );
+        vlc_object_release( p_input );
+    }
+}
+//---------------------------------------------------------------------------
 void VlcProc::MoveStream( long Pos )
 {
     if( p_intf->p_sys->p_input == NULL )
@@ -572,14 +626,10 @@ void VlcProc::ChangeVolume( unsigned int msg, long param )
             aout_VolumeDown( p_intf, 1, NULL );
             break;
         case VLC_VOLUME_SET:
-            aout_VolumeSet( p_intf, param * AOUT_VOLUME_MAX / SLIDER_RANGE );
+            aout_VolumeSet( p_intf, param * (AOUT_VOLUME_DEFAULT * 2) /
+                            SLIDER_RANGE );
             break;
     }
     aout_VolumeGet( p_intf, &volume );
-
-    PostMessage( NULL, CTRL_SET_SLIDER,
-        (unsigned int)
-            p_intf->p_sys->p_theme->EvtBank->Get( "volume_refresh" ),
-        (int)( volume * SLIDER_RANGE / AOUT_VOLUME_MAX ) );
 }
 //---------------------------------------------------------------------------
