@@ -1,8 +1,8 @@
 /*****************************************************************************
  * fileinfo.cpp : wxWindows plugin for vlc
  *****************************************************************************
- * Copyright (C) 2000-2001 VideoLAN
- * $Id: fileinfo.cpp,v 1.17 2003/07/12 13:33:10 gbazin Exp $
+ * Copyright (C) 2000-2004 VideoLAN
+ * $Id: fileinfo.cpp 7209 2004-03-31 20:52:31Z gbazin $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -24,31 +24,9 @@
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
-#include <stdlib.h>                                      /* malloc(), free() */
-#include <errno.h>                                                 /* ENOMEM */
-#include <string.h>                                            /* strerror() */
-#include <stdio.h>
-
 #include <vlc/vlc.h>
-
-#ifdef WIN32                                                 /* mingw32 hack */
-#undef Yield
-#undef CreateDialog
-#endif
-
-/* Let vlc take care of the i18n stuff */
-#define WXINTL_NO_GETTEXT_MACRO
-
-#include <wx/wxprec.h>
-#include <wx/wx.h>
-#include <wx/treectrl.h>
-
 #include <vlc/intf.h>
 
-#if defined MODULE_NAME_IS_skins
-#   include "../skins/src/skin_common.h"
-#endif
-
 #include "wxwindows.h"
 
 /*****************************************************************************
@@ -74,7 +52,7 @@ END_EVENT_TABLE()
  * Constructor.
  *****************************************************************************/
 FileInfo::FileInfo( intf_thread_t *_p_intf, wxWindow *p_parent ):
-    wxFrame( p_parent, -1, wxU(_("FileInfo")), wxDefaultPosition,
+    wxFrame( p_parent, -1, wxU(_("Stream and media info")), wxDefaultPosition,
              wxDefaultSize, wxDEFAULT_FRAME_STYLE )
 {
     /* Initializations */
@@ -92,18 +70,10 @@ FileInfo::FileInfo( intf_thread_t *_p_intf, wxWindow *p_parent ):
 
     fileinfo_root_label = wxT("");
 
-    /* Create the OK button */
-    wxButton *ok_button = new wxButton( panel, wxID_OK, wxU(_("Close")) );
-    ok_button->SetDefault();
-
     /* Place everything in sizers */
-    wxBoxSizer *ok_button_sizer = new wxBoxSizer( wxHORIZONTAL );
-    ok_button_sizer->Add( ok_button, 0, wxALL, 5 );
-    ok_button_sizer->Layout();
     wxBoxSizer *main_sizer = new wxBoxSizer( wxVERTICAL );
     wxBoxSizer *panel_sizer = new wxBoxSizer( wxVERTICAL );
     panel_sizer->Add( fileinfo_tree, 1, wxEXPAND | wxALL, 5 );
-    panel_sizer->Add( ok_button_sizer, 0, wxALIGN_CENTRE );
     panel_sizer->Layout();
     panel->SetSizerAndFit( panel_sizer );
     main_sizer->Add( panel, 1, wxEXPAND, 0 );
@@ -133,38 +103,35 @@ void FileInfo::UpdateFileInfo()
          * retrieved with the GetItemText() method, but it doesn't work on
          * Windows when the wxTR_HIDE_ROOT style is set. That's why we need to
          * use the fileinfo_root_label variable... */
-        fileinfo_root = fileinfo_tree->AddRoot( wxU(p_input->psz_name) );
-        fileinfo_root_label = wxU(p_input->psz_name);
+        fileinfo_root = fileinfo_tree->AddRoot( wxL2U(p_input->psz_name) );
+        fileinfo_root_label = wxL2U(p_input->psz_name);
     }
-    else if( fileinfo_root_label == wxU(p_input->psz_name) )
+    else if( fileinfo_root_label == wxL2U(p_input->psz_name) )
     {
         return;
     }
 
     /* We rebuild the tree from scratch */
     fileinfo_tree->DeleteChildren( fileinfo_root );
-    fileinfo_root_label = wxU(p_input->psz_name);
+    fileinfo_root_label = wxL2U(p_input->psz_name);
 
-    vlc_mutex_lock( &p_input->stream.stream_lock );
-
-    input_info_category_t *p_cat = p_input->stream.p_info;
-
-    while( p_cat )
+    vlc_mutex_lock( &p_input->p_item->lock );
+    for( int i = 0; i < p_input->p_item->i_categories; i++ )
     {
+        info_category_t *p_cat = p_input->p_item->pp_categories[i];
+
         wxTreeItemId cat = fileinfo_tree->AppendItem( fileinfo_root,
-                                                      wxU(p_cat->psz_name) );
-        input_info_t *p_info = p_cat->p_info;
-        while( p_info )
+                                                      wxL2U(p_cat->psz_name) );
+        for( int j = 0; j < p_cat->i_infos; j++ )
         {
-            fileinfo_tree->AppendItem( cat, (wxString)wxU(p_info->psz_name) +
-                                       wxT(": ") + wxU(p_info->psz_value) );
-            p_info = p_info->p_next;
+            info_t *p_info = p_cat->pp_infos[j];
+
+            fileinfo_tree->AppendItem( cat, (wxString)wxL2U(p_info->psz_name) +
+                                       wxT(": ") + wxL2U(p_info->psz_value) );
         }
-        p_cat = p_cat->p_next;
         fileinfo_tree->Expand( cat );
     }
-
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
+    vlc_mutex_unlock( &p_input->p_item->lock );
 
     return;
 }
