@@ -1,8 +1,8 @@
 /*****************************************************************************
  * streamout.cpp : wxWindows plugin for vlc
  *****************************************************************************
- * Copyright (C) 2000-2001 VideoLAN
- * $Id: streamout.cpp,v 1.41 2003/12/22 02:24:52 sam Exp $
+ * Copyright (C) 2000-2004 VideoLAN
+ * $Id: streamout.cpp 7681 2004-05-16 10:46:54Z zorglub $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -33,10 +33,7 @@
 #include <vlc/intf.h>
 
 #include "wxwindows.h"
-#include <wx/notebook.h>
-#include <wx/textctrl.h>
 #include <wx/combobox.h>
-#include <wx/spinctrl.h>
 #include <wx/statline.h>
 
 #ifndef wxRB_SINGLE
@@ -360,17 +357,26 @@ void SoutDialog::UpdateMRL()
         /* SAP only if UDP */
         if( sap_checkbox->IsChecked() )
         {
-            dup_opts += wxT(",sap=\"");
-            dup_opts += announce_addr->GetLineText(0);
-            dup_opts += wxT("\"");
+            dup_opts += wxT(",sap");
+            if( ! announce_addr->GetLineText(0).IsEmpty() )
+            {
+                dup_opts += wxT(",name=\"");
+                dup_opts += announce_addr->GetLineText(0);
+                dup_opts += wxT("\"");
+            }
         }
 
         /* SLP only if UDP */
         if( slp_checkbox->IsChecked() )
         {
-            dup_opts += wxT(",slp=\"");
-            dup_opts += announce_addr->GetLineText(0);
-            dup_opts += wxT("\"");
+            dup_opts += wxT(",slp");
+            if( ! sap_checkbox->IsChecked() &&
+                ! announce_addr->GetLineText(0).IsEmpty() )
+            {
+                dup_opts += wxT(",name=\"");
+                dup_opts += announce_addr->GetLineText(0);
+                dup_opts += wxT("\"");
+            }
         }
 
         dup_opts += wxT("}");
@@ -699,6 +705,7 @@ wxPanel *SoutDialog::TranscodingPanel( wxWindow* parent )
     {
         wxT("mpga"),
         wxT("mp3"),
+        wxT("mp4a"),
         wxT("a52"),
         wxT("vorb"),
         wxT("flac"),
