@@ -2,7 +2,7 @@
  * display.c: display stream output module
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: display.c,v 1.12 2004/01/25 14:34:25 gbazin Exp $
+ * $Id: display.c 7468 2004-04-24 12:49:53Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -31,28 +31,46 @@
 #include <vlc/input.h>
 #include <vlc/sout.h>
 
-#include "codecs.h"
-
-/*****************************************************************************
- * Exported prototypes
- *****************************************************************************/
-static int      Open    ( vlc_object_t * );
-static void     Close   ( vlc_object_t * );
-
-static sout_stream_id_t *Add ( sout_stream_t *, es_format_t * );
-static int               Del ( sout_stream_t *, sout_stream_id_t * );
-static int               Send( sout_stream_t *, sout_stream_id_t *, sout_buffer_t* );
-
 /*****************************************************************************
  * Module descriptor
  *****************************************************************************/
+#define AUDIO_TEXT N_("Enable audio")
+#define AUDIO_LONGTEXT N_( "Enable/disable audio rendering." )
+#define VIDEO_TEXT N_("Enable video")
+#define VIDEO_LONGTEXT N_( "Enable/disable video rendering." )
+#define DELAY_TEXT N_("Delay")
+#define DELAY_LONGTEXT N_( "Introduces a delay in the display of the stream." )
+
+static int  Open ( vlc_object_t * );
+static void Close( vlc_object_t * );
+
+#define SOUT_CFG_PREFIX "sout-display-"
+
 vlc_module_begin();
     set_description( _("Display stream output") );
     set_capability( "sout stream", 50 );
     add_shortcut( "display" );
+    add_bool( SOUT_CFG_PREFIX "audio", 1, NULL, AUDIO_TEXT,
+              AUDIO_LONGTEXT, VLC_TRUE );
+    add_bool( SOUT_CFG_PREFIX "video", 1, NULL, VIDEO_TEXT,
+              VIDEO_LONGTEXT, VLC_TRUE );
+    add_integer( SOUT_CFG_PREFIX "delay", 100, NULL, DELAY_TEXT,
+                 DELAY_LONGTEXT, VLC_TRUE );
     set_callbacks( Open, Close );
 vlc_module_end();
 
+
+/*****************************************************************************
+ * Exported prototypes
+ *****************************************************************************/
+static const char *ppsz_sout_options[] = {
+    "audio", "video", "delay", NULL
+};
+
+static sout_stream_id_t *Add ( sout_stream_t *, es_format_t * );
+static int               Del ( sout_stream_t *, sout_stream_id_t * );
+static int               Send( sout_stream_t *, sout_stream_id_t *, block_t* );
+
 struct sout_stream_sys_t
 {
     input_thread_t *p_input;
@@ -70,9 +88,14 @@ static int Open( vlc_object_t *p_this )
 {
     sout_stream_t     *p_stream = (sout_stream_t*)p_this;
     sout_stream_sys_t *p_sys;
-    char              *val;
-    p_sys           = malloc( sizeof( sout_stream_sys_t ) );
-    p_sys->p_input  = vlc_object_find( p_stream, VLC_OBJECT_INPUT, FIND_ANYWHERE );
+    vlc_value_t val;
+
+    sout_ParseCfg( p_stream, SOUT_CFG_PREFIX, ppsz_sout_options,
+                   p_stream->p_cfg );
+
+    p_sys          = malloc( sizeof( sout_stream_sys_t ) );
+    p_sys->p_input = vlc_object_find( p_stream, VLC_OBJECT_INPUT,
+                                      FIND_ANYWHERE );
     if( !p_sys->p_input )
     {
         msg_Err( p_stream, "cannot find p_input" );
@@ -80,21 +103,14 @@ static int Open( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-    p_sys->b_audio = VLC_TRUE;
-    p_sys->b_video = VLC_TRUE;
-    p_sys->i_delay = 100*1000;
-    if( sout_cfg_find( p_stream->p_cfg, "noaudio" ) )
-    {
-        p_sys->b_audio = VLC_FALSE;
-    }
-    if( sout_cfg_find( p_stream->p_cfg, "novideo" ) )
-    {
-        p_sys->b_video = VLC_FALSE;
-    }
-    if( ( val = sout_cfg_find_value( p_stream->p_cfg, "delay" ) ) )
-    {
-        p_sys->i_delay = (mtime_t)atoi( val ) * (mtime_t)1000;
-    }
+    var_Get( p_stream, SOUT_CFG_PREFIX "audio", &val );
+    p_sys->b_audio = val.b_bool;
+
+    var_Get( p_stream, SOUT_CFG_PREFIX "video", &val );
+    p_sys->b_video = val.b_bool;
+
+    var_Get( p_stream, SOUT_CFG_PREFIX "delay", &val );
+    p_sys->i_delay = (int64_t)val.i_int * 1000;
 
     p_stream->pf_add    = Add;
     p_stream->pf_del    = Del;
@@ -102,6 +118,9 @@ static int Open( vlc_object_t *p_this )
 
     p_stream->p_sys     = p_sys;
 
+    /* update p_sout->i_out_pace_nocontrol */
+    p_stream->p_sout->i_out_pace_nocontrol++;
+
     return VLC_SUCCESS;
 }
 
@@ -113,6 +132,9 @@ static void Close( vlc_object_t * p_this )
     sout_stream_t     *p_stream = (sout_stream_t*)p_this;
     sout_stream_sys_t *p_sys = p_stream->p_sys;
 
+    /* update p_sout->i_out_pace_nocontrol */
+    p_stream->p_sout->i_out_pace_nocontrol--;
+
     vlc_object_release( p_sys->p_input );
 
     free( p_sys );
@@ -136,38 +158,22 @@ static sout_stream_id_t * Add( sout_stream_t *p_stream, es_format_t *p_fmt )
 
     id = malloc( sizeof( sout_stream_id_t ) );
 
-    vlc_mutex_lock( &p_sys->p_input->stream.stream_lock );
-    id->p_es = input_AddES( p_sys->p_input,
-                            NULL,           /* no program */
-                            12,             /* es_id */
-                            p_fmt->i_cat,   /* es category */
-                            NULL,           /* description */
-                            0 );            /* no extra data */
-
-    if( !id->p_es )
-    {
-        vlc_mutex_unlock( &p_sys->p_input->stream.stream_lock );
-
-        msg_Err( p_stream, "cannot create es" );
-        free( id );
-        return NULL;
-    }
-    id->p_es->i_stream_id   = 1;
+    id->p_es = malloc( sizeof( es_descriptor_t ) );
+    memset( id->p_es, 0, sizeof( es_descriptor_t ) );
+    id->p_es->i_cat         = p_fmt->i_cat;
     id->p_es->i_fourcc      = p_fmt->i_codec;
     id->p_es->b_force_decoder = VLC_TRUE;
-
     es_format_Copy( &id->p_es->fmt, p_fmt );
 
-    if( input_SelectES( p_sys->p_input, id->p_es ) )
+    id->p_es->p_dec = input_RunDecoder( p_sys->p_input, id->p_es );
+    if( id->p_es->p_dec == NULL )
     {
-        input_DelES( p_sys->p_input, id->p_es );
-        vlc_mutex_unlock( &p_sys->p_input->stream.stream_lock );
-
-        msg_Err( p_stream, "cannot select es" );
+        msg_Err( p_stream, "cannot create decoder for fcc=`%4.4s'",
+                 (char*)&p_fmt->i_codec );
+        free( id->p_es );
         free( id );
         return NULL;
     }
-    vlc_mutex_unlock( &p_sys->p_input->stream.stream_lock );
 
     return id;
 }
@@ -176,42 +182,40 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
 
-    input_DelES( p_sys->p_input, id->p_es );
+    input_EndDecoder( p_sys->p_input, id->p_es );
 
+    free( id->p_es );
     free( id );
 
     return VLC_SUCCESS;
 }
 
 static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
-                 sout_buffer_t *p_buffer )
+                 block_t *p_buffer )
 {
     sout_stream_sys_t *p_sys = p_stream->p_sys;
 
     while( p_buffer )
     {
-        sout_buffer_t *p_next;
-        block_t *p_block;
+        block_t *p_next = p_buffer->p_next;
 
-        vlc_mutex_lock( &p_sys->p_input->stream.stream_lock );
-        if( id->p_es->p_dec && p_buffer->i_size > 0 &&
-            (p_block = block_New( p_stream, p_buffer->i_size )) )
+        p_buffer->p_next = NULL;
+
+        if( id->p_es->p_dec && p_buffer->i_buffer > 0 )
         {
-            p_block->i_dts = p_buffer->i_dts <= 0 ? 0 :
-                             p_buffer->i_dts + p_sys->i_delay;
-            p_block->i_pts = p_buffer->i_pts <= 0 ? 0 :
-                             p_buffer->i_pts + p_sys->i_delay;
+            if( p_buffer->i_dts <= 0 )
+                p_buffer->i_dts= 0;
+            else
+                p_buffer->i_dts += p_sys->i_delay;
 
-            p_stream->p_vlc->pf_memcpy( p_block->p_buffer,
-                                        p_buffer->p_buffer, p_buffer->i_size );
+            if( p_buffer->i_pts <= 0 )
+                p_buffer->i_pts= 0;
+            else
+                p_buffer->i_pts += p_sys->i_delay;
 
-            input_DecodeBlock( id->p_es->p_dec, p_block );
+            input_DecodeBlock( id->p_es->p_dec, p_buffer );
         }
-        vlc_mutex_unlock( &p_sys->p_input->stream.stream_lock );
 
-        /* *** go to next buffer *** */
-        p_next = p_buffer->p_next;
-        sout_BufferDelete( p_stream->p_sout, p_buffer );
         p_buffer = p_next;
     }
 
