@@ -1,8 +1,8 @@
 /*****************************************************************************
  * logo.c : logo video plugin for vlc
  *****************************************************************************
- * Copyright (C) 2000, 2001, 2002, 2003 VideoLAN
- * $Id: logo.c,v 1.6 2003/12/22 02:24:53 sam Exp $
+ * Copyright (C) 2003-2004 VideoLAN
+ * $Id: logo.c 7453 2004-04-23 20:01:59Z gbazin $
  *
  * Authors: Simon Latapie <garf@videolan.org>
  *
@@ -61,17 +61,19 @@ static int MouseEvent( vlc_object_t *, char const *,
 #define POSX_LONGTEXT N_("You can move the logo by left-clicking on it" )
 #define POSY_TEXT N_("Y coordinate of the logo")
 #define POSY_LONGTEXT N_("You can move the logo by left-clicking on it" )
-#define TRANS_TEXT N_("transparency of the logo")
+#define TRANS_TEXT N_("Transparency of the logo (255-0)")
 #define TRANS_LONGTEXT N_("You can change it by middle-clicking and moving mouse left or right")
 
 vlc_module_begin();
-    add_category_hint( N_("logo"), NULL, VLC_FALSE );
-    add_file( "logo_file", NULL, NULL, FILE_TEXT, FILE_LONGTEXT, VLC_FALSE );
-    add_integer( "logo_x", 0, NULL, POSX_TEXT, POSX_LONGTEXT, VLC_FALSE );
-    add_integer( "logo_y", 0, NULL, POSY_TEXT, POSY_LONGTEXT, VLC_FALSE );
-    add_float( "logo_transparency", 1, NULL, TRANS_TEXT, TRANS_LONGTEXT, VLC_FALSE );
-    set_description( _("logo video filter") );
+    set_description( _("Logo video filter") );
     set_capability( "video filter", 0 );
+
+    add_file( "logo-file", NULL, NULL, FILE_TEXT, FILE_LONGTEXT, VLC_FALSE );
+    add_integer( "logo-x", 0, NULL, POSX_TEXT, POSX_LONGTEXT, VLC_FALSE );
+    add_integer( "logo-y", 0, NULL, POSY_TEXT, POSY_LONGTEXT, VLC_FALSE );
+    add_integer_with_range( "logo-transparency", 255, 0, 255, NULL,
+        TRANS_TEXT, TRANS_LONGTEXT, VLC_FALSE );
+
     add_shortcut( "logo" );
     set_callbacks( Create, Destroy );
 vlc_module_end();
@@ -98,6 +100,14 @@ struct vout_sys_t
     int trans;
 };
 
+/*****************************************************************************
+ * Control: control facility for the vout (forwards to child vout)
+ *****************************************************************************/
+static int Control( vout_thread_t *p_vout, int i_query, va_list args )
+{
+    return vout_vaControl( p_vout->p_sys->p_vout, i_query, args );
+}
+
 /*****************************************************************************
  * Create: allocates logo video thread output method
  *****************************************************************************
@@ -120,6 +130,7 @@ static int Create( vlc_object_t *p_this )
     p_vout->pf_manage = NULL;
     p_vout->pf_render = Render;
     p_vout->pf_display = NULL;
+    p_vout->pf_control = Control;
 
     return VLC_SUCCESS;
 }
@@ -150,7 +161,7 @@ static int Init( vout_thread_t *p_vout )
     int i_parity_height;
 
     /*  read png file  */
-    filename = config_GetPsz( p_vout, "logo_file" );
+    filename = config_GetPsz( p_vout, "logo-file" );
     fp = fopen( filename , "rb");
     png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL , NULL , NULL);
     info_ptr = png_create_info_struct(png_ptr);
@@ -279,9 +290,9 @@ static int Init( vout_thread_t *p_vout )
 
     ADD_PARENT_CALLBACKS( SendEventsToChild );
 
-    p_vout->p_sys->posx = config_GetInt( p_vout, "logo_x" );
-    p_vout->p_sys->posy = config_GetInt( p_vout, "logo_y" );
-    p_vout->p_sys->trans = (int)(config_GetFloat( p_vout, "logo_transparency" ) * 255);
+    p_vout->p_sys->posx = config_GetInt( p_vout, "logo-x" );
+    p_vout->p_sys->posy = config_GetInt( p_vout, "logo-y" );
+    p_vout->p_sys->trans = config_GetInt( p_vout, "logo-transparency");
 
     return VLC_SUCCESS;
 }
@@ -307,9 +318,8 @@ static void End( vout_thread_t *p_vout )
     vlc_object_detach( p_vout->p_sys->p_vout );
     vout_Destroy( p_vout->p_sys->p_vout );
 
-    config_PutInt( p_vout, "logo_x", p_vout->p_sys->posx );
-    config_PutInt( p_vout, "logo_y", p_vout->p_sys->posy );
-    config_PutFloat( p_vout, "logo_transparency", (float)(p_vout->p_sys->trans) / 255.0 );
+    config_PutInt( p_vout, "logo-x", p_vout->p_sys->posx );
+    config_PutInt( p_vout, "logo-y", p_vout->p_sys->posy );
 
     if (p_vout->p_sys->error == 0)
     {
@@ -391,7 +401,7 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
             } else
             {
                 p_out  = p_outpic->p[i_index].p_pixels +
-                         (p_vout->p_sys->posy/2)* p_outpic->p[i_index].i_pitch +
+                            p_vout->p_sys->posy / 2 * p_outpic->p[i_index].i_pitch +
                          p_vout->p_sys->posx / 2;
                 i_max = p_vout->p_sys->height / 2;
                 j_max = p_vout->p_sys->width / 2;
@@ -406,7 +416,7 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
             {
                 for( j = 0 ; j < j_max ; j++)
                 {
-                    *p_out = ( *p_out * ( 65025 - *p_in_a * tr) + *p_in * *p_in_a * tr) >> 16;
+                    *p_out = ( *p_out * ( 65025 - *p_in_a * tr) + *p_in * *p_in_a * tr ) >> 16;
                     p_out++;
                     p_in++;
                     p_in_a++;
