@@ -2,9 +2,9 @@
  * vout.h: Windows DirectX video output header file
  *****************************************************************************
  * Copyright (C) 2001-2004 VideoLAN
- * $Id: vout.h,v 1.12 2004/01/02 22:17:57 gbazin Exp $
+ * $Id: vout.h 7427 2004-04-22 15:59:05Z gbazin $
  *
- * Authors: Gildas Bazin <gbazin@netcourrier.com>
+ * Authors: Gildas Bazin <gbazin@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -77,6 +77,13 @@ struct vout_sys_t
     RECT         rect_src_clipped;
     RECT         rect_dest;
     RECT         rect_dest_clipped;
+    RECT         rect_parent;
+
+    /* Overlay alignment restrictions */
+    int          i_align_src_boundary;
+    int          i_align_src_size;
+    int          i_align_dest_boundary;
+    int          i_align_dest_size;
 
     /* DDraw capabilities */
     int          b_caps_overlay_clipping;
@@ -91,9 +98,10 @@ struct vout_sys_t
     volatile mtime_t    i_lastmoved;
 
     /* Misc */
-    vlc_bool_t          b_on_top_change;
+    vlc_bool_t      b_on_top_change;
 
-    event_thread_t *    p_event;
+    event_thread_t *p_event;
+    vlc_mutex_t    lock;
 };
 
 /*****************************************************************************
@@ -124,8 +132,8 @@ void DirectXUpdateRects ( vout_thread_t *p_vout, vlc_bool_t b_force );
  * Constants
  *****************************************************************************/
 #define WM_VLC_HIDE_MOUSE WM_APP
-#define WM_VLC_CREATE_VIDEO_WIN WM_APP + 1
-#define WM_VLC_DESTROY_VIDEO_WIN WM_APP + 2
+#define WM_VLC_SHOW_MOUSE WM_APP + 1
+#define WM_VLC_CREATE_VIDEO_WIN WM_APP + 2
 #define WM_VLC_CHANGE_TEXT WM_APP + 3
 #define IDM_TOGGLE_ON_TOP WM_USER + 1
 #define DX_POSITION_CHANGE 0x1000
