@@ -1,8 +1,8 @@
 /*****************************************************************************
  * input_clock.c: Clock/System date convertions, stream management
  *****************************************************************************
- * Copyright (C) 1999-2001 VideoLAN
- * $Id: input_clock.c,v 1.44 2003/12/04 16:49:45 sam Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: input_clock.c 7548 2004-04-29 14:48:12Z gbazin $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -248,7 +248,16 @@ void input_ClockManageRef( input_thread_t * p_input,
              && p_input->stream.p_selected_program == p_pgrm )
         {
             p_pgrm->last_cr = i_clock;
-            mwait( ClockToSysdate( p_input, p_pgrm, i_clock ) );
+            if( !p_input->b_out_pace_control )
+            {
+                mtime_t i_wakeup = ClockToSysdate( p_input, p_pgrm, i_clock );
+                while( (i_wakeup - mdate()) / CLOCK_FREQ > 1 )
+                {
+                    msleep( CLOCK_FREQ );
+                    if( p_input->b_die ) i_wakeup = mdate();
+                }
+                mwait( i_wakeup );
+            }
         }
         else
         {
@@ -280,7 +289,16 @@ void input_ClockManageRef( input_thread_t * p_input,
             /* Wait a while before delivering the packets to the decoder.
              * In case of multiple programs, we arbitrarily follow the
              * clock of the selected program. */
-            mwait( ClockToSysdate( p_input, p_pgrm, i_clock ) );
+            if( !p_input->b_out_pace_control )
+            {
+                mtime_t i_wakeup = ClockToSysdate( p_input, p_pgrm, i_clock );
+                while( (i_wakeup - mdate()) / CLOCK_FREQ > 1 )
+                {
+                    msleep( CLOCK_FREQ );
+                    if( p_input->b_die ) i_wakeup = mdate();
+                }
+                mwait( i_wakeup );
+            }
 
             /* Now take into account interface changes. */
             input_ClockManageControl( p_input, p_pgrm, i_clock );
