@@ -1,8 +1,8 @@
 /*****************************************************************************
  * input_ext-plugins.c: useful functions for access and demux plug-ins
  *****************************************************************************
- * Copyright (C) 2001, 2002 VideoLAN
- * $Id: input_ext-plugins.c,v 1.38 2003/11/24 03:30:38 fenrir Exp $
+ * Copyright (C) 2001-2004 VideoLAN
+ * $Id: input_ext-plugins.c 7041 2004-03-11 16:48:27Z gbazin $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -141,7 +141,7 @@ void input_BuffersEnd( input_thread_t * p_input, input_buffers_t * p_buffers )
                 free( p_buf );
                 p_buf = p_next;
             }
-        } 
+        }
 
         if( p_buffers->i_allocated )
         {
@@ -536,9 +536,12 @@ ssize_t input_FillBuffer( input_thread_t * p_input )
  *             (min. i_size bytes)
  * Returns the number of bytes read, or -1 in case of error
  *****************************************************************************/
-ssize_t input_Peek( input_thread_t * p_input, byte_t ** pp_byte, size_t i_size )
+ssize_t input_Peek( input_thread_t * p_input, byte_t ** pp_byte,
+                    size_t i_size )
 {
-    if( p_input->p_last_data - p_input->p_current_data < (ptrdiff_t)i_size )
+    ssize_t i_data = p_input->p_last_data - p_input->p_current_data;
+
+    while( i_data < (ssize_t)i_size )
     {
         /* Go to the next buffer */
         ssize_t i_ret = input_FillBuffer( p_input );
@@ -548,11 +551,16 @@ ssize_t input_Peek( input_thread_t * p_input, byte_t ** pp_byte, size_t i_size )
             return -1;
         }
 
-        if( i_ret < (ssize_t)i_size )
+        if( i_ret == i_data )
         {
-            i_size = i_ret;
+            /* We didn't get anymore data, must be the EOF */
+            i_size = i_data;
+            break;
         }
+
+        i_data = i_ret;
     }
+
     *pp_byte = p_input->p_current_data;
     return i_size;
 }
@@ -564,7 +572,9 @@ ssize_t input_Peek( input_thread_t * p_input, byte_t ** pp_byte, size_t i_size )
 ssize_t input_SplitBuffer( input_thread_t * p_input,
                            data_packet_t ** pp_data, size_t i_size )
 {
-    if( p_input->p_last_data - p_input->p_current_data < (ptrdiff_t)i_size )
+    ssize_t i_data = p_input->p_last_data - p_input->p_current_data;
+
+    while( i_data < (ssize_t)i_size )
     {
         /* Go to the next buffer */
         ssize_t i_ret = input_FillBuffer( p_input );
@@ -574,13 +584,17 @@ ssize_t input_SplitBuffer( input_thread_t * p_input,
             return -1;
         }
 
-        if( i_ret < (ssize_t)i_size )
+        if( i_ret == i_data )
         {
-            i_size = i_ret;
+            /* We didn't get anymore data, must be the EOF */
+            i_size = i_data;
+            break;
         }
+
+        i_data = i_ret;
     }
 
-    if ( i_size < 0)
+    if( i_size < 0)
     {
         return 0;
     }
@@ -598,7 +612,7 @@ ssize_t input_SplitBuffer( input_thread_t * p_input,
     vlc_mutex_lock( &p_input->stream.stream_lock );
     p_input->stream.p_selected_area->i_tell += i_size;
     vlc_mutex_unlock( &p_input->stream.stream_lock );
- 
+
     return i_size;
 }
 
