@@ -1,8 +1,8 @@
 /*****************************************************************************
  * input_programs.c: es_descriptor_t, pgrm_descriptor_t management
  *****************************************************************************
- * Copyright (C) 1999-2002 VideoLAN
- * $Id: input_programs.c,v 1.127 2003/12/22 02:24:50 sam Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: input_programs.c 7701 2004-05-17 16:10:32Z gbazin $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -51,8 +51,6 @@ static int NavigationCallback( vlc_object_t *, char const *,
 static int ESCallback( vlc_object_t *, char const *,
                        vlc_value_t, vlc_value_t, void * );
 
-static es_format_t null_es_format = {0};
-
 /*****************************************************************************
  * input_InitStream: init the stream descriptor of the given input
  *****************************************************************************/
@@ -93,7 +91,11 @@ int input_InitStream( input_thread_t * p_input, size_t i_data_len )
     var_Set( p_input, "intf-change", val );
 
     /* Create a few object variables used for navigation in the interfaces */
-    var_Create( p_input, "program", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
+    var_Create( p_input, "program", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE |
+                VLC_VAR_DOINHERIT );
+    var_Get( p_input, "program", &val );
+    if( val.i_int <= 0 )
+        var_Change( p_input, "program", VLC_VAR_DELCHOICE, &val, NULL );
     text.psz_string = _("Program");
     var_Change( p_input, "program", VLC_VAR_SETTEXT, &text, NULL );
 
@@ -110,13 +112,13 @@ int input_InitStream( input_thread_t * p_input, size_t i_data_len )
     var_Change( p_input, "navigation", VLC_VAR_SETTEXT, &text, NULL );
 
     var_Create( p_input, "video-es", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
-    text.psz_string = _("Video track");
+    text.psz_string = _("Video Track");
     var_Change( p_input, "video-es", VLC_VAR_SETTEXT, &text, NULL );
     var_Create( p_input, "audio-es", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
-    text.psz_string = _("Audio track");
+    text.psz_string = _("Audio Track");
     var_Change( p_input, "audio-es", VLC_VAR_SETTEXT, &text, NULL );
     var_Create( p_input, "spu-es", VLC_VAR_INTEGER | VLC_VAR_HASCHOICE );
-    text.psz_string = _("Subtitles track");
+    text.psz_string = _("Subtitles Track");
     var_Change( p_input, "spu-es", VLC_VAR_SETTEXT, &text, NULL );
 
     var_AddCallback( p_input, "program", ProgramCallback, NULL );
@@ -126,7 +128,7 @@ int input_InitStream( input_thread_t * p_input, size_t i_data_len )
     var_AddCallback( p_input, "audio-es", ESCallback, NULL );
     var_AddCallback( p_input, "spu-es", ESCallback, NULL );
 
-    return 0;
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
@@ -134,6 +136,8 @@ int input_InitStream( input_thread_t * p_input, size_t i_data_len )
  *****************************************************************************/
 void input_EndStream( input_thread_t * p_input )
 {
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+
     /* Free all programs and associated ES, and associated decoders. */
     while( p_input->stream.i_pgrm_number )
     {
@@ -163,6 +167,8 @@ void input_EndStream( input_thread_t * p_input )
         free( p_input->stream.p_demux_data );
     }
 
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
+
     /* Free navigation variables */
     var_Destroy( p_input, "program" );
     var_Destroy( p_input, "title" );
@@ -617,6 +623,7 @@ es_descriptor_t * input_AddES( input_thread_t * p_input,
 
     /* Init its values */
     p_es->i_id = i_es_id;
+    p_es->i_stream_id = 0;
     p_es->p_pes = NULL;
     p_es->p_dec = NULL;
     p_es->i_cat = i_category;
@@ -624,7 +631,8 @@ es_descriptor_t * input_AddES( input_thread_t * p_input,
     p_es->c_packets = 0;
     p_es->c_invalid_packets = 0;
     p_es->b_force_decoder = VLC_FALSE;
-    p_es->fmt = null_es_format;
+    es_format_Init( &p_es->fmt, UNKNOWN_ES, 0 );
+    p_es->fmt.b_packetized = VLC_FALSE; /* Only there for old mpeg demuxers */
 
     if( i_data_len )
     {
@@ -806,6 +814,9 @@ void input_DelES( input_thread_t * p_input, es_descriptor_t * p_es )
         free( p_es->psz_desc );
     }
 
+    /* Clean the es format */
+    es_format_Clean( &p_es->fmt );
+
     /* Find the ES in the ES table */
     for( i_es_index = 0; i_es_index < p_input->stream.i_es_number;
          i_es_index++ )
@@ -843,6 +854,10 @@ int input_SelectES( input_thread_t * p_input, es_descriptor_t * p_es )
     if( p_es->i_cat == VIDEO_ES || p_es->i_cat == SPU_ES )
     {
         var_Get( p_input, "video", &val );
+        if( val.b_bool && p_input->stream.p_sout )
+        {
+            var_Get( p_input, "sout-video", &val );
+        }
         if( !val.b_bool )
         {
             msg_Dbg( p_input, "video is disabled, not selecting ES 0x%x",
@@ -854,6 +869,10 @@ int input_SelectES( input_thread_t * p_input, es_descriptor_t * p_es )
     if( p_es->i_cat == AUDIO_ES )
     {
         var_Get( p_input, "audio", &val );
+        if( val.b_bool && p_input->stream.p_sout )
+        {
+            var_Get( p_input, "sout-audio", &val );
+        }
         if( !val.b_bool )
         {
             msg_Dbg( p_input, "audio is disabled, not selecting ES 0x%x",
@@ -947,8 +966,13 @@ int input_UnselectES( input_thread_t * p_input, es_descriptor_t * p_es )
         var_Change( p_input, psz_var, VLC_VAR_SETVALUE, &val, NULL );
     }
 
+    /* FIXME: input_UnselectES() shouldn't actually be entered with the
+     * input lock, the locking should be done here and only where necessary. */
+    vlc_mutex_unlock( &p_input->stream.stream_lock );
     /* Actually unselect the ES */
     input_EndDecoder( p_input, p_es );
+    vlc_mutex_lock( &p_input->stream.stream_lock );
+
     p_es->p_pes = NULL;
 
     if( ( p_es->p_dec == NULL ) &&
