@@ -1,8 +1,8 @@
 /*****************************************************************************
  * libvlc.c: main libvlc source
  *****************************************************************************
- * Copyright (C) 1998-2002 VideoLAN
- * $Id: libvlc.c,v 1.106 2003/12/24 10:06:53 gbazin Exp $
+ * Copyright (C) 1998-2004 VideoLAN
+ * $Id: libvlc.c 7398 2004-04-20 17:57:58Z gbazin $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -76,6 +76,8 @@
 #include "vlc_video.h"
 #include "video_output.h"
 
+#include "stream_output.h"
+
 #include "libvlc.h"
 
 /*****************************************************************************
@@ -99,6 +101,8 @@ static void ShowConsole   ( void );
 #endif
 static int  ConsoleWidth  ( void );
 
+static int  VerboseCallback( vlc_object_t *, char const *,
+                             vlc_value_t, vlc_value_t, void * );
 
 /*****************************************************************************
  * vlc_current_object: return the current object.
@@ -202,6 +206,7 @@ int VLC_Create( void )
     {
         return VLC_EGENERIC;
     }
+    p_vlc->thread_id = 0;
     vlc_thread_set_priority( p_vlc, VLC_THREAD_PRIORITY_LOW );
 
     p_vlc->psz_object_name = "root";
@@ -300,6 +305,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         return VLC_EGENERIC;
     }
     p_help_module->psz_object_name = "help";
+    p_help_module->psz_longname = N_("Help options");
     config_Duplicate( p_help_module, p_help_config );
     vlc_object_attach( p_help_module, libvlc.p_module_bank );
     /* End hack */
@@ -368,10 +374,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
 
         textdomain( PACKAGE );
 
-#if defined( SYS_BEOS ) || defined ( SYS_DARWIN ) || \
-    ( defined( WIN32 ) && !defined( HAVE_INCLUDED_GETTEXT ) )
-        /* BeOS only support UTF8 strings */
-        /* Mac OS X prefers UTF8 */
+#if defined( ENABLE_UTF8 )
         bind_textdomain_codeset( PACKAGE, "UTF-8" );
 #endif
 
@@ -497,18 +500,17 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     /*
      * Message queue options
      */
+
+    var_Create( p_vlc, "verbose", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     if( config_GetInt( p_vlc, "quiet" ) )
     {
-        libvlc.i_verbose = -1;
-    }
-    else
-    {
-        int i_tmp = config_GetInt( p_vlc, "verbose" );
-        if( i_tmp >= 0 )
-        {
-            libvlc.i_verbose = __MIN( i_tmp, 2 );
-        }
+        vlc_value_t val;
+        val.i_int = -1;
+        var_Set( p_vlc, "verbose", val );
     }
+    var_AddCallback( p_vlc, "verbose", VerboseCallback, NULL );
+    var_Change( p_vlc, "verbose", VLC_VAR_TRIGGER_CALLBACKS, NULL, NULL );
+
     libvlc.b_color = libvlc.b_color && config_GetInt( p_vlc, "color" );
 
     /*
@@ -527,6 +529,8 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         libvlc.i_cpu &= ~CPU_CAPABILITY_MMXEXT;
     if( !config_GetInt( p_vlc, "sse" ) )
         libvlc.i_cpu &= ~CPU_CAPABILITY_SSE;
+    if( !config_GetInt( p_vlc, "sse2" ) )
+        libvlc.i_cpu &= ~CPU_CAPABILITY_SSE2;
 #endif
 #if defined( __powerpc__ ) || defined( SYS_DARWIN )
     if( !config_GetInt( p_vlc, "altivec" ) )
@@ -549,6 +553,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     PRINT_CAPABILITY( CPU_CAPABILITY_3DNOW, "3DNow!" );
     PRINT_CAPABILITY( CPU_CAPABILITY_MMXEXT, "MMXEXT" );
     PRINT_CAPABILITY( CPU_CAPABILITY_SSE, "SSE" );
+    PRINT_CAPABILITY( CPU_CAPABILITY_SSE2, "SSE2" );
     PRINT_CAPABILITY( CPU_CAPABILITY_ALTIVEC, "AltiVec" );
     PRINT_CAPABILITY( CPU_CAPABILITY_FPU, "FPU" );
     msg_Dbg( p_vlc, "CPU has capabilities %s", p_capabilities );
@@ -556,7 +561,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     /*
      * Choose the best memcpy module
      */
-    p_vlc->p_memcpy_module = module_Need( p_vlc, "memcpy", "$memcpy" );
+    p_vlc->p_memcpy_module = module_Need( p_vlc, "memcpy", "$memcpy", 0 );
 
     if( p_vlc->pf_memcpy == NULL )
     {
@@ -611,7 +616,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
         if( psz_temp )
         {
             sprintf( psz_temp, "%s,none", psz_module );
-            VLC_AddIntf( 0, psz_temp, VLC_FALSE );
+            VLC_AddIntf( 0, psz_temp, VLC_FALSE, VLC_FALSE );
             free( psz_temp );
         }
     }
@@ -623,7 +628,7 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
     /*
      * Allways load the hotkeys interface if it exists
      */
-    VLC_AddIntf( 0, "hotkeys,none", VLC_FALSE );
+    VLC_AddIntf( 0, "hotkeys,none", VLC_FALSE, VLC_FALSE );
 
     /*
      * FIXME: kludge to use a p_vlc-local variable for the Mozilla plugin
@@ -656,9 +661,11 @@ int VLC_Init( int i_object, int i_argc, char *ppsz_argv[] )
  * This function opens an interface plugin and runs it. If b_block is set
  * to 0, VLC_AddIntf will return immediately and let the interface run in a
  * separate thread. If b_block is set to 1, VLC_AddIntf will continue until
- * user requests to quit.
+ * user requests to quit. If b_play is set to 1, VLC_AddIntf will start playing
+ * the playlist when it is completely initialised.
  *****************************************************************************/
-int VLC_AddIntf( int i_object, char const *psz_module, vlc_bool_t b_block )
+int VLC_AddIntf( int i_object, char const *psz_module,
+                 vlc_bool_t b_block, vlc_bool_t b_play )
 {
     int i_err;
     intf_thread_t *p_intf;
@@ -679,7 +686,11 @@ int VLC_AddIntf( int i_object, char const *psz_module, vlc_bool_t b_block )
         return VLC_EGENERIC;
     }
 
+    /* Interface doesn't handle play on start so do it ourselves */
+    if( !p_intf->b_play && b_play ) VLC_Play( i_object );
+
     /* Try to run the interface */
+    p_intf->b_play = b_play;
     p_intf->b_block = b_block;
     i_err = intf_RunThread( p_intf );
     if( i_err )
@@ -821,8 +832,8 @@ int VLC_AddTarget( int i_object, char const *psz_target,
         vlc_object_yield( p_playlist );
     }
 
-    i_err = playlist_Add( p_playlist, psz_target, ppsz_options, i_options,
-                          i_mode, i_pos );
+    i_err = playlist_AddExt( p_playlist, psz_target, psz_target,
+                             i_mode, i_pos, -1, ppsz_options, i_options);
 
     vlc_object_release( p_playlist );
 
@@ -905,6 +916,77 @@ int VLC_Get( int i_object, char const *psz_var, vlc_value_t *p_value )
 
 /* FIXME: temporary hacks */
 
+
+/*****************************************************************************
+ * VLC_IsPlaying: Query for Playlist Status
+ *****************************************************************************/
+vlc_bool_t VLC_IsPlaying( int i_object )
+{
+
+    playlist_t * p_playlist;
+    vlc_bool_t   playing;
+
+    vlc_t *p_vlc = vlc_current_object( i_object );
+
+    /* Check that the handle is valid */
+    if( !p_vlc )
+    {
+        return VLC_ENOOBJ;
+    }
+
+    p_playlist = vlc_object_find( p_vlc, VLC_OBJECT_PLAYLIST, FIND_CHILD );
+
+    if( !p_playlist )
+    {
+        if( i_object ) vlc_object_release( p_vlc );
+        return VLC_ENOOBJ;
+    }
+
+    playing = playlist_IsPlaying( p_playlist );
+
+    vlc_object_release( p_playlist );
+
+    if( i_object ) vlc_object_release( p_vlc );
+
+    return playing;
+
+}
+
+
+/*****************************************************************************
+ * VLC_ClearPlaylist: Query for Playlist Status
+ *
+ * return: 0
+ *****************************************************************************/
+int VLC_ClearPlaylist( int i_object )
+{
+
+    playlist_t * p_playlist;
+    vlc_t *p_vlc = vlc_current_object( i_object );
+
+    /* Check that the handle is valid */
+    if( !p_vlc )
+    {
+        return VLC_ENOOBJ;
+    }
+
+    p_playlist = vlc_object_find( p_vlc, VLC_OBJECT_PLAYLIST, FIND_CHILD );
+
+    if( !p_playlist )
+    {
+        if( i_object ) vlc_object_release( p_vlc );
+        return VLC_ENOOBJ;
+    }
+
+    playlist_Clear(p_playlist);
+
+    vlc_object_release( p_playlist );
+
+    if( i_object ) vlc_object_release( p_vlc );
+    return 0;
+}
+
+
 /*****************************************************************************
  * VLC_Play: play
  *****************************************************************************/
@@ -949,10 +1031,11 @@ int VLC_Play( int i_object )
  *****************************************************************************/
 int VLC_Stop( int i_object )
 {
-    intf_thread_t *   p_intf;
-    playlist_t    *   p_playlist;
-    vout_thread_t *   p_vout;
-    aout_instance_t * p_aout;
+    intf_thread_t      * p_intf;
+    playlist_t         * p_playlist;
+    vout_thread_t      * p_vout;
+    aout_instance_t    * p_aout;
+    announce_handler_t * p_announce;
     vlc_t *p_vlc = vlc_current_object( i_object );
 
     /* Check that the handle is valid */
@@ -977,7 +1060,7 @@ int VLC_Stop( int i_object )
     /*
      * Free playlists
      */
-    
+
     msg_Dbg( p_vlc, "removing all playlists" );
     while( (p_playlist = vlc_object_find( p_vlc, VLC_OBJECT_PLAYLIST,
                                           FIND_CHILD )) )
@@ -986,7 +1069,7 @@ int VLC_Stop( int i_object )
         vlc_object_release( p_playlist );
         playlist_Destroy( p_playlist );
     }
-    
+
     /*
      * Free video outputs
      */
@@ -1009,6 +1092,18 @@ int VLC_Stop( int i_object )
         aout_Delete( p_aout );
     }
 
+    /*
+     * Free announce handler(s?)
+     */
+    msg_Dbg( p_vlc, "removing announce handler" );
+    while( (p_announce = vlc_object_find( p_vlc, VLC_OBJECT_ANNOUNCE,
+                                                 FIND_CHILD ) ) )
+   {
+        vlc_object_detach( p_announce );
+        vlc_object_release( p_announce );
+        announce_HandlerDestroy( p_announce );
+   }
+
     if( i_object ) vlc_object_release( p_vlc );
     return VLC_SUCCESS;
 }
@@ -1133,10 +1228,7 @@ static void SetLanguage ( char const *psz_lang )
     /* Set the default domain */
     textdomain( PACKAGE );
 
-#if defined( SYS_BEOS ) || defined ( SYS_DARWIN ) || \
-    ( defined( WIN32 ) && !defined( HAVE_INCLUDED_GETTEXT ) )
-    /* BeOS only support UTF8 strings */
-    /* Mac OS X prefers UTF8 */
+#if defined( ENABLE_UTF8 )
     bind_textdomain_codeset( PACKAGE, "UTF-8" );
 #endif
 
@@ -1171,8 +1263,7 @@ static int GetFilenames( vlc_t *p_vlc, int i_argc, char *ppsz_argv[] )
         VLC_AddTarget( p_vlc->i_object_id, ppsz_argv[ i_opt ],
                        (char const **)( i_options ? &ppsz_argv[i_opt + 1] :
                                         NULL ), i_options,
-                       PLAYLIST_INSERT | (i_opt == optind ? PLAYLIST_GO : 0),
-                       0 );
+                       PLAYLIST_INSERT, 0 );
     }
 
     return VLC_SUCCESS;
@@ -1208,6 +1299,7 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
     char psz_short[4];
     int i_index;
     int i_width = ConsoleWidth() - (PADDING_SPACES+LINE_START+1);
+    vlc_bool_t b_advanced = config_GetInt( p_this, "advanced" );
 
     memset( psz_spaces, ' ', PADDING_SPACES+LINE_START );
     psz_spaces[PADDING_SPACES+LINE_START] = '\0';
@@ -1240,6 +1332,23 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
             continue;
         }
 
+        /* Ignore modules with only advanced config options if requested */
+        if( !b_advanced )
+        {
+            for( p_item = p_parser->p_config;
+                 p_item->i_type != CONFIG_HINT_END;
+                 p_item++ )
+            {
+                if( (p_item->i_type & CONFIG_ITEM) &&
+                    !p_item->b_advanced ) break;
+            }
+            if( p_item->i_type == CONFIG_HINT_END ) continue;
+        }
+
+        /* Print name of module */
+        if( strcmp( "main", p_parser->psz_object_name ) )
+        fprintf( stdout, "\n %s\n", p_parser->psz_longname );
+
         b_help_module = !strcmp( "help", p_parser->psz_object_name );
 
         /* Print module options */
@@ -1251,14 +1360,18 @@ static void Usage( vlc_t *p_this, char const *psz_module_name )
             char *psz_bra = NULL, *psz_type = NULL, *psz_ket = NULL;
             char *psz_suf = "", *psz_prefix = NULL;
             int i;
-            if ( p_item->b_advanced && !config_GetInt( p_this, "advanced" ))
+
+            /* Skip advanced options if requested */
+            if( p_item->b_advanced && !b_advanced )
             {
                 continue;
             }
+
             switch( p_item->i_type )
             {
             case CONFIG_HINT_CATEGORY:
             case CONFIG_HINT_USAGE:
+                if( !strcmp( "main", p_parser->psz_object_name ) )
                 fprintf( stdout, "\n %s\n", p_item->psz_text );
                 break;
 
@@ -1553,3 +1666,15 @@ static int ConsoleWidth( void )
 
     return i_width;
 }
+
+static int VerboseCallback( vlc_object_t *p_this, const char *psz_variable,
+                     vlc_value_t old_val, vlc_value_t new_val, void *param)
+{
+    vlc_t *p_vlc = (vlc_t *)p_this;
+
+    if( new_val.i_int >= -1 )
+    {
+        p_vlc->p_libvlc->i_verbose = __MIN( new_val.i_int, 2 );
+    }
+    return VLC_SUCCESS;
+}
