@@ -2,7 +2,7 @@
  * block.c: Data blocks management functions
  *****************************************************************************
  * Copyright (C) 2003-2004 VideoLAN
- * $Id: block.c,v 1.7 2004/02/25 17:48:52 fenrir Exp $
+ * $Id: block.c 7678 2004-05-15 14:42:16Z fenrir $
  *
  * Authors: Laurent Aimar <fenrir@videolan.org>
  *
@@ -73,6 +73,8 @@ static block_t *__BlockDupContent( block_t *p_block )
     p_dup->i_flags         = p_block->i_flags;
     p_dup->i_pts           = p_block->i_pts;
     p_dup->i_dts           = p_block->i_dts;
+    p_dup->i_length        = p_block->i_length;
+    p_dup->i_rate          = p_block->i_rate;
 
     return p_dup;
 }
@@ -128,52 +130,48 @@ static block_t *BlockDuplicate( block_t *p_block )
 
 static block_t *BlockRealloc( block_t *p_block, int i_prebody, int i_body )
 {
+    int i_buffer_size = i_prebody + i_body;
+
+    if( i_body < 0 || i_buffer_size <= 0 ) return NULL;
 
     vlc_mutex_lock( &p_block->p_sys->lock );
-    if( i_prebody < 0 || p_block->p_buffer - i_prebody >
-        p_block->p_sys->p_allocated_buffer )
+
+    if( i_prebody < ( p_block->p_buffer - p_block->p_sys->p_allocated_buffer +
+                      p_block->p_sys->i_allocated_buffer ) ||
+        p_block->p_buffer - i_prebody > p_block->p_sys->p_allocated_buffer )
     {
         p_block->p_buffer -= i_prebody;
         p_block->i_buffer += i_prebody;
         i_prebody = 0;
     }
-    if( i_body < 0 ||
-        p_block->p_buffer + i_body < p_block->p_sys->p_allocated_buffer +
+    if( p_block->p_buffer + i_body < p_block->p_sys->p_allocated_buffer +
         p_block->p_sys->i_allocated_buffer )
     {
-        p_block->i_buffer = i_body;
+        p_block->i_buffer = i_buffer_size;
         i_body = 0;
     }
+
+    if( i_body > 0 || i_prebody > 0 )
+    {
+        block_t *p_rea = block_New( p_block->p_manager, i_buffer_size );
+
+        p_rea->i_dts   = p_block->i_dts;
+        p_rea->i_pts   = p_block->i_pts;
+        p_rea->i_flags = p_block->i_flags;
+        p_rea->i_length= p_block->i_length;
+        p_rea->i_rate  = p_block->i_rate;
+
+        memcpy( p_rea->p_buffer + i_prebody, p_block->p_buffer,
+                __MIN( p_block->i_buffer, p_rea->i_buffer - i_prebody ) );
+
+        vlc_mutex_unlock( &p_block->p_sys->lock );
+        block_Release( p_block );
+
+        return p_rea;
+    }
+
     vlc_mutex_unlock( &p_block->p_sys->lock );
 
-    if( i_prebody > 0 )
-    {
-        block_t *p_rea = block_New( p_block->p_manager, i_prebody + i_body );
-
-        memcpy( &p_rea->p_buffer[i_prebody], p_block->p_buffer,
-                p_block->i_buffer );
-
-        return p_rea;
-    }
-
-    if( i_body > 0 )
-    {
-        int i_start;
-        block_t *p_rea = BlockModify( p_block, VLC_TRUE );
-
-        i_start = p_rea->p_buffer - p_rea->p_sys->p_allocated_buffer;
-
-        p_rea->p_sys->i_allocated_buffer += i_body - p_rea->i_buffer;
-        p_rea->p_sys->p_allocated_buffer =
-            realloc( p_rea->p_sys->p_allocated_buffer,
-                     p_rea->p_sys->i_allocated_buffer );
-
-        p_rea->p_buffer = &p_rea->p_sys->p_allocated_buffer[i_start];
-        p_rea->i_buffer = i_body;
-
-        return p_rea;
-    }
-
     return p_block;
 }
 
@@ -194,6 +192,7 @@ block_t *block_NewEmpty( void )
     p_block->i_pts          = 0;
     p_block->i_dts          = 0;
     p_block->i_length       = 0;
+    p_block->i_rate         = 0;
 
     p_block->i_buffer       = 0;
     p_block->p_buffer       = NULL;
@@ -215,12 +214,6 @@ block_t *__block_New( vlc_object_t *p_obj, int i_size )
 
     p_block = block_NewEmpty();
 
-    p_block->i_buffer       = i_size;
-    if( i_size > 0 )
-    {
-        p_block->p_buffer   = malloc( i_size );
-    }
-
     p_block->pf_release     = BlockRelease;
     p_block->pf_duplicate   = BlockDuplicate;
     p_block->pf_modify      = BlockModify;
@@ -231,8 +224,13 @@ block_t *__block_New( vlc_object_t *p_obj, int i_size )
 
     p_block->p_sys = p_sys = malloc( sizeof( block_sys_t ) );
     vlc_mutex_init( p_obj, &p_sys->lock );
-    p_sys->p_allocated_buffer = p_block->p_buffer;
-    p_sys->i_allocated_buffer = p_block->i_buffer;
+
+    /* XXX align on 16 and add 32 prebuffer/posbuffer bytes */
+    p_sys->i_allocated_buffer = i_size + 32 + 32 + 16;
+    p_sys->p_allocated_buffer = malloc( p_sys->i_allocated_buffer );
+    p_block->i_buffer         = i_size;
+    p_block->p_buffer         = &p_sys->p_allocated_buffer[32+15-((long)p_sys->p_allocated_buffer % 16 )];
+
     p_sys->i_duplicated = 0;
     p_sys->b_modify = VLC_TRUE;
 
@@ -241,7 +239,6 @@ block_t *__block_New( vlc_object_t *p_obj, int i_size )
 
 void block_ChainAppend( block_t **pp_list, block_t *p_block )
 {
-
     if( *pp_list == NULL )
     {
         *pp_list = p_block;
@@ -258,6 +255,18 @@ void block_ChainAppend( block_t **pp_list, block_t *p_block )
     }
 }
 
+void block_ChainLastAppend( block_t ***ppp_last, block_t *p_block  )
+{
+    block_t *p_last = p_block;
+
+    /* Append the block */
+    **ppp_last = p_block;
+
+    /* Update last pointer */
+    while( p_last->p_next ) p_last = p_last->p_next;
+    *ppp_last = &p_last->p_next;
+}
+
 void block_ChainRelease( block_t *p_block )
 {
     while( p_block )
