@@ -1,8 +1,8 @@
 /*****************************************************************************
  * modules.c : Builtin and plugin modules management functions
  *****************************************************************************
- * Copyright (C) 2001 VideoLAN
- * $Id: modules.c,v 1.141 2003/11/25 12:35:15 sam Exp $
+ * Copyright (C) 2001-2004 VideoLAN
+ * $Id: modules.c 7209 2004-03-31 20:52:31Z gbazin $
  *
  * Authors: Sam Hocevar <sam@zoy.org>
  *          Ethan C. Baldridge <BaldridgeE@cadmus.com>
@@ -82,7 +82,6 @@
 #include "vlc_error.h"
 
 #include "vlc_interface.h"
-#include "vlc_playlist.h"
 #include "intf_eject.h"
 
 #include "stream_control.h"
@@ -91,6 +90,8 @@
 #include "input_ext-plugins.h"
 #include "ninput.h"
 
+#include "vlc_playlist.h"
+
 #include "vlc_video.h"
 #include "video_output.h"
 #include "vout_synchro.h"
@@ -99,14 +100,16 @@
 #include "aout_internal.h"
 
 #include "stream_output.h"
-/*#include "announce.h"*/
 #include "osd.h"
+#include "vlc_httpd.h"
 
 #include "iso_lang.h"
 #include "charset.h"
 
 #include "vlc_block.h"
 
+#include "vlc_vlm.h"
+
 #if defined( UNDER_CE )
 #   define MYCHAR wchar_t
 #else
@@ -124,6 +127,7 @@
 #else
 #    include "modules_builtin.h"
 #endif
+#include "network.h"
 
 /*****************************************************************************
  * Local prototypes
@@ -261,7 +265,7 @@ void __module_LoadPlugins( vlc_object_t * p_this )
  * This function returns the module that best fits the asked capabilities.
  *****************************************************************************/
 module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
-                          const char *psz_name )
+                          const char *psz_name, vlc_bool_t b_strict )
 {
     typedef struct module_list_t module_list_t;
 
@@ -298,7 +302,7 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
     /* Count how many different shortcuts were asked for */
     if( psz_name && *psz_name )
     {
-        char *psz_parser;
+        char *psz_parser, *psz_last_shortcut;
 
         /* If the user wants none, give him none. */
         if( !strcmp( psz_name, "none" ) )
@@ -308,7 +312,7 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
         }
 
         i_shortcuts++;
-        psz_shortcuts = strdup( psz_name );
+        psz_shortcuts = psz_last_shortcut = strdup( psz_name );
 
         for( psz_parser = psz_shortcuts; *psz_parser; psz_parser++ )
         {
@@ -316,6 +320,22 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
             {
                  *psz_parser = '\0';
                  i_shortcuts++;
+                 psz_last_shortcut = psz_parser + 1;
+            }
+        }
+
+        /* Check if the user wants to override the "strict" mode */
+        if( psz_last_shortcut )
+        {
+            if( !strcmp(psz_last_shortcut, "none") )
+            {
+                b_strict = VLC_TRUE;
+                i_shortcuts--;
+            }
+            else if( !strcmp(psz_last_shortcut, "any") )
+            {
+                b_strict = VLC_FALSE;
+                i_shortcuts--;
             }
         }
     }
@@ -364,7 +384,7 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
         }
 
         /* If we required a shortcut, check this plugin provides it. */
-        if( i_shortcuts )
+        if( i_shortcuts > 0 )
         {
             vlc_bool_t b_trash;
             int i_dummy, i_short = i_shortcuts;
@@ -374,17 +394,8 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
              * explicitly requested) */
             b_trash = !p_module->i_score;
 
-            while( i_short )
+            while( i_short > 0 )
             {
-                /* If the last given shortcut is "none" and we couldn't
-                 * find the module in the list of provided shortcuts,
-                 * then kick the bastard out of here!!! */
-                if( (i_short == 1) && !strcmp(psz_name, "none") )
-                {
-                    b_trash = VLC_TRUE;
-                    break;
-                }
-
                 for( i_dummy = 0; p_module->pp_shortcuts[i_dummy]; i_dummy++ )
                 {
                     if( !strcasecmp( psz_name,
@@ -412,6 +423,14 @@ module_t * __module_Need( vlc_object_t *p_this, const char *psz_capability,
                 i_short--;
             }
 
+            /* If we are in "strict" mode and we couldn't
+             * find the module in the list of provided shortcuts,
+             * then kick the bastard out of here!!! */
+            if( i_short == 0 && b_strict )
+            {
+                b_trash = VLC_TRUE;
+            }
+
             if( b_trash )
             {
                 continue;
