@@ -1,8 +1,8 @@
 /*****************************************************************************
  * threads.c : threads implementation for the VideoLAN client
  *****************************************************************************
- * Copyright (C) 1999, 2000, 2001, 2002 VideoLAN
- * $Id: threads.c,v 1.40 2003/03/10 00:04:14 massiot Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: threads.c 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -41,10 +41,10 @@ static volatile int i_initializations = 0;
 #elif defined( ST_INIT_IN_ST_H )
 #elif defined( UNDER_CE )
 #elif defined( WIN32 )
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     static pthread_mutex_t once_mutex = PTHREAD_MUTEX_INITIALIZER;
 #elif defined( HAVE_CTHREADS_H )
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
 #endif
 
 /*****************************************************************************
@@ -81,10 +81,10 @@ int __vlc_threads_init( vlc_object_t *p_this )
 #elif defined( UNDER_CE )
 #elif defined( WIN32 )
     HINSTANCE hInstLib;
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     pthread_mutex_lock( &once_mutex );
 #elif defined( HAVE_CTHREADS_H )
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
 #endif
 
     if( i_status == VLC_THREADS_UNINITIALIZED )
@@ -124,9 +124,9 @@ int __vlc_threads_init( vlc_object_t *p_this )
         p_libvlc->b_fast_mutex = 0;
         p_libvlc->i_win9x_cv = 0;
 
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
 #elif defined( HAVE_CTHREADS_H )
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
 #endif
 
         vlc_object_create( p_libvlc, VLC_OBJECT_ROOT );
@@ -157,12 +157,12 @@ int __vlc_threads_init( vlc_object_t *p_this )
     while( i_status == VLC_THREADS_PENDING ) msleep( THREAD_SLEEP );
 #elif defined( WIN32 )
     while( i_status == VLC_THREADS_PENDING ) msleep( THREAD_SLEEP );
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
+    while( i_status == VLC_THREADS_PENDING ) msleep( THREAD_SLEEP );
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     pthread_mutex_unlock( &once_mutex );
 #elif defined( HAVE_CTHREADS_H )
     while( i_status == VLC_THREADS_PENDING ) msleep( THREAD_SLEEP );
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    while( i_status == VLC_THREADS_PENDING ) msleep( THREAD_SLEEP );
 #endif
 
     if( i_status != VLC_THREADS_READY )
@@ -197,6 +197,9 @@ int __vlc_threads_end( vlc_object_t *p_this )
 #elif defined( WIN32 )
     i_initializations--;
 
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
+    i_initializations--;
+
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     pthread_mutex_lock( &once_mutex );
     i_initializations--;
@@ -205,53 +208,10 @@ int __vlc_threads_end( vlc_object_t *p_this )
 #elif defined( HAVE_CTHREADS_H )
     i_initializations--;
 
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    i_initializations--;
-
 #endif
     return VLC_SUCCESS;
 }
 
-/*****************************************************************************
- * Prototype for GPROF wrapper
- *****************************************************************************/
-#ifdef GPROF
-typedef void *(*vlc_thread_func_t)(void *p_data);
-
-/* Wrapper function for profiling */
-static void *      vlc_thread_wrapper ( void *p_wrapper );
-
-#   if defined( WIN32 ) && !defined( UNDER_CE )
-
-#       define ITIMER_REAL 1
-#       define ITIMER_PROF 2
-
-struct itimerval
-{
-    struct timeval it_value;
-    struct timeval it_interval;
-};
-
-int setitimer(int kind, const struct itimerval* itnew, struct itimerval* itold);
-#   endif /* WIN32 && !UNDER_CE */
-
-typedef struct wrapper_t
-{
-    /* Data lock access */
-    vlc_mutex_t lock;
-    vlc_cond_t  wait;
-
-    /* Data used to spawn the real thread */
-    vlc_thread_func_t func;
-    void *p_data;
-
-    /* Profiling timer passed to the thread */
-    struct itimerval itimer;
-
-} wrapper_t;
-
-#endif /* GPROF */
-
 /*****************************************************************************
  * vlc_mutex_init: initialize a mutex
  *****************************************************************************/
@@ -289,26 +249,6 @@ int __vlc_mutex_init( vlc_object_t *p_this, vlc_mutex_t *p_mutex )
         return 0;
     }
 
-#elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
-#   if defined(DEBUG) && defined(SYS_LINUX)
-    {
-        /* Create error-checking mutex to detect problems more easily. */
-        pthread_mutexattr_t attr;
-        int                 i_result;
-
-        pthread_mutexattr_init( &attr );
-        pthread_mutexattr_setkind_np( &attr, PTHREAD_MUTEX_ERRORCHECK_NP );
-        i_result = pthread_mutex_init( &p_mutex->mutex, &attr );
-        pthread_mutexattr_destroy( &attr );
-        return( i_result );
-    }
-#   endif
-    return pthread_mutex_init( &p_mutex->mutex, NULL );
-
-#elif defined( HAVE_CTHREADS_H )
-    mutex_init( p_mutex );
-    return 0;
-
 #elif defined( HAVE_KERNEL_SCHEDULER_H )
     /* check the arguments and whether it's already been initialized */
     if( p_mutex == NULL )
@@ -330,6 +270,26 @@ int __vlc_mutex_init( vlc_object_t *p_this, vlc_mutex_t *p_mutex )
     p_mutex->init = 9999;
     return B_OK;
 
+#elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
+#   if defined(DEBUG) && defined(SYS_LINUX)
+    {
+        /* Create error-checking mutex to detect problems more easily. */
+        pthread_mutexattr_t attr;
+        int                 i_result;
+
+        pthread_mutexattr_init( &attr );
+        pthread_mutexattr_setkind_np( &attr, PTHREAD_MUTEX_ERRORCHECK_NP );
+        i_result = pthread_mutex_init( &p_mutex->mutex, &attr );
+        pthread_mutexattr_destroy( &attr );
+        return( i_result );
+    }
+#   endif
+    return pthread_mutex_init( &p_mutex->mutex, NULL );
+
+#elif defined( HAVE_CTHREADS_H )
+    mutex_init( p_mutex );
+    return 0;
+
 #endif
 }
 
@@ -364,6 +324,15 @@ int __vlc_mutex_destroy( char * psz_file, int i_line, vlc_mutex_t *p_mutex )
     }
     return 0;
 
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
+    if( p_mutex->init == 9999 )
+    {
+        delete_sem( p_mutex->lock );
+    }
+
+    p_mutex->init = 0;
+    return B_OK;
+
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_result = pthread_mutex_destroy( &p_mutex->mutex );
     if ( i_result )
@@ -375,14 +344,6 @@ int __vlc_mutex_destroy( char * psz_file, int i_line, vlc_mutex_t *p_mutex )
 #elif defined( HAVE_CTHREADS_H )
     return 0;
 
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    if( p_mutex->init == 9999 )
-    {
-        delete_sem( p_mutex->lock );
-    }
-
-    p_mutex->init = 0;
-    return B_OK;
 #endif
 
     if( i_result )
@@ -458,18 +419,6 @@ int __vlc_cond_init( vlc_object_t *p_this, vlc_cond_t *p_condvar )
         return !p_condvar->semaphore || !p_condvar->event;
     }
 
-#elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
-    return pthread_cond_init( &p_condvar->cond, NULL );
-
-#elif defined( HAVE_CTHREADS_H )
-    /* condition_init() */
-    spin_lock_init( &p_condvar->lock );
-    cthread_queue_init( &p_condvar->queue );
-    p_condvar->name = 0;
-    p_condvar->implications = 0;
-
-    return 0;
-
 #elif defined( HAVE_KERNEL_SCHEDULER_H )
     if( !p_condvar )
     {
@@ -484,6 +433,19 @@ int __vlc_cond_init( vlc_object_t *p_this, vlc_cond_t *p_condvar )
     p_condvar->thread = -1;
     p_condvar->init = 9999;
     return 0;
+
+#elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
+    return pthread_cond_init( &p_condvar->cond, NULL );
+
+#elif defined( HAVE_CTHREADS_H )
+    /* condition_init() */
+    spin_lock_init( &p_condvar->lock );
+    cthread_queue_init( &p_condvar->queue );
+    p_condvar->name = 0;
+    p_condvar->implications = 0;
+
+    return 0;
+
 #endif
 }
 
@@ -513,6 +475,10 @@ int __vlc_cond_destroy( char * psz_file, int i_line, vlc_cond_t *p_condvar )
         i_result = !CloseHandle( p_condvar->event )
           || !CloseHandle( p_condvar->semaphore );
 
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
+    p_condvar->init = 0;
+    return 0;
+
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_result = pthread_cond_destroy( &p_condvar->cond );
     if ( i_result )
@@ -524,9 +490,6 @@ int __vlc_cond_destroy( char * psz_file, int i_line, vlc_cond_t *p_condvar )
 #elif defined( HAVE_CTHREADS_H )
     return 0;
 
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    p_condvar->init = 0;
-    return 0;
 #endif
 
     if( i_result )
@@ -549,29 +512,10 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
                          int i_priority, vlc_bool_t b_wait )
 {
     int i_ret;
-    void *p_data;
+    void *p_data = (void *)p_this;
 
     vlc_mutex_lock( &p_this->object_lock );
 
-#ifdef GPROF
-    wrapper_t wrapper;
-
-    /* Initialize the wrapper structure */
-    wrapper.func = func;
-    wrapper.p_data = (void *)p_this;
-    getitimer( ITIMER_PROF, &wrapper.itimer );
-    vlc_mutex_init( p_this, &wrapper.lock );
-    vlc_cond_init( p_this, &wrapper.wait );
-    vlc_mutex_lock( &wrapper.lock );
-
-    /* Alter user-passed data so that we call the wrapper instead
-     * of the real function */
-    p_data = &wrapper;
-    func = vlc_thread_wrapper;
-#else
-    p_data = (void *)p_this;
-#endif
-
 #if defined( PTH_INIT_IN_PTH_H )
     p_this->thread_id = pth_spawn( PTH_ATTR_DEFAULT, func, p_data );
     i_ret = p_this->thread_id == NULL;
@@ -608,15 +552,25 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
 
     i_ret = ( p_this->thread_id ? 0 : 1 );
 
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
+    p_this->thread_id = spawn_thread( (thread_func)func, psz_name,
+                                      i_priority, p_data );
+    i_ret = resume_thread( p_this->thread_id );
+
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_ret = pthread_create( &p_this->thread_id, NULL, func, p_data );
 
-#ifdef SYS_DARWIN
-    if ( i_priority )
+    if ( i_priority
+#ifndef SYS_DARWIN
+            && config_GetInt( p_this, "rt-priority" )
+#endif
+       )
     {
         int i_error, i_policy;
         struct sched_param param;
+
         memset( &param, 0, sizeof(struct sched_param) );
+        i_priority += config_GetInt( p_this, "rt-offset" );
         if ( i_priority < 0 )
         {
             param.sched_priority = (-1) * i_priority;
@@ -635,28 +589,15 @@ int __vlc_thread_create( vlc_object_t *p_this, char * psz_file, int i_line,
             i_priority = 0;
         }
     }
-#endif
+    else
+    {
+        i_priority = 0;
+    }
 
 #elif defined( HAVE_CTHREADS_H )
     p_this->thread_id = cthread_fork( (cthread_fn_t)func, (any_t)p_data );
     i_ret = 0;
 
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    p_this->thread_id = spawn_thread( (thread_func)func, psz_name,
-                                      i_priority, p_data );
-    i_ret = resume_thread( p_this->thread_id );
-
-#endif
-
-#ifdef GPROF
-    if( i_ret == 0 )
-    {
-        vlc_cond_wait( &wrapper.wait, &wrapper.lock );
-    }
-
-    vlc_mutex_unlock( &wrapper.lock );
-    vlc_mutex_destroy( &wrapper.lock );
-    vlc_cond_destroy( &wrapper.wait );
 #endif
 
     if( i_ret == 0 )
@@ -706,13 +647,17 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
     }
 
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
-
-#ifdef SYS_DARWIN
-    if ( i_priority )
+    if ( i_priority
+#ifndef SYS_DARWIN
+            && config_GetInt( p_this, "rt-priority" )
+#endif
+        )
     {
         int i_error, i_policy;
         struct sched_param param;
+
         memset( &param, 0, sizeof(struct sched_param) );
+        i_priority += config_GetInt( p_this, "rt-offset" );
         if ( i_priority < 0 )
         {
             param.sched_priority = (-1) * i_priority;
@@ -723,16 +668,16 @@ int __vlc_thread_set_priority( vlc_object_t *p_this, char * psz_file,
             param.sched_priority = i_priority;
             i_policy = SCHED_RR;
         }
-        if ( (i_error = pthread_setschedparam( pthread_self(),
+        if ( !p_this->thread_id )
+            p_this->thread_id = pthread_self();
+        if ( (i_error = pthread_setschedparam( p_this->thread_id,
                                                i_policy, &param )) )
         {
-            msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",     
+            msg_Warn( p_this, "couldn't set thread priority (%s:%d): %s",
                       psz_file, i_line, strerror(i_error) );
             i_priority = 0;
         }
     }
-#endif
-
 #endif
 
     return 0;
@@ -767,6 +712,10 @@ void __vlc_thread_join( vlc_object_t *p_this, char * psz_file, int i_line )
 #elif defined( WIN32 )
     WaitForSingleObject( p_this->thread_id, INFINITE );
 
+#elif defined( HAVE_KERNEL_SCHEDULER_H )
+    int32_t exit_value;
+    wait_for_thread( p_this->thread_id, &exit_value );
+
 #elif defined( PTHREAD_COND_T_IN_PTHREAD_H )
     i_ret = pthread_join( p_this->thread_id, NULL );
 
@@ -774,10 +723,6 @@ void __vlc_thread_join( vlc_object_t *p_this, char * psz_file, int i_line )
     cthread_join( p_this->thread_id );
     i_ret = 1;
 
-#elif defined( HAVE_KERNEL_SCHEDULER_H )
-    int32 exit_value;
-    wait_for_thread( p_this->thread_id, &exit_value );
-
 #endif
 
     if( i_ret )
@@ -800,25 +745,3 @@ void __vlc_thread_join( vlc_object_t *p_this, char * psz_file, int i_line )
     p_this->b_thread = 0;
 }
 
-/*****************************************************************************
- * vlc_thread_wrapper: wrapper around thread functions used when profiling.
- *****************************************************************************/
-#ifdef GPROF
-static void *vlc_thread_wrapper( void *p_wrapper )
-{
-    /* Put user data in thread-local variables */
-    void *            p_data = ((wrapper_t*)p_wrapper)->p_data;
-    vlc_thread_func_t func   = ((wrapper_t*)p_wrapper)->func;
-
-    /* Set the profile timer value */
-    setitimer( ITIMER_PROF, &((wrapper_t*)p_wrapper)->itimer, NULL );
-
-    /* Tell the calling thread that we don't need its data anymore */
-    vlc_mutex_lock( &((wrapper_t*)p_wrapper)->lock );
-    vlc_cond_signal( &((wrapper_t*)p_wrapper)->wait );
-    vlc_mutex_unlock( &((wrapper_t*)p_wrapper)->lock );
-
-    /* Call the real function */
-    return func( p_data );
-}
-#endif
