@@ -2,7 +2,7 @@
  * stream_output.c : stream output module
  *****************************************************************************
  * Copyright (C) 2002-2004 VideoLAN
- * $Id: stream_output.c,v 1.40 2004/02/22 16:08:47 fenrir Exp $
+ * $Id: stream_output.c 7643 2004-05-12 17:45:18Z gbazin $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -31,8 +31,10 @@
 #include <string.h>                                            /* strerror() */
 
 #include <vlc/vlc.h>
-
 #include <vlc/sout.h>
+
+#include "vlc_meta.h"
+
 #undef DEBUG_BUFFER
 /*****************************************************************************
  * Local prototypes
@@ -42,9 +44,6 @@ static void sout_cfg_free( sout_cfg_t * );
 #define sout_stream_url_to_chain( p, s ) _sout_stream_url_to_chain( VLC_OBJECT(p), s )
 static char *_sout_stream_url_to_chain( vlc_object_t *, char * );
 
-#define module_NeedStrict(a,b,c) __module_NeedStrict(VLC_OBJECT(a),b,c)
-static module_t *__module_NeedStrict( vlc_object_t *, const char *, const char * );
-
 /*
  * Generic MRL parser
  *
@@ -52,10 +51,8 @@ static module_t *__module_NeedStrict( vlc_object_t *, const char *, const char *
 
 typedef struct
 {
-    char            *psz_access;
-
-    char            *psz_way;
-
+    char *psz_access;
+    char *psz_way;
     char *psz_name;
 } mrl_t;
 
@@ -69,8 +66,7 @@ static void mrl_Clean( mrl_t *p_mrl );
 /*****************************************************************************
  * sout_NewInstance: creates a new stream output instance
  *****************************************************************************/
-sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
-                                       char * psz_dest )
+sout_instance_t *__sout_NewInstance( vlc_object_t *p_parent, char * psz_dest )
 {
     sout_instance_t *p_sout;
     vlc_value_t keep;
@@ -83,12 +79,14 @@ sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
     else if( keep.b_bool )
     {
         msg_Warn( p_parent, "sout-keep true" );
-        if( ( p_sout = vlc_object_find( p_parent, VLC_OBJECT_SOUT, FIND_ANYWHERE ) ) )
+        if( ( p_sout = vlc_object_find( p_parent, VLC_OBJECT_SOUT,
+                                        FIND_ANYWHERE ) ) )
         {
             if( !strcmp( p_sout->psz_sout, psz_dest ) )
             {
                 msg_Warn( p_parent, "sout keep : reusing sout" );
-                msg_Warn( p_parent, "sout keep : you probably want to use gather stream_out" );
+                msg_Warn( p_parent, "sout keep : you probably want to use "
+                          "gather stream_out" );
                 vlc_object_detach( p_sout );
                 vlc_object_attach( p_sout, p_parent );
                 vlc_object_release( p_sout );
@@ -103,7 +101,8 @@ sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
     }
     else if( !keep.b_bool )
     {
-        while( ( p_sout = vlc_object_find( p_parent, VLC_OBJECT_SOUT, FIND_PARENT ) ) )
+        while( ( p_sout = vlc_object_find( p_parent, VLC_OBJECT_SOUT,
+                                           FIND_PARENT ) ) )
         {
             msg_Warn( p_parent, "sout keep : destroying old sout" );
             sout_DeleteInstance( p_sout );
@@ -120,8 +119,8 @@ sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
 
     /* *** init descriptor *** */
     p_sout->psz_sout    = strdup( psz_dest );
-    p_sout->i_preheader = 0;
-    p_sout->i_padding   = 0;
+    p_sout->p_meta      = NULL;
+    p_sout->i_out_pace_nocontrol = 0;
     p_sout->p_sys       = NULL;
 
     vlc_mutex_init( p_sout, &p_sout->lock );
@@ -134,6 +133,10 @@ sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
         p_sout->psz_chain = sout_stream_url_to_chain( p_sout, psz_dest );
         msg_Dbg( p_sout, "using sout chain=`%s'", p_sout->psz_chain );
     }
+    p_sout->p_stream = NULL;
+
+    /* attach it for inherit */
+    vlc_object_attach( p_sout, p_parent );
 
     p_sout->p_stream = sout_stream_new( p_sout, p_sout->psz_chain );
 
@@ -145,11 +148,9 @@ sout_instance_t * __sout_NewInstance ( vlc_object_t *p_parent,
         FREE( p_sout->psz_chain );
 
         vlc_object_destroy( p_sout );
-        return( NULL );
+        return NULL;
     }
 
-    vlc_object_attach( p_sout, p_parent );
-
     return p_sout;
 }
 /*****************************************************************************
@@ -160,11 +161,19 @@ void sout_DeleteInstance( sout_instance_t * p_sout )
     /* Unlink object */
     vlc_object_detach( p_sout );
 
+    /* remove the stream out chain */
+    sout_stream_delete( p_sout->p_stream );
+
     /* *** free all string *** */
     FREE( p_sout->psz_sout );
     FREE( p_sout->psz_chain );
 
-    sout_stream_delete( p_sout->p_stream );
+    /* delete meta */
+    if( p_sout->p_meta )
+    {
+        vlc_meta_Delete( p_sout->p_meta );
+    }
+
     vlc_mutex_destroy( &p_sout->lock );
 
     /* *** free structure *** */
@@ -206,7 +215,9 @@ sout_packetizer_input_t *sout_InputNew( sout_instance_t *p_sout,
     return( p_input );
 }
 
-
+/*****************************************************************************
+ *
+ *****************************************************************************/
 int sout_InputDelete( sout_packetizer_input_t *p_input )
 {
     sout_instance_t     *p_sout = p_input->p_sout;
@@ -225,22 +236,24 @@ int sout_InputDelete( sout_packetizer_input_t *p_input )
     return( VLC_SUCCESS);
 }
 
+/*****************************************************************************
+ *
+ *****************************************************************************/
 int sout_InputSendBuffer( sout_packetizer_input_t *p_input,
-                          sout_buffer_t *p_buffer )
+                          block_t *p_buffer )
 {
     sout_instance_t     *p_sout = p_input->p_sout;
     int                 i_ret;
 
     if( p_input->p_fmt->i_codec == VLC_FOURCC( 'n', 'u', 'l', 'l' ) )
     {
-        sout_BufferDelete( p_input->p_sout, p_buffer );
+        block_Release( p_buffer );
         return VLC_SUCCESS;
     }
-
-    if( !p_buffer->i_dts )
+    if( p_buffer->i_dts <= 0 )
     {
         msg_Warn( p_sout, "trying to send non-dated packet to stream output!");
-        sout_BufferDelete( p_input->p_sout, p_buffer );
+        block_Release( p_buffer );
         return VLC_SUCCESS;
     }
 
@@ -280,9 +293,11 @@ sout_access_out_t *sout_AccessOutNew( sout_instance_t *p_sout,
     p_access->pf_seek    = NULL;
     p_access->pf_read    = NULL;
     p_access->pf_write   = NULL;
+    p_access->p_module   = NULL;
+    vlc_object_attach( p_access, p_sout );
 
     p_access->p_module   =
-        module_NeedStrict( p_access, "sout access", p_access->psz_access );
+        module_Need( p_access, "sout access", p_access->psz_access, VLC_TRUE );
 
     if( !p_access->p_module )
     {
@@ -299,6 +314,7 @@ sout_access_out_t *sout_AccessOutNew( sout_instance_t *p_sout,
  *****************************************************************************/
 void sout_AccessOutDelete( sout_access_out_t *p_access )
 {
+    vlc_object_detach( p_access );
     if( p_access->p_module )
     {
         module_Unneed( p_access, p_access->p_module );
@@ -323,26 +339,25 @@ int sout_AccessOutSeek( sout_access_out_t *p_access, off_t i_pos )
 /*****************************************************************************
  * sout_AccessRead:
  *****************************************************************************/
-int sout_AccessOutRead( sout_access_out_t *p_access, sout_buffer_t *p_buffer )
+int sout_AccessOutRead( sout_access_out_t *p_access, block_t *p_buffer )
 {
-    return ( p_access->pf_read ?
-             p_access->pf_read( p_access, p_buffer ) : VLC_EGENERIC );
+    return( p_access->pf_read ?
+            p_access->pf_read( p_access, p_buffer ) : VLC_EGENERIC );
 }
 
 /*****************************************************************************
  * sout_AccessWrite:
  *****************************************************************************/
-int sout_AccessOutWrite( sout_access_out_t *p_access, sout_buffer_t *p_buffer )
+int sout_AccessOutWrite( sout_access_out_t *p_access, block_t *p_buffer )
 {
     return p_access->pf_write( p_access, p_buffer );
 }
 
 /*****************************************************************************
- * MuxNew: allocate a new mux
+ * sout_MuxNew: create a new mux
  *****************************************************************************/
-sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
-                                   char *psz_mux,
-                                   sout_access_out_t *p_access )
+sout_mux_t * sout_MuxNew( sout_instance_t *p_sout, char *psz_mux,
+                          sout_access_out_t *p_access )
 {
     sout_mux_t *p_mux;
     char       *psz_next;
@@ -362,7 +377,6 @@ sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
         free( psz_next );
     }
     p_mux->p_access     = p_access;
-    p_mux->i_preheader  = 0;
     p_mux->pf_capacity  = NULL;
     p_mux->pf_addstream = NULL;
     p_mux->pf_delstream = NULL;
@@ -371,9 +385,12 @@ sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
     p_mux->pp_inputs    = NULL;
 
     p_mux->p_sys        = NULL;
+    p_mux->p_module = NULL;
+
+    vlc_object_attach( p_mux, p_sout );
 
     p_mux->p_module     =
-        module_NeedStrict( p_mux, "sout mux", p_mux->psz_mux );
+        module_Need( p_mux, "sout mux", p_mux->psz_mux, VLC_TRUE );
 
     if( p_mux->p_module == NULL )
     {
@@ -428,8 +445,12 @@ sout_mux_t * sout_MuxNew         ( sout_instance_t *p_sout,
     return p_mux;
 }
 
+/*****************************************************************************
+ * sout_MuxDelete:
+ *****************************************************************************/
 void sout_MuxDelete( sout_mux_t *p_mux )
 {
+    vlc_object_detach( p_mux );
     if( p_mux->p_module )
     {
         module_Unneed( p_mux, p_mux->p_module );
@@ -441,6 +462,9 @@ void sout_MuxDelete( sout_mux_t *p_mux )
     vlc_object_destroy( p_mux );
 }
 
+/*****************************************************************************
+ * sout_MuxAddStream:
+ *****************************************************************************/
 sout_input_t *sout_MuxAddStream( sout_mux_t *p_mux, es_format_t *p_fmt )
 {
     sout_input_t *p_input;
@@ -463,7 +487,7 @@ sout_input_t *sout_MuxAddStream( sout_mux_t *p_mux, es_format_t *p_fmt )
     p_input = malloc( sizeof( sout_input_t ) );
     p_input->p_sout = p_mux->p_sout;
     p_input->p_fmt  = p_fmt;
-    p_input->p_fifo = sout_FifoCreate( p_mux->p_sout );
+    p_input->p_fifo = block_FifoNew( p_mux->p_sout );
     p_input->p_sys  = NULL;
 
     TAB_APPEND( p_mux->i_nb_inputs, p_mux->pp_inputs, p_input );
@@ -471,19 +495,29 @@ sout_input_t *sout_MuxAddStream( sout_mux_t *p_mux, es_format_t *p_fmt )
     {
             msg_Err( p_mux, "cannot add this stream" );
             TAB_REMOVE( p_mux->i_nb_inputs, p_mux->pp_inputs, p_input );
-            sout_FifoDestroy( p_mux->p_sout, p_input->p_fifo );
+            block_FifoRelease( p_input->p_fifo );
             free( p_input );
-            return( NULL );
+            return NULL;
     }
 
-    return( p_input );
+    return p_input;
 }
 
-void sout_MuxDeleteStream     ( sout_mux_t *p_mux,
-                                sout_input_t *p_input )
+/*****************************************************************************
+ * sout_MuxDeleteStream:
+ *****************************************************************************/
+void sout_MuxDeleteStream( sout_mux_t *p_mux, sout_input_t *p_input )
 {
     int i_index;
 
+    if( p_mux->b_waiting_stream && p_input->p_fifo->i_depth > 0 )
+    {
+        /* We stop waiting, and call the muxer for taking care of the data
+         * before we remove this es */
+        p_mux->b_waiting_stream = VLC_FALSE;
+        p_mux->pf_mux( p_mux );
+    }
+
     TAB_FIND( p_mux->i_nb_inputs, p_mux->pp_inputs, p_input, i_index );
     if( i_index >= 0 )
     {
@@ -500,16 +534,18 @@ void sout_MuxDeleteStream     ( sout_mux_t *p_mux,
             msg_Warn( p_mux, "no more input stream for this mux" );
         }
 
-        sout_FifoDestroy( p_mux->p_sout, p_input->p_fifo );
+        block_FifoRelease( p_input->p_fifo );
         free( p_input );
     }
 }
 
-void sout_MuxSendBuffer       ( sout_mux_t    *p_mux,
-                                sout_input_t  *p_input,
-                                sout_buffer_t *p_buffer )
+/*****************************************************************************
+ * sout_MuxSendBuffer:
+ *****************************************************************************/
+void sout_MuxSendBuffer( sout_mux_t *p_mux, sout_input_t *p_input,
+                         block_t *p_buffer )
 {
-    sout_FifoPut( p_input->p_fifo, p_buffer );
+    block_FifoPut( p_input->p_fifo, p_buffer );
 
     if( p_mux->b_waiting_stream )
     {
@@ -527,267 +563,10 @@ void sout_MuxSendBuffer       ( sout_mux_t    *p_mux,
     p_mux->pf_mux( p_mux );
 }
 
-
-
-sout_fifo_t *sout_FifoCreate( sout_instance_t *p_sout )
-{
-    sout_fifo_t *p_fifo;
-
-    if( !( p_fifo = malloc( sizeof( sout_fifo_t ) ) ) )
-    {
-        return( NULL );
-    }
-
-    vlc_mutex_init( p_sout, &p_fifo->lock );
-    vlc_cond_init ( p_sout, &p_fifo->wait );
-    p_fifo->i_depth = 0;
-    p_fifo->p_first = NULL;
-    p_fifo->pp_last = &p_fifo->p_first;
-
-    return( p_fifo );
-}
-
-void sout_FifoFree( sout_instance_t *p_sout, sout_fifo_t *p_fifo )
-{
-    sout_buffer_t *p_buffer;
-
-    vlc_mutex_lock( &p_fifo->lock );
-    p_buffer = p_fifo->p_first;
-    while( p_buffer )
-    {
-        sout_buffer_t *p_next;
-        p_next = p_buffer->p_next;
-        sout_BufferDelete( p_sout, p_buffer );
-        p_buffer = p_next;
-    }
-    vlc_mutex_unlock( &p_fifo->lock );
-
-    return;
-}
-void sout_FifoDestroy( sout_instance_t *p_sout, sout_fifo_t *p_fifo )
-{
-    sout_FifoFree( p_sout, p_fifo );
-    vlc_mutex_destroy( &p_fifo->lock );
-    vlc_cond_destroy ( &p_fifo->wait );
-
-    free( p_fifo );
-}
-
-void sout_FifoPut( sout_fifo_t *p_fifo, sout_buffer_t *p_buffer )
-{
-    vlc_mutex_lock( &p_fifo->lock );
-
-    do
-    {
-        *p_fifo->pp_last = p_buffer;
-        p_fifo->pp_last = &p_buffer->p_next;
-        p_fifo->i_depth++;
-
-        p_buffer = p_buffer->p_next;
-
-    } while( p_buffer );
-
-    /* warm there is data in this fifo */
-    vlc_cond_signal( &p_fifo->wait );
-    vlc_mutex_unlock( &p_fifo->lock );
-}
-
-sout_buffer_t *sout_FifoGet( sout_fifo_t *p_fifo )
-{
-    sout_buffer_t *p_buffer;
-
-    vlc_mutex_lock( &p_fifo->lock );
-
-    if( p_fifo->p_first == NULL )
-    {
-        vlc_cond_wait( &p_fifo->wait, &p_fifo->lock );
-    }
-
-    p_buffer = p_fifo->p_first;
-
-    p_fifo->p_first = p_buffer->p_next;
-    p_fifo->i_depth--;
-
-    if( p_fifo->p_first == NULL )
-    {
-        p_fifo->pp_last = &p_fifo->p_first;
-    }
-
-    vlc_mutex_unlock( &p_fifo->lock );
-
-    p_buffer->p_next = NULL;
-    return( p_buffer );
-}
-
-sout_buffer_t *sout_FifoShow( sout_fifo_t *p_fifo )
-{
-    sout_buffer_t *p_buffer;
-
-    vlc_mutex_lock( &p_fifo->lock );
-
-    if( p_fifo->p_first == NULL )
-    {
-        vlc_cond_wait( &p_fifo->wait, &p_fifo->lock );
-    }
-
-    p_buffer = p_fifo->p_first;
-
-    vlc_mutex_unlock( &p_fifo->lock );
-
-    return( p_buffer );
-}
-
-sout_buffer_t *sout_BufferNew( sout_instance_t *p_sout, size_t i_size )
-{
-    sout_buffer_t *p_buffer;
-    size_t        i_preheader, i_padding;
-
-    p_buffer = malloc( sizeof( sout_buffer_t ) );
-    i_preheader = p_sout->i_preheader;
-    i_padding = p_sout->i_padding;
-
-#ifdef DEBUG_BUFFER
-    msg_Dbg( p_sout, "allocating an new buffer, size:%d, preheader:%d, "
-             "padding:%d", (uint32_t)i_size, i_preheader, i_padding );
-#endif
-
-    if( i_size > 0 )
-    {
-        p_buffer->p_allocated_buffer =
-            malloc( i_size + i_preheader + i_padding );
-        p_buffer->p_buffer = p_buffer->p_allocated_buffer + i_preheader;
-
-        if( p_buffer->p_allocated_buffer && i_padding )
-            memset( p_buffer->p_allocated_buffer + i_size + i_preheader, 0,
-                    i_padding );
-    }
-    else
-    {
-        p_buffer->p_allocated_buffer = NULL;
-        p_buffer->p_buffer = NULL;
-    }
-    p_buffer->i_allocated_size = i_size + i_preheader + i_padding;
-    p_buffer->i_buffer_size = i_size;
-
-    p_buffer->i_size    = i_size;
-    p_buffer->i_length  = 0;
-    p_buffer->i_dts     = 0;
-    p_buffer->i_pts     = 0;
-    p_buffer->i_bitrate = 0;
-    p_buffer->i_flags   = 0x0000;
-    p_buffer->p_next = NULL;
-
-    return( p_buffer );
-}
-
-int sout_BufferRealloc( sout_instance_t *p_sout, sout_buffer_t *p_buffer,
-                        size_t i_size )
-{
-    size_t i_preheader, i_padding;
-
-    i_preheader = p_buffer->p_buffer - p_buffer->p_allocated_buffer;
-    i_padding =  p_buffer->i_allocated_size - p_buffer->i_buffer_size
-                 - i_preheader;
-
-#ifdef DEBUG_BUFFER
-    msg_Dbg( p_sout, "realloc buffer old size:%d new size:%d, preheader:%d, "
-             "padding:%d", (uint32_t)p_buffer->i_allocated_size,
-             (uint32_t)i_size, i_preheader, i_padding );
-#endif
-
-    if( !( p_buffer->p_allocated_buffer =
-           realloc( p_buffer->p_allocated_buffer,
-                    i_size + i_preheader + i_padding ) ) )
-    {
-        msg_Err( p_sout, "realloc failed" );
-        p_buffer->i_allocated_size = 0;
-        p_buffer->i_buffer_size = 0;
-        p_buffer->i_size = 0;
-        p_buffer->p_buffer = NULL;
-        return( -1 );
-    }
-    p_buffer->p_buffer = p_buffer->p_allocated_buffer + i_preheader;
-
-    p_buffer->i_allocated_size = i_size + i_preheader + i_padding;
-    p_buffer->i_buffer_size = i_size;
-
-    if( i_padding )
-        memset( p_buffer->p_allocated_buffer + i_size + i_preheader, 0,
-                i_padding );
-
-    return( 0 );
-}
-
-int sout_BufferReallocFromPreHeader( sout_instance_t *p_sout,
-                                     sout_buffer_t *p_buffer, size_t i_size )
-{
-    size_t i_preheader;
-
-    i_preheader = p_buffer->p_buffer - p_buffer->p_allocated_buffer;
-
-    if( i_preheader < i_size )
-    {
-        return( -1 );
-    }
-
-    p_buffer->p_buffer -= i_size;
-    p_buffer->i_size += i_size;
-    p_buffer->i_buffer_size += i_size;
-
-    return( 0 );
-}
-
-int sout_BufferDelete( sout_instance_t *p_sout, sout_buffer_t *p_buffer )
-{
-#ifdef DEBUG_BUFFER
-    msg_Dbg( p_sout, "freeing buffer, size:%d", p_buffer->i_size );
-#endif
-    if( p_buffer->p_allocated_buffer )
-    {
-        free( p_buffer->p_allocated_buffer );
-    }
-    free( p_buffer );
-    return( 0 );
-}
-
-sout_buffer_t *sout_BufferDuplicate( sout_instance_t *p_sout,
-                                     sout_buffer_t *p_buffer )
-{
-    sout_buffer_t *p_dup;
-
-    p_dup = sout_BufferNew( p_sout, p_buffer->i_size );
-
-    p_dup->i_bitrate= p_buffer->i_bitrate;
-    p_dup->i_dts    = p_buffer->i_dts;
-    p_dup->i_pts    = p_buffer->i_pts;
-    p_dup->i_length = p_buffer->i_length;
-    p_dup->i_flags  = p_buffer->i_flags;
-    p_sout->p_vlc->pf_memcpy( p_dup->p_buffer, p_buffer->p_buffer, p_buffer->i_size );
-
-    return( p_dup );
-}
-
-void sout_BufferChain( sout_buffer_t **pp_chain,
-                       sout_buffer_t *p_buffer )
-{
-    if( *pp_chain == NULL )
-    {
-        *pp_chain = p_buffer;
-    }
-    else if( p_buffer != NULL )
-    {
-        sout_buffer_t *p = *pp_chain;
-
-        while( p->p_next )
-        {
-            p = p->p_next;
-        }
-
-        p->p_next = p_buffer;
-    }
-}
-
-static int  mrl_Parse( mrl_t *p_mrl, char *psz_mrl )
+/*****************************************************************************
+ *
+ *****************************************************************************/
+static int mrl_Parse( mrl_t *p_mrl, char *psz_mrl )
 {
     char * psz_dup = strdup( psz_mrl );
     char * psz_parser = psz_dup;
@@ -925,80 +704,48 @@ static void mrl_Clean( mrl_t *p_mrl )
     module{option=*:option=*}[:module{option=*:...}]
  */
 
-static char *_strndup( char *str, int i_len )
-{
-    char *p;
-
-    p = malloc( i_len + 1 );
-    strncpy( p, str, i_len );
-    p[i_len] = '\0';
-
-    return( p );
-}
-
 /*
  * parse module{options=str, option="str "}:
  *  return a pointer on the rest
  *  XXX: psz_chain is modified
  */
 #define SKIPSPACE( p ) { while( *p && ( *p == ' ' || *p == '\t' ) ) p++; }
+#define SKIPTRAILINGSPACE( p, e ) \
+    { while( e > p && ( *(e-1) == ' ' || *(e-1) == '\t' ) ) e--; }
+
 /* go accross " " and { } */
 static char *_get_chain_end( char *str )
 {
-    char *p = str;
+    char c, *p = str;
 
     SKIPSPACE( p );
 
     for( ;; )
     {
-        if( *p == '{' || *p == '"' || *p == '\'')
-        {
-            char c;
+        if( !*p || *p == ',' || *p == '}' ) return p;
 
-            if( *p == '{' )
-            {
-                c = '}';
-            }
-            else
-            {
-                c = *p;
-            }
-            p++;
-
-            for( ;; )
-            {
-                if( *p == '\0' )
-                {
-                    return p;
-                }
-
-                if( *p == c )
-                {
-                    p++;
-                    return p;
-                }
-                else if( *p == '{' && c == '}' )
-                {
-                    p = _get_chain_end( p );
-                }
-                else
-                {
-                    p++;
-                }
-            }
-        }
-        else if( *p == '\0' || *p == ',' || *p == '}' || *p == ' ' || *p == '\t' )
-        {
-            return p;
-        }
-        else
+        if( *p != '{' && *p != '"' && *p != '\'' )
         {
             p++;
+            continue;
+        }
+
+        if( *p == '{' ) c = '}';
+        else c = *p;
+        p++;
+
+        for( ;; )
+        {
+            if( !*p ) return p;
+
+            if( *p == c ) return ++p;
+            else if( *p == '{' && c == '}' ) p = _get_chain_end( p );
+            else p++;
         }
     }
 }
 
-char * sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
+char *sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
 {
     sout_cfg_t *p_cfg = NULL;
     char       *p = psz_chain;
@@ -1006,26 +753,15 @@ char * sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
     *ppsz_name = NULL;
     *pp_cfg    = NULL;
 
-    if( p == NULL )
-    {
-        return NULL;
-    }
+    if( !p ) return NULL;
 
     SKIPSPACE( p );
 
-    while( *p && *p != '{' && *p != ':' && *p != ' ' && *p != '\t' )
-    {
-        p++;
-    }
+    while( *p && *p != '{' && *p != ':' && *p != ' ' && *p != '\t' ) p++;
 
-    if( p == psz_chain )
-    {
-        return NULL;
-    }
+    if( p == psz_chain ) return NULL;
 
-    *ppsz_name = _strndup( psz_chain, p - psz_chain );
-
-    /* fprintf( stderr, "name=%s - rest=%s\n", *ppsz_name, p ); */
+    *ppsz_name = strndup( psz_chain, p - psz_chain );
 
     SKIPSPACE( p );
 
@@ -1043,10 +779,8 @@ char * sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
 
             psz_name = p;
 
-            while( *p && *p != '=' && *p != ',' && *p != '}' && *p != ' ' && *p != '\t' )
-            {
-                p++;
-            }
+            while( *p && *p != '=' && *p != ',' && *p != '{' && *p != '}' &&
+                   *p != ' ' && *p != '\t' ) p++;
 
             /* fprintf( stderr, "name=%s - rest=%s\n", psz_name, p ); */
             if( p == psz_name )
@@ -1055,48 +789,17 @@ char * sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
                 break;
             }
 
-            cfg.psz_name = _strndup( psz_name, p - psz_name );
+            cfg.psz_name = strndup( psz_name, p - psz_name );
 
             SKIPSPACE( p );
 
-            if( *p == '=' )
+            if( *p == '=' || *p == '{' )
             {
                 char *end;
+                vlc_bool_t b_keep_brackets = (*p == '{');
 
-                p++;
-#if 0
-                SKIPSPACE( p );
+                if( *p == '=' ) p++;
 
-                if( *p == '"' )
-                {
-                    char *end;
-
-                    p++;
-                    end = strchr( p, '"' );
-
-                    if( end )
-                    {
-/*                        fprintf( stderr, "##%s -- %s\n", p, end ); */
-                        cfg.psz_value = _strndup( p, end - p );
-                        p = end + 1;
-                    }
-                    else
-                    {
-                        cfg.psz_value = strdup( p );
-                        p += strlen( p );
-                    }
-
-                }
-                else
-                {
-                    psz_value = p;
-                    while( *p && *p != ',' && *p != '}' && *p != ' ' && *p != '\t' )
-                    {
-                        p++;
-                    }
-                    cfg.psz_value = _strndup( psz_value, p - psz_value );
-                }
-#endif
                 end = _get_chain_end( p );
                 if( end <= p )
                 {
@@ -1104,18 +807,34 @@ char * sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
                 }
                 else
                 {
-                    if( *p == '\'' || *p =='"' || *p == '{' )
+                    /* Skip heading and trailing spaces.
+                     * This ain't necessary but will avoid simple
+                     * user mistakes. */
+                    SKIPSPACE( p );
+                }
+
+                if( end <= p )
+                {
+                    cfg.psz_value = NULL;
+                }
+                else
+                {
+                    if( *p == '\'' || *p == '"' ||
+                        ( !b_keep_brackets && *p == '{' ) )
                     {
                         p++;
-                        end--;
-                    }
-                    if( end <= p )
-                    {
-                        cfg.psz_value = NULL;
+
+                        if( *(end-1) != '\'' && *(end-1) == '"' )
+                            SKIPTRAILINGSPACE( p, end );
+
+                        if( end - 1 <= p ) cfg.psz_value = NULL;
+                        else cfg.psz_value = strndup( p, end -1 - p );
                     }
                     else
                     {
-                        cfg.psz_value = _strndup( p, end - p );
+                        SKIPTRAILINGSPACE( p, end );
+                        if( end <= p ) cfg.psz_value = NULL;
+                        else cfg.psz_value = strndup( p, end - p );
                     }
                 }
 
@@ -1143,26 +862,19 @@ char * sout_cfg_parser( char **ppsz_name, sout_cfg_t **pp_cfg, char *psz_chain )
                 *pp_cfg = p_cfg;
             }
 
-            if( *p == ',' )
-            {
-                p++;
-            }
+            if( *p == ',' ) p++;
 
             if( *p == '}' )
             {
                 p++;
-
                 break;
             }
         }
     }
 
-    if( *p == ':' )
-    {
-        return( strdup( p + 1 ) );
-    }
+    if( *p == ':' ) return( strdup( p + 1 ) );
 
-    return( NULL );
+    return NULL;
 }
 
 static void sout_cfg_free( sout_cfg_t *p_cfg )
@@ -1181,12 +893,110 @@ static void sout_cfg_free( sout_cfg_t *p_cfg )
     }
 }
 
+void __sout_ParseCfg( vlc_object_t *p_this, char *psz_prefix,
+                      const char **ppsz_options, sout_cfg_t *cfg )
+{
+    char *psz_name;
+    int  i_type;
+    int  i;
+
+    /* First, var_Create all variables */
+    for( i = 0; ppsz_options[i] != NULL; i++ )
+    {
+        asprintf( &psz_name, "%s%s", psz_prefix, ppsz_options[i] );
+
+        i_type = config_GetType( p_this, psz_name );
+
+        var_Create( p_this, psz_name, i_type | VLC_VAR_DOINHERIT );
+        free( psz_name );
+    }
+
+    /* Now parse options and set value */
+    if( psz_prefix == NULL ) psz_prefix = "";
+
+    while( cfg )
+    {
+        vlc_value_t val;
+        vlc_bool_t b_yes = VLC_TRUE;
+
+        if( cfg->psz_name == NULL || *cfg->psz_name == '\0' )
+        {
+            cfg = cfg->p_next;
+            continue;
+        }
+        for( i = 0; ppsz_options[i] != NULL; i++ )
+        {
+            if( !strcmp( ppsz_options[i], cfg->psz_name ) )
+            {
+                break;
+            }
+            if( ( !strncmp( cfg->psz_name, "no-", 3 ) &&
+                  !strcmp( ppsz_options[i], cfg->psz_name + 3 ) ) ||
+                ( !strncmp( cfg->psz_name, "no", 2 ) &&
+                  !strcmp( ppsz_options[i], cfg->psz_name + 2 ) ) )
+            {
+                b_yes = VLC_FALSE;
+                break;
+            }
+        }
+        if( ppsz_options[i] == NULL )
+        {
+            msg_Warn( p_this, "option %s is unknown", cfg->psz_name );
+            cfg = cfg->p_next;
+            continue;
+        }
+
+        /* create name */
+        asprintf( &psz_name, "%s%s", psz_prefix, ppsz_options[i] );
+
+        /* get the type of the variable */
+        i_type = config_GetType( p_this, psz_name );
+        if( !i_type )
+        {
+            msg_Warn( p_this, "unknown option %s (value=%s)",
+                      cfg->psz_name, cfg->psz_value );
+            goto next;
+        }
+        if( i_type != VLC_VAR_BOOL && cfg->psz_value == NULL )
+        {
+            msg_Warn( p_this, "missing value for option %s", cfg->psz_name );
+            goto next;
+        }
+
+        switch( i_type )
+        {
+            case VLC_VAR_BOOL:
+                val.b_bool = b_yes;
+                break;
+            case VLC_VAR_INTEGER:
+                val.i_int = strtol( cfg->psz_value ? cfg->psz_value : "0",
+                                    NULL, 0 );
+                break;
+            case VLC_VAR_FLOAT:
+                val.f_float = atof( cfg->psz_value ? cfg->psz_value : "0" );
+                break;
+            case VLC_VAR_STRING:
+                val.psz_string = cfg->psz_value;
+                break;
+            default:
+                msg_Warn( p_this, "unhandled config var type" );
+                memset( &val, 0, sizeof( vlc_value_t ) );
+                break;
+        }
+        var_Set( p_this, psz_name, val );
+        msg_Dbg( p_this, "set sout option: %s to %s", psz_name, cfg->psz_value );
+
+    next:
+        free( psz_name );
+        cfg = cfg->p_next;
+    }
+}
+
 
 /*
  * XXX name and p_cfg are used (-> do NOT free them)
  */
-sout_stream_t *sout_stream_new( sout_instance_t *p_sout,
-                                char *psz_chain )
+sout_stream_t *sout_stream_new( sout_instance_t *p_sout, char *psz_chain )
 {
     sout_stream_t *p_stream;
 
@@ -1212,8 +1022,10 @@ sout_stream_t *sout_stream_new( sout_instance_t *p_sout,
 
     msg_Dbg( p_sout, "stream=`%s'", p_stream->psz_name );
 
+    vlc_object_attach( p_stream, p_sout );
+
     p_stream->p_module =
-        module_NeedStrict( p_stream, "sout stream", p_stream->psz_name );
+        module_Need( p_stream, "sout stream", p_stream->psz_name, VLC_TRUE );
 
     if( !p_stream->p_module )
     {
@@ -1227,6 +1039,8 @@ sout_stream_t *sout_stream_new( sout_instance_t *p_sout,
 void sout_stream_delete( sout_stream_t *p_stream )
 {
     msg_Dbg( p_stream, "destroying chain... (name=%s)", p_stream->psz_name );
+
+    vlc_object_detach( p_stream );
     if( p_stream->p_module ) module_Unneed( p_stream, p_stream->p_module );
 
     FREE( p_stream->psz_name );
@@ -1251,7 +1065,8 @@ static char *_sout_stream_url_to_chain( vlc_object_t *p_this, char *psz_url )
 
     if( config_GetInt( p_this, "sout-display" ) )
     {
-        p += sprintf( p, "duplicate{dst=display,dst=std{mux=\"%s\",access=\"%s\",url=\"%s\"}}",
+        p += sprintf( p, "duplicate{dst=display,dst=std{mux=\"%s\","
+                      "access=\"%s\",url=\"%s\"}}",
                       mrl.psz_way, mrl.psz_access, mrl.psz_name );
     }
     else
@@ -1263,28 +1078,3 @@ static char *_sout_stream_url_to_chain( vlc_object_t *p_this, char *psz_url )
     mrl_Clean( &mrl );
     return( psz_chain );
 }
-
-/*****************************************************************************/
-static module_t *__module_NeedStrict( vlc_object_t *p_obj, const char *psz_capacity, const char *psz_name )
-{
-    module_t *p_module;
-
-    if( !psz_name || !*psz_name )
-    {
-        p_module = module_Need( p_obj, psz_capacity, psz_name );
-    }
-    else
-    {
-        char *psz_name_strict = malloc( strlen( psz_name ) + 6 );
-        strcpy( psz_name_strict, psz_name );
-        strcat( psz_name_strict, ",none" );
-
-        p_module = module_Need( p_obj, psz_capacity, psz_name_strict );
-
-        free( psz_name_strict );
-    }
-
-    return p_module;
-}
-
-
