@@ -5,7 +5,7 @@
  * thread, and destroy a previously oppened video output thread.
  *****************************************************************************
  * Copyright (C) 2000-2004 VideoLAN
- * $Id: video_output.c,v 1.245 2004/01/25 17:16:06 zorglub Exp $
+ * $Id: video_output.c 7694 2004-05-16 22:06:34Z gbazin $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -57,9 +57,9 @@ static int      BinaryLog         ( uint32_t );
 static void     MaskToShift       ( int *, int *, uint32_t );
 static void     InitWindowSize    ( vout_thread_t *, int *, int * );
 
+void vout_IntfInit( vout_thread_t * );
+
 /* Object variables callbacks */
-static int FullscreenCallback( vlc_object_t *, char const *,
-                               vlc_value_t, vlc_value_t, void * );
 static int DeinterlaceCallback( vlc_object_t *, char const *,
                                 vlc_value_t, vlc_value_t, void * );
 static int FilterCallback( vlc_object_t *, char const *,
@@ -290,6 +290,8 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     p_vout->render_time  = 10;
     p_vout->c_fps_samples = 0;
     p_vout->b_filter_change = 0;
+    p_vout->pf_control = 0;
+    p_vout->p_parent_intf = 0;
 
     /* Mouse coordinates */
     var_Create( p_vout, "mouse-x", VLC_VAR_INTEGER );
@@ -310,12 +312,8 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     /* Attach the new object now so we can use var inheritance below */
     vlc_object_attach( p_vout, p_parent );
 
-    /* Create a few object variables we'll need later on */
-    var_Create( p_vout, "aspect-ratio", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
-    var_Create( p_vout, "width", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
-    var_Create( p_vout, "height", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
-    var_Create( p_vout, "zoom", VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
-    var_Create( p_vout, "align", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    /* Take care of some "interface/control" related initialisations */
+    vout_IntfInit( p_vout );
 
     p_vout->b_override_aspect = VLC_FALSE;
 
@@ -403,10 +401,8 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
 
     /* Create the vout thread */
     p_vout->p_module = module_Need( p_vout,
-                           ( p_vout->psz_filter_chain &&
-                               *p_vout->psz_filter_chain ) ?
-                           "video filter" : "video output",
-                           psz_plugin );
+        ( p_vout->psz_filter_chain && *p_vout->psz_filter_chain ) ?
+        "video filter" : "video output", psz_plugin, 0 );
 
     if( psz_plugin ) free( psz_plugin );
     if( p_vout->p_module == NULL )
@@ -416,8 +412,8 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
         return NULL;
     }
 
-    p_vout->p_text_renderer_module = module_Need( p_vout, "text renderer",
-                                                  NULL );
+    p_vout->p_text_renderer_module =
+        module_Need( p_vout, "text renderer", NULL, 0 );
     if( p_vout->p_text_renderer_module == NULL )
     {
         msg_Warn( p_vout, "no suitable text renderer module" );
@@ -425,17 +421,6 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     }
 
     /* Create a few object variables for interface interaction */
-    var_Create( p_vout, "fullscreen", VLC_VAR_BOOL );
-    text.psz_string = _("Fullscreen");
-    var_Change( p_vout, "fullscreen", VLC_VAR_SETTEXT, &text, NULL );
-    var_Change( p_vout, "fullscreen", VLC_VAR_INHERITVALUE, &val, NULL );
-    if( val.b_bool )
-    {
-        /* user requested fullscreen */
-        p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
-    }
-    var_AddCallback( p_vout, "fullscreen", FullscreenCallback, NULL );
-
     var_Create( p_vout, "deinterlace", VLC_VAR_STRING | VLC_VAR_HASCHOICE );
     text.psz_string = _("Deinterlace");
     var_Change( p_vout, "deinterlace", VLC_VAR_SETTEXT, &text, NULL );
@@ -488,6 +473,20 @@ vout_thread_t * __vout_Create( vlc_object_t *p_parent,
     {
         msg_Err( p_vout, "out of memory" );
         module_Unneed( p_vout, p_vout->p_module );
+        vlc_object_detach( p_vout );
+        vlc_object_destroy( p_vout );
+        return NULL;
+    }
+
+    if( p_vout->b_error )
+    {
+        msg_Err( p_vout, "video output creation failed" );
+
+        /* Make sure the thread is destroyed */
+        p_vout->b_die = VLC_TRUE;
+        vlc_thread_join( p_vout );
+
+        vlc_object_detach( p_vout );
         vlc_object_destroy( p_vout );
         return NULL;
     }
@@ -669,7 +668,7 @@ static int InitThread( vout_thread_t *p_vout )
         p_vout->b_direct = 0;
 
         /* Choose the best module */
-        p_vout->chroma.p_module = module_Need( p_vout, "chroma", NULL );
+        p_vout->chroma.p_module = module_Need( p_vout, "chroma", NULL, 0 );
 
         if( p_vout->chroma.p_module == NULL )
         {
@@ -1323,31 +1322,6 @@ int vout_VarCallback( vlc_object_t * p_this, const char * psz_variable,
  * object variables callbacks: a bunch of object variables are used by the
  * interfaces to interact with the vout.
  *****************************************************************************/
-static int FullscreenCallback( vlc_object_t *p_this, char const *psz_cmd,
-                       vlc_value_t oldval, vlc_value_t newval, void *p_data )
-{
-    vout_thread_t *p_vout = (vout_thread_t *)p_this;
-    input_thread_t *p_input;
-    vlc_value_t val;
-
-    p_vout->i_changes |= VOUT_FULLSCREEN_CHANGE;
-
-    p_input = (input_thread_t *)vlc_object_find( p_this, VLC_OBJECT_INPUT,
-                                                 FIND_PARENT );
-    if( p_input )
-    {
-        /* Modify input as well because the vout might have to be restarted */
-        var_Create( p_input, "fullscreen", VLC_VAR_BOOL );
-        var_Set( p_input, "fullscreen", newval );
-
-        vlc_object_release( p_input );
-    }
-
-    val.b_bool = VLC_TRUE;
-    var_Set( p_vout, "intf-change", val );
-    return VLC_SUCCESS;
-}
-
 static int DeinterlaceCallback( vlc_object_t *p_this, char const *psz_cmd,
                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
